Package: NormalyzerDE
Title: Evaluation of normalization methods and calculation of
        differential expression analysis statistics
Version: 1.30.0
Author: Jakob Willforss
Authors@R: c(
    person("Jakob", "Willforss", email="jakob.willforss@hotmail.com", role=c("aut", "cre")),
    person("Aakash", "Chawade", role="aut"),
    person("Fredrik", "Levander", email="fredrik.levander@immun.lth.se", role=c("aut", "ths")),
    person("Måns", "Zamore", email="mans.bioc@zamore.se", role=c("aut")))
Description: NormalyzerDE provides screening of normalization methods for 
    LC-MS based expression data. It calculates a range of normalized matrices 
    using both existing approaches and a novel time-segmented approach, 
    calculates performance measures and generates an evaluation report. 
    Furthermore, it provides an easy utility for Limma- or ANOVA- based 
    differential expression analysis.
Imports: vsn, preprocessCore, limma, MASS, ape, car, ggplot2, methods,
        utils, stats, SummarizedExperiment, matrixStats, ggforce
Suggests: knitr, testthat, rmarkdown, roxygen2, hexbin, BiocStyle
VignetteBuilder: knitr
biocViews: Normalization, MultipleComparison, Visualization, Bayesian,
        Proteomics, Metabolomics, DifferentialExpression
License: Artistic-2.0
Encoding: UTF-8
RoxygenNote: 7.3.3
URL: https://computationalproteomics.github.io/NormalyzerDE/,
        https://github.com/ComputationalProteomics/NormalyzerDE
Depends: R (>= 4.1.0)
git_url: https://git.bioconductor.org/packages/NormalyzerDE
git_branch: RELEASE_3_23
git_last_commit: a701c99
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:37:52 UTC; biocbuild
Maintainer: Jakob Willforss <jakob.willforss@hotmail.com>

# Generated by roxygen2: do not edit by hand

export(NormalyzerEvaluationResults)
export(NormalyzerResults)
export(NormalyzerStatistics)
export(analyzeNormalizations)
export(calculateContrasts)
export(generateAnnotatedMatrix)
export(generatePlots)
export(generateStatsReport)
export(getRTNormalizedMatrix)
export(getSmoothedRTNormalizedMatrix)
export(getVerifiedNormalyzerObject)
export(globalIntensityNormalization)
export(meanNormalization)
export(medianNormalization)
export(normMethods)
export(normalyzer)
export(normalyzerDE)
export(performCyclicLoessNormalization)
export(performGlobalRLRNormalization)
export(performQuantileNormalization)
export(performSMADNormalization)
export(performVSNNormalization)
export(reduceTechnicalReplicates)
export(setupJobDir)
export(setupRawContrastObject)
export(setupRawDataObject)
export(setupTestData)
export(writeNormalizedDatasets)
import(MASS)
import(limma)
import(methods)
import(preprocessCore)
importClassesFrom(SummarizedExperiment,SummarizedExperiment)

<!-- badges: start -->
[![Bioc release status](https://bioconductor.org/shields/build/release/bioc/NormalyzerDE.svg)](https://bioconductor.org/checkResults/release/bioc-LATEST/NormalyzerDE/)
[![Bioc devel status](https://bioconductor.org/shields/build/devel/bioc/NormalyzerDE.svg)](https://bioconductor.org/checkResults/devel/bioc-LATEST/NormalyzerDE/)
[![R-CMD-check-bioc](https://github.com/ComputationalProteomics/NormalyzerDE/actions/workflows/check-bioc.yml/badge.svg?branch=devel)](https://github.com/ComputationalProteomics/NormalyzerDE/actions/workflows/check-bioc.yml?query=branch%3Adevel)
[![pkgdown](https://github.com/ComputationalProteomics/NormalyzerDE/actions/workflows/pkgdown.yaml/badge.svg?branch=devel)](https://github.com/ComputationalProteomics/NormalyzerDE/actions/workflows/pkgdown.yaml?query=branch%3Adevel)
[![Bioc downloads rank](https://bioconductor.org/shields/downloads/release/NormalyzerDE.svg)](https://bioconductor.org/packages/stats/bioc/NormalyzerDE/)
[![Bioc support](https://bioconductor.org/shields/posts/NormalyzerDE.svg)](https://support.bioconductor.org/tag/NormalyzerDE)
[![Bioc history](https://bioconductor.org/shields/years-in-bioc/NormalyzerDE.svg)](https://bioconductor.org/packages/release/bioc/html/NormalyzerDE.html#since)
[![Bioc last commit](https://bioconductor.org/shields/lastcommit/devel/bioc/NormalyzerDE.svg)](https://bioconductor.org/checkResults/devel/bioc-LATEST/NormalyzerDE/)
[![Bioc dependencies](https://bioconductor.org/shields/dependencies/release/NormalyzerDE.svg)](https://bioconductor.org/packages/release/bioc/html/NormalyzerDE.html)
<!-- badges: end -->

# About

An online server running NormalyzerDE can be accessed at the following link:

https://quantitativeproteomics.org/normalyzerde

Alternatively:

https://normalyzerde.serve.scilifelab.se

NormalyzerDE is a software designed to ease the process of selecting an optimal normalization approach for your dataset and to perform subsequent differential expression analysis.

NormalyzerDE includes several normalization approaches, a empirical Bayes-based statistical approach implemented as part of Limma and a newly implemented retention-time segmented normalization approach inspired by previously outlined approaches. The emprical-based based statistics has been shown to increase sensitivity over ANOVA when detecting differentially expressed features.

# Cite NormalyzerDE

NormalyzerDE is published [here](https://pubs.acs.org/doi/10.1021/acs.jproteome.8b00523)

Willforss, J., Chawade, A., Levander, F. 
NormalyzerDE: Online tool for improved normalization of omics expression data and high-sensitivity differential expression analysis. *Journal of Proteome Research* **2018**, 10.1021/acs.jproteome.8b00523.

# Installation

NormalyzerDE can be installed from [Bioconductor](https://www.bioconductor.org/packages/release/bioc/html/NormalyzerDE.html), or directly from GitHub:

```
install.packages("devtools")
devtools::install_github("ComputationalProteomics/NormalyzerDE")
```

# Running NormalyzerDE - Minimal example

```
library(NormalyzerDE)
```

Generate normalizations and normalization performance report.

```
normalyzer(jobName="rscript_norm", designPath="test_design.tsv", dataPath="test_data.tsv")
```

Calculate differential expression between groups 1-2 and 1-3 (defined in the design matrix).

```
normalyzerDE(jobName="rscript_de", designPath="test_design.tsv", dataPath="test_data.tsv", comparisons=c("1-2", "1-3"))
```

For more comprehensive documentation, check the [Vignette](https://bioconductor.org/packages/devel/bioc/vignettes/NormalyzerDE/inst/doc/vignette.pdf) at NormalyzerDE's [Bioconductor page](https://bioconductor.org/packages/devel/bioc/html/NormalyzerDE.html). More information about required input formats is available [here](https://quantitativeproteomics.org/normalyzerde/help).

# Executing from command line

If you want to run NormalyzerDE directly from the command line this is possible by executing it through the `Rscript` command.

```
Rscript -e 'NormalyzerDE::normalyzer(jobName="rscript_norm", designPath="test_design.tsv", dataPath="test_data.tsv")'
Rscript -e 'NormalyzerDE::normalyzerDE(jobName="rscript_de", designPath="test_design.tsv", dataPath="test_data.tsv", comparisons=c("1-2", "1-3"))'
```

# References

(1) Bolstad, B. preprocessCore: A collection of pre-processing functions. **2018**; https://github.com/bmbolstad/preprocessCore.

(2) Gentleman, R. C. et al. Bioconductor: open software development for computational biology and bioinformatics. *Genome Biol.* **2004**, 5, R80.

(3) Huber, W.; von Heydebreck, A.; Sultmann, H.; Poustka, A.; Vingron, M. Variance stabilization
applied to microarray data calibration and to the quantification of differential
expression. *Bioinformatics* **2002**, 18, S96–S104.

(4) Kammers, K.; Cole, R. N.; Tiengwe, C.; Ruczinski, I. Detecting significant changes in protein abundance. *EuPA Open Proteom.* **2015**, 7, 11-19.

(5) Lyutvinskiy, Y.; Yang, H.; Rutishauser, D.; Zubarev, R. A. In Silico Instrumental Response Correction Improves Precision of Label-free Proteomics and Accuracy of Proteomics-based Predictive Models. *Mol. Cell Proteomics* **2013**, 12, 2324–2331.

(6) Ritchie, M. E.; Phipson, B.; Wu, D.; Hu, Y.; Law, C. W.; Shi, W.; Smyth, G. K. limma powers differential expression analyses for RNA-sequencing and microarray studies. *Nucleic Acids Res.* **2015**, 43, e47.

(7) van Ooijen, M. P.; Jong, V. L.; Eijkemans, M. J.; Heck, A. J.; Andeweg, A. C.; Binai, N. A.; van den Ham, H.-J. Identification of differentially expressed peptides in high-throughput proteomics data. *Brief. Bioinform.* **2017**, 1–11.

(8) Wolfgang, H. et al. Orchestrating high-throughput genomic analysis with Bioconductor. *Nat. Methods* **2015**, 12, 115–121.

# Code organization

NormalyzerDE consists of a number of scripts and classes. They are focused around
two separate workflows. One is for normalizing and evaluating the normalizations. The
second is for performing differential expression analysis. Classes are contained in scripts with the same name.

![NormalyzerDE schematics](vignettes/180813_normalyzerde_schematics.png)

The standard workflow for the normalization is the following:

* The `normalyzer` function in the `NormalyzerDE.R` script is called, starting the process.
* If applicable (that is, input is in Proteois or MaxQuant format), the dataset is preprocessed into the standard format using code in `preparsers.R`.
* The input is verified to capture standard errors early on using code in `inputVerification.R`. This results in an instance of the `NormalyzerDataset` class.
* The data is normalized using several normalization methods present in `normMethods.R`. This yields an instance of `NormalyzerResults` which links to the original `NormalyzerDataset` instance and also contains all the resulting normalized datasets.
* If specified (and if a column with retention time values is present) retention-time segmented approaches are performed by applying normalizations from `normMethods.R` over retention time using functions present in `higherOrderNormMethods.R`.
* The results are analyzed using functions present in `analyzeResults.R`. This yields an instance of `NormalyzerEvaluationResults` containing the evaluation results. This instance is attached to the `NormalyzerResults` object.
* The final results are sent to `outputUtils.R` where the normalizations are written to an output directory, and to `generatePlots.R` which contains visualizations for the performance measures. It also uses code in `printMeta.R` and `printPlots.R` to output the results in a desired format.

When a normalized matrix is selected the analysis proceeds to the statistical analysis.

* The `normalyzerde` function in the `NormalyzerDE.R` script is called starting the differential expression analysis pipeline.
* An instance of `NormalyzerStatistics` is prepared containing the input data.
* Code in the `calculateStatistics.R` script is used to calculate the statistical contrasts. The results are attached to the `NormalyzerStatistics` object.
* The resulting statistics are used to generate a report and an annotated output matrix where key statistical measures are attached to the original matrix.

#' Representation of evaluation results by calculating performance measures
#' for an an NormalyzerResults instance
#' 
#' Contains the resulting information from the processing which subsequently
#' can be used to generate the quality assessment report.
#' 
#' @slot avgcvmem Average coefficient of variance per method
#' @slot avgcvmempdiff Percentage difference of mean coefficient of variance
#'  compared to log2-transformed data
#' @slot featureCVPerMethod CV calculated per feature and normalization method.  
#' @slot avgmadmem Average median absolute deviation 
#' @slot avgmadmempdiff Percentage difference of median absolute deviation
#'  compared to log2-transformed data
#' @slot avgvarmem Average variance per method
#' @slot avgvarmempdiff Percentage difference of mean variance compared
#'  to log2-transformed data
#' @slot lowVarFeaturesCVs List of 5% least variable entries based on ANOVA
#'  for log2-transformed data
#' @slot lowVarFeaturesCVsPercDiff Coefficient of variance for least variable
#'  entries
#' @slot anovaP ANOVA calculated p-values
#' @slot repCorPear Within group Pearson correlations
#' @slot repCorSpear Within group Spearman correlations
NormalyzerEvaluationResults <- setClass("NormalyzerEvaluationResults",
                                           slots=c(
                                               avgcvmem = "matrix",
                                               avgcvmempdiff = "numeric",
                                               featureCVPerMethod = "matrix",
                                               
                                               avgmadmem = "matrix",
                                               avgmadmempdiff = "numeric",
                                               
                                               avgvarmem = "matrix",
                                               avgvarmempdiff = "numeric",
                                               
                                               lowVarFeaturesCVs = "numeric",
                                               lowVarFeaturesCVsPercDiff = "numeric",
                                               
                                               anovaP = "matrix",
                                               
                                               repCorPear = "matrix",
                                               repCorSpear = "matrix"
                                           ))

#' Constructor for NormalyzerEvaluationResults
#' 
#' @param nr NormalyzerResults object
#' @param categoricalAnova Should ANOVA be categorical or not
#' @return nds Generated NormalyzerEvaluationResults instance
#' @export
#' @examples
#' data(example_summarized_experiment)
#' normObj <- getVerifiedNormalyzerObject("job_name", example_summarized_experiment)
#' normResults <- normMethods(normObj)
#' normEval <- NormalyzerEvaluationResults(normResults)
NormalyzerEvaluationResults <- function (nr, categoricalAnova=TRUE) {

               nds <- nds(nr)
               sampleReplicateGroups <- sampleReplicateGroups(nds) 
               methodList <- normalizations(nr)
               sampleGroupsWithReplicates <- samplesGroupsWithReplicates(nds)
               singleReplicateRun <- singleReplicateRun(nds)
               tinyRun <- isTinyRun(nds)
               
               # Calculate CV related measures               
               avgCVPerNormAndReplicates <- calculateReplicateCV(methodList, sampleReplicateGroups)
               avgcvmem <- avgCVPerNormAndReplicates
               featureCVPerMethod <- calculateFeatureCV(methodList)
               
               avgcvmempdiff <- calculatePercentageAvgDiffInMat(avgCVPerNormAndReplicates)
               
               if (!singleReplicateRun && !tinyRun) {

                   # MAD
                   avgmadmem <- calculateAvgMadMem(methodList, sampleReplicateGroups)
                   avgmadmempdiff<- calculatePercentageAvgDiffInMat(avgmadmem)
                   
                   # Variance measures
                   avgVarianceMat <- calculateAvgReplicateVariation(methodList, sampleReplicateGroups)
                   avgvarmem <- avgVarianceMat
                   avgvarmempdiff <- calculatePercentageAvgDiffInMat(avgVarianceMat)
                   
                   # Significance measures
                   anovaPValsWithNAMat <- calculateANOVAPValues(methodList, sampleReplicateGroups, categoricalANOVA=categoricalAnova)
                   validPValuesContrast <- !is.na(anovaPValsWithNAMat[, 1])
                   
                   log2AnovaFDR <- rep(NA, length(anovaPValsWithNAMat[, 1]))
                   log2AnovaFDR[validPValuesContrast] <- stats::p.adjust(
                       anovaPValsWithNAMat[, 1][validPValuesContrast],
                       method="BH")
                   
                   lowVarFeaturesCVs <- findLowlyVariableFeaturesCVs(log2AnovaFDR, methodList)
                   lowVarFeaturesCVsPercDiff <- vapply(
                       seq_along(lowVarFeaturesCVs),
                       function(sampleIndex) {
                           (lowVarFeaturesCVs[sampleIndex] * 100) / lowVarFeaturesCVs[1]
                       },
                       0
                   )
                   
                   anovaP <- anovaPValsWithNAMat
                   lowVarFeaturesCVs <- lowVarFeaturesCVs
                   lowVarFeaturesCVsPercDiff <- lowVarFeaturesCVsPercDiff
               }

               # Correlation measures
               repCorPear <- calculateSummarizedCorrelationVector(
                   methodList,
                   sampleReplicateGroups,
                   sampleGroupsWithReplicates, 
                   "pearson"
               )
               
               repCorSpear <- calculateSummarizedCorrelationVector(
                   methodList,
                   sampleReplicateGroups,
                   sampleGroupsWithReplicates,
                   "spearman"
               )
               
               if (!singleReplicateRun && !tinyRun) {
                   object <- new(
                       "NormalyzerEvaluationResults",
                       avgcvmem=avgcvmem,
                       avgcvmempdiff=avgcvmempdiff,
                       featureCVPerMethod=featureCVPerMethod,
                       avgmadmem=avgmadmem,
                       avgmadmempdiff=avgmadmempdiff,
                       avgvarmem=avgvarmem,
                       avgvarmempdiff=avgvarmempdiff,
                       lowVarFeaturesCVs=lowVarFeaturesCVs,
                       lowVarFeaturesCVsPercDiff=lowVarFeaturesCVsPercDiff,
                       anovaP=anovaP,
                       repCorPear=repCorPear,
                       repCorSpear=repCorSpear
                   )
               }
               else {
                   object <- new(
                       "NormalyzerEvaluationResults",
                       avgcvmem=avgcvmem,
                       avgcvmempdiff=avgcvmempdiff,
                       featureCVPerMethod=featureCVPerMethod,
                       repCorPear=repCorPear,
                       repCorSpear=repCorSpear
                   )
               }
               
               return (object)
           }

setGeneric("avgcvmem", function(object) { standardGeneric("avgcvmem") })
setMethod("avgcvmem", signature(object="NormalyzerEvaluationResults"),
          function(object) { slot(object, "avgcvmem") })

setGeneric("avgcvmempdiff", function(object) { standardGeneric("avgcvmempdiff") })
setMethod("avgcvmempdiff", signature(object="NormalyzerEvaluationResults"),
          function(object) { slot(object, "avgcvmempdiff") })

setGeneric("featureCVPerMethod", function(object) { standardGeneric("featureCVPerMethod") })
setMethod("featureCVPerMethod", signature(object="NormalyzerEvaluationResults"),
          function(object) { slot(object, "featureCVPerMethod") })

setGeneric("avgmadmem", function(object) { standardGeneric("avgmadmem") })
setMethod("avgmadmem", signature(object="NormalyzerEvaluationResults"),
          function(object) { slot(object, "avgmadmem") })

setGeneric("avgmadmempdiff", function(object) { standardGeneric("avgmadmempdiff") })
setMethod("avgmadmempdiff", signature(object="NormalyzerEvaluationResults"),
          function(object) { slot(object, "avgmadmempdiff") })

setGeneric("avgvarmem", function(object) { standardGeneric("avgvarmem") })
setMethod("avgvarmem", signature(object="NormalyzerEvaluationResults"),
          function(object) { slot(object, "avgvarmem") })

setGeneric("avgvarmempdiff", function(object) { standardGeneric("avgvarmempdiff") })
setMethod("avgvarmempdiff", signature(object="NormalyzerEvaluationResults"),
          function(object) { slot(object, "avgvarmempdiff") })

setGeneric("lowVarFeaturesCVs", function(object) { standardGeneric("lowVarFeaturesCVs") })
setMethod("lowVarFeaturesCVs", signature(object="NormalyzerEvaluationResults"),
          function(object) { slot(object, "lowVarFeaturesCVs") })

setGeneric("lowVarFeaturesCVsPercDiff", function(object) { standardGeneric("lowVarFeaturesCVsPercDiff") })
setMethod("lowVarFeaturesCVsPercDiff", signature(object="NormalyzerEvaluationResults"),
          function(object) { slot(object, "lowVarFeaturesCVsPercDiff") })

setGeneric("anovaP", function(object) { standardGeneric("anovaP") })
setMethod("anovaP", signature(object="NormalyzerEvaluationResults"),
          function(object) { slot(object, "anovaP") })

setGeneric("repCorPear", function(object) { standardGeneric("repCorPear") })
setMethod("repCorPear", signature(object="NormalyzerEvaluationResults"),
          function(object) { slot(object, "repCorPear") })

setGeneric("repCorSpear", function(object) { standardGeneric("repCorSpear") })
setMethod("repCorSpear", signature(object="NormalyzerEvaluationResults"),
          function(object) { slot(object, "repCorSpear") })

#' Class representing a dataset for statistical processing in NormalyzerDE
#' 
#' Is initialized with an annotation matrix, a data matrix and a design
#' data frame. This object can subsequently be processed to generate statistical
#' values and in turn used to write a full matrix with additional statistical
#' information as well as a graphical report of the comparisons.
#' 
#' @slot annotMat Matrix containing annotation information
#' @slot dataMat Matrix containing (normalized) expression data
#' @slot filteredDataMat Filtered matrix with low-count rows removed
#' @slot designDf Data frame containing design conditions
#' @slot filteringContrast Vector showing which entries are filtered 
#'   (due to low count)
#' @slot pairwiseCompsP List with P-values for pairwise comparisons
#' @slot pairwiseCompsFdr List with FDR-values for pairwise comparisons
#' @slot pairwiseCompsAve List with average expression values
#' @slot pairwiseCompsFold List with log2 fold-change values for pairwise 
#'   comparisons
#' @slot contrasts Spot for saving vector of last used contrasts
#' @slot condCol Column containing last used conditions
#' @slot batchCol Column containing last used batch conditions
NormalyzerStatistics <- setClass("NormalyzerStatistics",
                                 slots = c(
                                     annotMat = "matrix",
                                     dataMat = "matrix",
                                     designDf = "data.frame",

                                     pairwiseCompsP = "list",
                                     pairwiseCompsFdr = "list",
                                     pairwiseCompsAve = "list",
                                     pairwiseCompsFold = "list",
                                     pairwiseCompsSig = "list",
                                     
                                     comparisons = "character",
                                     condCol = "character",
                                     batchCol = "numeric"
                                 ))

#' Constructor for NormalyzerStatistics
#' 
#' @param experimentObj Instance of SummarizedExperiment containing matrix
#'   and design information as column data
#' @param logTrans Whether the input data should be log transformed
#' @return nds Generated NormalyzerStatistics instance
#' @export
#' @examples
#' data(example_stat_summarized_experiment)
#' nst <- NormalyzerStatistics(example_stat_summarized_experiment)
NormalyzerStatistics <- function(experimentObj, logTrans=FALSE) { 

              dataMat <- SummarizedExperiment::assay(experimentObj)
              if (logTrans) {
                  dataMat <- log2(dataMat)
              }
              
              annotMat <- SummarizedExperiment::rowData(experimentObj)
              designDf <- SummarizedExperiment::colData(experimentObj)

              nst <- new("NormalyzerStatistics",
                         annotMat=as.matrix(annotMat), 
                         dataMat=as.matrix(dataMat), 
                         designDf=as.data.frame(designDf)
              )

              nst
          }

setGeneric("condCol", function(object) { standardGeneric("condCol") })
setMethod("condCol", signature(object="NormalyzerStatistics"), 
          function(object) { slot(object, "condCol") })
setGeneric("condCol<-", function(object, value) { standardGeneric("condCol<-") })
setReplaceMethod("condCol", signature(object="NormalyzerStatistics"), 
                 function(object, value) { 
                     slot(object, "condCol") <- value
                     validObject(object)
                     object
                 })

setGeneric("batchCol", function(object) { standardGeneric("batchCol") })
setMethod("batchCol", signature(object="NormalyzerStatistics"), 
          function(object) { slot(object, "batchCol") })
setGeneric("batchCol<-", function(object, value) { standardGeneric("batchCol<-") })
setReplaceMethod("batchCol", signature(object="NormalyzerStatistics"), 
                 function(object, value) { 
                     slot(object, "batchCol") <- value
                     validObject(object)
                     object
                 })

setGeneric("comparisons", function(object) { standardGeneric("comparisons") })
setMethod("comparisons", signature(object="NormalyzerStatistics"), 
          function(object) { slot(object, "comparisons") })
setGeneric("comparisons<-", function(object, value) { standardGeneric("comparisons<-") })
setReplaceMethod("comparisons", signature(object="NormalyzerStatistics"), 
                 function(object, value) { 
                     slot(object, "comparisons") <- value
                     validObject(object)
                     object
                 })

setGeneric("annotMat", function(object) { standardGeneric("annotMat") })
setMethod("annotMat", signature(object="NormalyzerStatistics"), 
          function(object) { slot(object, "annotMat") })

setGeneric("dataMat", function(object) { standardGeneric("dataMat") })
setMethod("dataMat", signature(object="NormalyzerStatistics"), 
          function(object) { slot(object, "dataMat") })
setGeneric("dataMat<-", function(object, value) { standardGeneric("dataMat<-") })
setReplaceMethod("dataMat", signature(object="NormalyzerStatistics"), 
                 function(object, value) { 
                     
                     slot(object, "dataMat") <- value
                     validObject(object)
                     object
                 })

setGeneric("filteredDataMat", function(object) { standardGeneric("filteredDataMat") })
setMethod("filteredDataMat", signature(object="NormalyzerStatistics"), 
          function(object) { slot(object, "filteredDataMat") })

setGeneric("designDf", function(object) { standardGeneric("designDf") })
setMethod("designDf", signature(object="NormalyzerStatistics"), 
          function(object) { slot(object, "designDf") })
setGeneric("designDf<-", function(object, value) { standardGeneric("designDf<-") })
setReplaceMethod("designDf", signature(object="NormalyzerStatistics"), 
                 function(object, value) { 
                     slot(object, "designDf") <- value
                     validObject(object)
                     object
                 })

setGeneric("filteringContrast", function(object) { standardGeneric("filteringContrast") })
setMethod("filteringContrast", signature(object="NormalyzerStatistics"), 
          function(object) { slot(object, "filteringContrast") })

setGeneric("pairwiseCompsP", function(object) { standardGeneric("pairwiseCompsP") })
setMethod("pairwiseCompsP", signature(object="NormalyzerStatistics"), 
          function(object) { slot(object, "pairwiseCompsP") })
setGeneric("pairwiseCompsP<-", function(object, value) { standardGeneric("pairwiseCompsP<-") })
setReplaceMethod("pairwiseCompsP", signature(object="NormalyzerStatistics"), 
                 function(object, value) { 
                     slot(object, "pairwiseCompsP") <- value
                     validObject(object)
                     object
                 })

setGeneric("pairwiseCompsFdr", function(object) { standardGeneric("pairwiseCompsFdr") })
setMethod("pairwiseCompsFdr", signature(object="NormalyzerStatistics"), 
          function(object) { slot(object, "pairwiseCompsFdr") })
setGeneric("pairwiseCompsFdr<-", function(object, value) { standardGeneric("pairwiseCompsFdr<-") })
setReplaceMethod("pairwiseCompsFdr", signature(object="NormalyzerStatistics"), 
                 function(object, value) { 
                     slot(object, "pairwiseCompsFdr") <- value
                     validObject(object)
                     object
                 })

setGeneric("pairwiseCompsAve", function(object) { standardGeneric("pairwiseCompsAve") })
setMethod("pairwiseCompsAve", signature(object="NormalyzerStatistics"), 
          function(object) { slot(object, "pairwiseCompsAve") })
setGeneric("pairwiseCompsAve<-", function(object, value) { standardGeneric("pairwiseCompsAve<-") })
setReplaceMethod("pairwiseCompsAve", signature(object="NormalyzerStatistics"), 
                 function(object, value) { 
                     slot(object, "pairwiseCompsAve") <- value
                     validObject(object)
                     object
                 })

setGeneric("pairwiseCompsFold", function(object) { standardGeneric("pairwiseCompsFold") })
setMethod("pairwiseCompsFold", signature(object="NormalyzerStatistics"), 
          function(object) { slot(object, "pairwiseCompsFold") })
setGeneric("pairwiseCompsFold<-", function(object, value) { standardGeneric("pairwiseCompsFold<-") })
setReplaceMethod("pairwiseCompsFold", signature(object="NormalyzerStatistics"), 
                 function(object, value) { 
                     slot(object, "pairwiseCompsFold") <- value
                     validObject(object)
                     object
                 })


#' Performs statistical comparisons between the supplied conditions.
#' It uses the design matrix and data matrix in the supplied 
#' NormalyzerStatistics object. A column is supplied specifying which of the
#' columns in the design matrix that is used for deciding the sample groups.
#' The comparisons vector specifies which pairwise comparisons between
#' condition levels that are to be calculated.
#' 
#' Optionally, a batch column can be specified allowing compensation for
#' covariate variation in the statistical model. This is only compatible
#' with a Limma-based statistical analysis.
#'
#' @param nst Results evaluation object.
#' @param comparisons String with comparisons for contrasts.
#' @param condCol Column name in design matrix containing condition information.
#' @param batchCol Column name in design matrix containing batch information.
#' @param splitter Character dividing contrast conditions.
#' @param type Type of statistical test (Limma or welch).
#' @param leastRepCount Least replicates in each group to be retained for 
#'   contrast calculations
#' @return nst Statistics object with statistical measures calculated
#' @rdname calculateContrasts 
#' @export
#' @examples
#' data(example_stat_summarized_experiment)
#' nst <- NormalyzerStatistics(example_stat_summarized_experiment)
#' results <- calculateContrasts(nst, c("1-2", "2-3"), "group")
#' resultsBatch <- calculateContrasts(nst, c("1-2", "2-3"), "group", batchCol="batch")
setGeneric(name="calculateContrasts", 
           function(nst, comparisons, condCol, batchCol=NULL, splitter="-", 
                    type="limma", leastRepCount=1) standardGeneric("calculateContrasts"))

#' @rdname calculateContrasts
setMethod(f="calculateContrasts", 
          signature=c("NormalyzerStatistics"),
          function(nst, comparisons, condCol, batchCol=NULL, splitter="-", 
                   type="limma", leastRepCount=1) {
              
              dataMat <- dataMat(nst)
              designDf <- designDf(nst)

              comparisons(nst) <- comparisons
              condCol(nst) <- as.character(designDf[, condCol])

              if (!is.null(batchCol)) {
                  conditionCombs <- paste(designDf[, condCol], designDf[, batchCol], sep="_")
                  batchCol(nst) <- as.factor(designDf[, batchCol])
              }
              else {
                  conditionCombs <- designDf[, condCol]
                  batchCol(nst) <- numeric()
              }
              
              rownames(dataMat) <- seq_len(nrow(dataMat))
              dataMatNAFiltered <- filterLowRep(
                  dataMat, 
                  conditionCombs, 
                  leastRep=leastRepCount
              )
              
              if (nrow(dataMatNAFiltered) == 0) {
                  stop("No rows remained after NA-filtering for condition: '", 
                       condCol, 
                       "' and batchCol: '", batchCol, "' (if empty then batchCol is not specified)\n",
                       "Consider whether you can reduce the 'leastRepCount' setting which sets the lower limit ",
                       "of number of NA values in each condition-level combination ",
                       "You could also try running without batchCol and see if there is enough data per condition then"
                       )
              }
              
              naFilterContrast <- rownames(dataMat) %in% rownames(dataMatNAFiltered)
              sampleReplicateGroupsStrings <- as.character(designDf(nst)[, condCol])
              statMeasures <- c("P", "FDR", "Ave", "Fold")
              
              verifyContrasts(sampleReplicateGroupsStrings, comparisons)
              
              model <- setupModel(nst, condCol, batchCol=batchCol, type=type)

              if (type %in% c("limma", "limma_intensity")) {
                  limmaDesign <- stats::model.matrix(model)
                  limmaFit <- limma::lmFit(dataMatNAFiltered, limmaDesign)
              }
              
              compLists <- list()
              for (statMeasure in statMeasures) {
                  compLists[[statMeasure]] <- list()
              }

              for (comp in comparisons) {
                  
                  compSplit <- unlist(strsplit(comp, splitter))
                  
                  if (length(compSplit) != 2) {
                      stop("Comparison should be in format cond1-cond2 ", 
                           "here the split product was: ", 
                           paste(compSplit, collapse=" "))
                  }
                  
                  level1 <- compSplit[1]
                  level2 <- compSplit[2]
                  
                  if (length(sampleReplicateGroupsStrings %in% level1) == 0) {
                      stop("No samples matching condition ", 
                           level1, 
                           " found in conditions: ", 
                           paste(sampleReplicateGroupsStrings, collapse=" "))
                  }
                  
                  if (length(sampleReplicateGroupsStrings %in% level2) == 0) {
                      stop("No samples matching condition ", 
                           level2, " found in conditions: ", 
                           paste(sampleReplicateGroupsStrings, collapse=" "))
                  }
                  
                  if (type == "welch") {
                      statResults <- calculateWelch(
                          dataMatNAFiltered, 
                          sampleReplicateGroupsStrings, 
                          c(level1, level2))
                  }
                  else if (type == "limma") {
                      
                      statResults <- calculateLimmaContrast(
                          dataMatNAFiltered, 
                          limmaDesign, 
                          limmaFit, 
                          c(level1, level2), 
                          useIntensityTrend = FALSE)
                  }
                  else if (type == "limma_intensity") {
                      
                      statResults <- calculateLimmaContrast(
                          dataMatNAFiltered, 
                          limmaDesign, 
                          limmaFit, 
                          c(level1, level2), 
                          useIntensityTrend = TRUE)
                  }
                  else {
                      stop("Unknown statistics type: ", type)
                  }
                  
                  for (statMeasure in statMeasures) {
                      compLists[[statMeasure]][[comp]] <- c()
                      compLists[[statMeasure]][[comp]][naFilterContrast] <- statResults[[statMeasure]]
                  }
              }
              
              pairwiseCompsP(nst) <- compLists[["P"]]
              pairwiseCompsFdr(nst) <- compLists[["FDR"]]
              pairwiseCompsAve(nst) <- compLists[["Ave"]]
              pairwiseCompsFold(nst) <- compLists[["Fold"]]
              
              nst
          })


#' Check that a given contrast string is valid given a particular design
#' matrix. Each level tested for in the contrast should be present in the
#' condition column for the design matrix.
#' 
#' Mainly meant to verify strings received during server usage.
#'
#' @param designLevels Vector containing condition levels present in design
#' @param contrasts A string containing one or several (comma delimited)
#'   strings for which contrasts should be performed
#' @return None
#' @keywords internal
verifyContrasts <- function(designLevels, contrasts) {
    
    for (contrast in contrasts) {
        parts <- unlist(strsplit(contrast, "-"))
        
        if (length(parts) != 2) {
            stop("A contrast string delimited by one dash (-) was expected. Instead following was found: ", contrast)
        }
        
        if (!all(parts %in% designLevels)) {
            stop("There were issues in your contrast. \n", 
                 "All contrasts: ", paste(contrasts, collapse=", "), "\n",
                 "Part with issue: ", contrast, "\n", 
                 "Not all parts was found in the design column levels. Levels present in design: \n",
                 paste(unique(designLevels), collapse=", ")
                 )
        }
    }
}

setupModel <- function(nst, condCol, batchCol=NULL, type="limma") {
    
    if (is.null(batchCol)) {
        Variable <- as.factor(designDf(nst)[, condCol])
        model <- ~0+Variable
    }
    else {
        if (!(type %in% c("limma", "limma_intensity"))) {
            stop(
                "Batch compensation only compatible with Limma, got: ", 
                type
            )
        }
        Variable <- as.factor(designDf(nst)[, condCol])
        Batch <- as.factor(designDf(nst)[, batchCol])
        model <- ~0+Variable+Batch
    }
    model
}

calculateWelch <- function(dataMat, groupHeader, levels) {
    
    s1cols <- which(groupHeader %in% levels[1])
    s2cols <- which(groupHeader %in% levels[2])
    
    doTTest <- function(c1Vals, c2Vals, default=NA) {
        if (length(stats::na.omit(c1Vals)) > 1 && 
            length(stats::na.omit(c2Vals)) > 1) {
            tryCatch(stats::t.test(c1Vals, c2Vals)[[3]], error=function(x) NA)
        }
        else {
            NA
        }
    }
    
    welchPValCol <- apply(
        dataMat, 1, 
        function(row) doTTest(row[s1cols], row[s2cols], default=NA))
    welchFDRCol <- stats::p.adjust(welchPValCol, method="BH")
    
    statResults <- list()

    statResults[["P"]] <- welchPValCol
    statResults[["FDR"]] <- welchFDRCol
    statResults[["Ave"]] <- rowMeans(dataMat, na.rm=TRUE)
    statResults[["Fold"]] <- rowMeans(dataMat[, s1cols], na.rm=TRUE) - rowMeans(dataMat[, s2cols], na.rm=TRUE)

    statResults
}

calculateLimmaContrast <- function(dataMat, limmaDesign, limmaFit, levels, useIntensityTrend) {

    myContrast <- paste0("Variable", levels[1], "-", "Variable", levels[2])
    contrastMatrix <- limma::makeContrasts(
        contrasts=c(myContrast), 
        levels=limmaDesign)
    fitContrasts <- limma::contrasts.fit(limmaFit, contrastMatrix)
    fitBayes <- limma::eBayes(fitContrasts, trend=useIntensityTrend)
    limmaTable <- limma::topTable(fitBayes, coef=1, number=Inf, sort.by="none")

    statResults <- list()
    statResults[["P"]] <- limmaTable$P.Value
    statResults[["FDR"]] <- limmaTable$adj.P.Val
    statResults[["Ave"]] <- limmaTable$AveExpr
    statResults[["Fold"]] <- limmaTable$logFC
    statResults
}

#' Calculate measures for normalization results
#' 
#' This function prepares an NormalyzerEvaluationResults object containing
#' the evaluation measures CV (coefficient of variance), MAD (median absolute
#' deviation), average variance, significance measures (ANOVA between
#' condition groups) and correlation between replicates.
#' 
#' @param nr Normalyzer results object with calculated results.
#' @param categoricalAnova Whether categorical or numerical (ordered) ANOVA
#' should be calculated.
#' @return Normalyzer results with attached evaluation results object.
#' @export
#' @examples
#' data(example_summarized_experiment)
#' normObj <- getVerifiedNormalyzerObject("job_name", example_summarized_experiment)
#' normResults <- normMethods(normObj)
#' normResultsWithEval <- analyzeNormalizations(normResults)
analyzeNormalizations <- function(nr, categoricalAnova=FALSE) {

    ner <- NormalyzerEvaluationResults(nr, categoricalAnova)
    ner(nr) <- ner
    
    nr
}

#' Calculate CV per replicate group and normalization technique
#' 
#' Iterates through each normalization method and calculate average CV values
#' per replicate group.
#' 
#' @param methodList List containing normalized matrices.
#' @param sampleReplicateGroups Condition header.
#' @return avgCVPerNormAndReplicates Matrix with group CVs as rows and
#'   normalization technique as columns
#' @keywords internal
calculateReplicateCV <- function(methodList, sampleReplicateGroups) {
    
    calculateFeatureCVs <- function(feature, groups) {

        featureCVs <- vapply(
            unique(groups), 
            function(group) {
                targetFeatures = feature[groups == group]
                stats::sd(targetFeatures, na.rm=TRUE) / mean(targetFeatures, na.rm=TRUE) 
            }, 
            c(1)
        )
        featureCVs
    }
    
    calculateMethodReplicateCVs <- function(methodData, groups) {
        
        cvPerFeatureAndGroup <- apply(
            methodData,
            1,
            calculateFeatureCVs,
            groups=sampleReplicateGroups
        )
        
        if (length(unique(groups)) > 1) {
            # Transpose to get groups as column heads
            featureCondCVs <- t(cvPerFeatureAndGroup)
        }
        else if (length(unique(groups)) == 1) {
            # For one feature the CVs are dropped to a vector
            featureCondCVs <- data.frame(cvPerFeatureAndGroup)
            colnames(featureCondCVs) <- groups[1]
        }
        else {
            stop("Unknown state encountered for groups:", paste(groups, collapse=", "))
        }
        
        # Calculate mean CV for all features for each condition
        summedCondCVs <- colMeans(featureCondCVs, na.rm=TRUE)
        summedCondCVs * 100
    }
    
    avgCVPerNormAndReplicates <- vapply(
        methodList,
        calculateMethodReplicateCVs,
        rep(0, length(unique(sampleReplicateGroups))),
        groups=sampleReplicateGroups
    )
    
    # If only one group, this is needed to get data in correct shape and format
    if (length(unique(sampleReplicateGroups)) == 1) {
        avgCVPerNormAndReplicates <- t(as.matrix(avgCVPerNormAndReplicates))
    }

    avgCVPerNormAndReplicates
}

#' Calculate CV values for each feature. Iterates through each normalization 
#' method and calculates a matrix of CV values where each column correspond to 
#' a method and each row corresponds to a feature.
#' 
#' @param methodList List containing normalized matrices.
#' @return methodFeatureCVMatrix Matrix with feature as rows and normalization
#'   method as columns
#' @keywords internal
calculateFeatureCV <- function(methodList) {
    
    methodCount <- length(methodList)
    numberFeatures <- nrow(methodList[[1]])

    calculateFeatureCVVector <- function(processedDataMatrix) {
        cv <- function(row) {
            stDev <- stats::sd(row, na.rm=TRUE)
            meanVal <- mean(unlist(row), na.rm=TRUE)
            stDev / mean(meanVal) * 100
        }
        featureCVs <- apply(processedDataMatrix, 1, cv)
        featureCVs
    }
    
    methodFeatureCVMatrix <- vapply(
        methodList,
        calculateFeatureCVVector,
        rep(0, numberFeatures)
    )

    colnames(methodFeatureCVMatrix) <- names(methodList)
    methodFeatureCVMatrix
}

#' Calculate average MAD (Median Absolute Deviation) for each feature in
#' each condition and then calculates the average for each replicate group
#' 
#' @param methodList List containing normalized matrices.
#' @param sampleReplicateGroups Condition header.
#' @return condAvgMadMat Matrix with average MAD for each biological condition.
#' @keywords internal
calculateAvgMadMem <- function(methodList, sampleReplicateGroups) {
    
    groupIndexList <- getIndexList(sampleReplicateGroups)

    calculateAvgFeatureMadForGroup <- function(groupIndices, methodData) {
        groupData <- methodData[, groupIndices]
        featureMAD <- matrixStats::rowMads(groupData, na.rm=TRUE)
        featureMAD
    }
        
    calculateGroupMadForMethod <- function(methodData, groups, indexList) {

        # Extracts groups of replicates and calculate MAD for each feature
        featureMADMat <- vapply(
            indexList,
            calculateAvgFeatureMadForGroup,
            rep(0, nrow(methodData)),
            methodData=methodData
        )
        
        methodRepGroupMADMean <- colMeans(featureMADMat, na.rm=TRUE)
        methodRepGroupMADMean
    }
    
    condAvgMadMat <- vapply(
        methodList,
        calculateGroupMadForMethod,
        rep(0, length(unique(sampleReplicateGroups))),
        groups=sampleReplicateGroups,
        indexList=groupIndexList
    ) 

    condAvgMadMat
}

#' Calculate average variance for each feature in each condition and then
#' calculate the average for each replicate group
#' 
#' @param methodList List containing normalized matrices.
#' @param sampleReplicateGroups Condition header.
#' @return avgVarianceMat Matrix with average variance for each biological
#' condition
#' @keywords internal
calculateAvgReplicateVariation <- function(methodList, sampleReplicateGroups) {

    groupIndexList <- getIndexList(sampleReplicateGroups)
    
    calculateReplicateGroupVariance <- function(groupIndices, methodData) {
        
        groupData <- methodData[, groupIndices]
        rowNonNACount <- rowSums(!is.na(groupData)) - 1
        rowVariances <- rowNonNACount * matrixStats::rowVars(groupData, na.rm=TRUE)
        replicateGroupVariance <- sum(rowVariances, na.rm=TRUE) / sum(rowNonNACount, na.rm=TRUE)
        replicateGroupVariance
    }
    
    avgVarianceMat <- vapply(
        methodList,
        function(methodData) {
            replicateGroupVariance <- vapply(
                groupIndexList,
                calculateReplicateGroupVariance,
                0,
                methodData=methodData
            )
            replicateGroupVariance
        },
        rep(0, length(unique(sampleReplicateGroups)))
    )
    
    avgVarianceMat
}

#' Calculates correlation values between replicates for each condition matrix.
#' Finally returns a matrix containing the results for all dataset
#' 
#' @param methodlist List containing normalized matrices for each normalization
#'   method
#' @param allReplicateGroups Vector with condition groups matching the columns
#'   found in the normalization methods
#' @param sampleGroupsWithReplicates Unique vector with condition groups
#'   present in two or more samples
#' @param corrType Type of correlation (Pearson or Spearman)
#' @return avgCorSum Matrix with column corresponding to normalization
#' approaches and rows corresponding to replicate group
#' @keywords internal
calculateSummarizedCorrelationVector <- function(
    methodlist, allReplicateGroups, sampleGroupsWithReplicates, corrType) {
    
    validCorrTypes <- c("pearson", "spearman")
    if (!corrType %in% validCorrTypes) {
        stop("Unknown correlation type: ", 
             corrType, 
             " valid are: ", 
             paste(validCorrTypes, collapse=", "))
    }
    
    corr_combination_count <- function(allReplicateGroups) {
        replicate_counts <- table(allReplicateGroups)
        sum(vapply(
            replicate_counts, 
            function(count) { (count * (count-1)) / 2 }, 
            0))
    }
    
    avgCorSum <- vapply(
        methodlist,
        calculateCorrSum,
        rep(0, corr_combination_count(allReplicateGroups)),
        allReplicateGroups=allReplicateGroups,
        sampleGroupsWithReplicates=sampleGroupsWithReplicates,
        corrType=corrType
    )
    
    avgCorSum
}

#' Calculates internal correlations for each condition having at least two
#' samples and returns a vector with correlation values corresponding to each
#' condition
#' 
#' @param methodData Expression data matrix
#' @param allReplicateGroups Full condition header corresponding to data tables
#'   columns
#' @param sampleGroupsWithReplicates Unique conditions where number of
#'   replicates exceeds one
#' @param corrType Type of correlation (Pearson or Spearman)
#' @return corSums
#' @keywords internal
calculateCorrSum <- function(methodData, allReplicateGroups, 
                             sampleGroupsWithReplicates, corrType) {
    
    corSums <- vector()
    for (groupNbr in seq_along(sampleGroupsWithReplicates)) {

        specificReplicateVals <- as.matrix(
            methodData[, which(allReplicateGroups == sampleGroupsWithReplicates[groupNbr])])
        class(specificReplicateVals) <- "numeric"
        corVals <- stats::cor(
            specificReplicateVals ,
            use="pairwise.complete.obs",
            method=corrType)

        for (index in seq_len(ncol(specificReplicateVals) - 1)) {
            corSums <- c(
                corSums,
                corVals[index, -(seq_len(index)), drop="FALSE"]
            )
        }
    }

    corSums
}

#' Calculates ANOVA p-values comparing the different condition groups
#' returning a vector of resulting p-values with NA-values where too few
#' values were present in at least one of the groups.
#' 
#' @param methodList List containing normalized matrices
#' @param sampleReplicateGroups Condition header
#' @param categoricalANOVA Whether the ANOVA should be calculated using ordered
#' or categorical groups
#' @return avgVarianceMat Matrix with average variance for each biological
#' condition
#' @keywords internal
calculateANOVAPValues <- function(methodList, 
                                  sampleReplicateGroups, 
                                  categoricalANOVA) {
    
    calculateANOVAsForMethod <- function(methodData, sampleReplicateGroups) {

        naFilterContrast <- getRowNAFilterContrast(
            methodData,
            sampleReplicateGroups,
            2)
        
        naFilteredData <- methodData[naFilterContrast,, drop=FALSE]
        
        if (categoricalANOVA) {
            testLevels <- factor(sampleReplicateGroups)
        }
        else {
            testLevels <- sampleReplicateGroups
        }
        
        anovaPValCol <- apply(
            naFilteredData, 
            1, 
            function(sampleIndex) {
                summary(stats::aov(unlist(sampleIndex)~testLevels))[[1]][[5]][1]
            } 
        )

        anovaPValsWithNACol <- rep(NA, nrow(methodData))        
        anovaPValsWithNACol[naFilterContrast] <- anovaPValCol
        anovaPValsWithNACol
    }
    
    anovaPValsWithNA <- vapply(
        methodList,
        calculateANOVAsForMethod,
        rep(0, nrow(methodList[[1]])),
        sampleReplicateGroups=sampleReplicateGroups
    )
    
    anovaPValsWithNA
}

#' Uses a list of FDR-values to extract features with low variance in the
#' log2-transformed dataset. This is then used to calculate the average CV
#' for these 'lowly variable' features in each normalization approach
#' 
#' @param referenceFDR List of FDR values used as non-normalized reference
#' @param methodList List containing normalized matrices
#' @return lowVarFeaturesAverageCVs Average CV values for lowly variable
#' features in each normalization approach
#' @keywords internal
findLowlyVariableFeaturesCVs <- function(referenceFDR, methodList) {
    
    referenceFDRWoNA <- stats::na.omit(referenceFDR)
    
    fivePercCount <- 5 * length(referenceFDRWoNA) / 100
    pThresLowVal <- min(utils::head(rev(sort(referenceFDRWoNA)), n=fivePercCount))
    nbrAbovePThres <- sum(referenceFDRWoNA >= pThresLowVal)
    
    if (is.infinite(pThresLowVal) || nbrAbovePThres == 0) {
        warning("Too few successful ANOVA calculations to generate lowly ",
                "variable features, skipping")
        return(NULL)
    }
    
    lowlyVariableFeatures <- referenceFDR >= pThresLowVal
    
    if (length(lowlyVariableFeatures) != nrow(methodList[[1]])) {
        stop("Lowly variable features contrast needs to be same length as ",
             "number of rows in matrix")
    }
    
    lowVarFeaturesAverageCVs <- vector()
    methodCount <- length(methodList)

    calculateAverageCVs <- function(methodData, lowVarContrast) {
        
        lowVarFeatures <- methodData[lowVarContrast, ]
        averageCVs <- mean(
            matrixStats::rowSds(lowVarFeatures, na.rm=TRUE) / rowMeans(lowVarFeatures, na.rm=TRUE) * 100,
            na.rm=TRUE)
        averageCVs
    }
    
    lowVarFeaturesAverageCVs <- vapply(
        methodList,
        calculateAverageCVs,
        0,
        lowVarContrast=lowlyVariableFeatures
    )
    
    lowVarFeaturesAverageCVs
}

context("analyzeResults.R")

data("example_data_only_values")
data("example_design")
test_design <- example_design[example_design$group %in% c("1", "2", "3"), ]
test_data <- example_data_only_values[, as.character(test_design$sample)]
group_header <- test_design$group
unique_groups <- unique(group_header)

# Setup equivalent to regression_test_nr
data("example_wide_data")
data("example_wide_design")
sdf <- example_wide_data[, example_wide_design$sample]
adf <- example_wide_data[, !colnames(example_wide_data) %in% example_wide_design$sample]
se <- SummarizedExperiment::SummarizedExperiment(
    assays = list(raw=as.matrix(sdf)),
    colData=example_wide_design,
    rowData=adf
)
SummarizedExperiment::metadata(se) <- list(
    sample="sample",
    group="group"
)

nds <- NormalyzerDE::getVerifiedNormalyzerObject(
    "unit_test_run_norm",
    summarizedExp = se
)
regression_test_nr <- NormalyzerDE::normMethods(nds)
regression_test_nr <- NormalyzerDE::analyzeNormalizations(regression_test_nr)
regression_test_ner <- regression_test_nr@ner

#data("regression_test_nr")
#regression_test_ner <- ner(regression_test_nr)
#nds <- nds(regression_test_nr)

# browser()

sampleReplicateGroups <- sampleReplicateGroups(nds)
normMatrices <- normalizations(regression_test_nr)
anova_pvalues <- calculateANOVAPValues(
    normMatrices,
    sampleReplicateGroups,
    categoricalANOVA=FALSE)

test_that("calculateCorrSum gives same Pearson output", {

    expected_group_pearson <- c(
        0.9890682, 0.9885302, 0.9972236, 
        0.9937719, 0.9976159, 0.9948526, 
        0.6108868, 0.9951509, 0.5691397
    )
    
    pear_out <- calculateCorrSum(test_data, group_header, unique_groups, "pearson")
    expect_true(
        all.equal(
            expected_group_pearson,
            round(pear_out, 7)
        )
    )
})

test_that("calculateCorrSum gives same Spearman output", {

    expected_group_spearman <- c(
        0.9234304, 0.9665739, 0.9497886,
        0.9686747, 0.9551886, 0.9420609, 
        0.9250426, 0.9549601, 0.9315987
    )
    
    spear_out <- calculateCorrSum(test_data, group_header, unique_groups, "spearman")
    expect_true(
        all.equal(
            expected_group_spearman,
            round(spear_out, 7)
        )
    )
})

test_that("calculateSummarizedCorrelationVector_Spearman", {
    
    # expected_out <- repCorSpear(regression_test_ner)
    expected_out <- read.csv(system.file(package="NormalyzerDE", "testdata", "calculateSummarizedCorrelationVector_spearman.csv"), check.names = FALSE)
    out <- calculateSummarizedCorrelationVector(
        normMatrices, 
        sampleReplicateGroups,
        unique(sampleReplicateGroups),
        "spearman"
    )
    
    expect_true(
        all.equal(
            round(as.matrix(expected_out), 5),
            round(out, 5)
        )
    )
})

test_that("calculateSummarizedCorrelationVector_Pearson", {
    
    #expected_out <- repCorPear(regression_test_ner)
    expected_out <- read.csv(system.file(package="NormalyzerDE", "testdata", "calculateSummarizedCorrelationVector_pearson.csv"), check.names = FALSE)
    out <- as.matrix(unlist(calculateSummarizedCorrelationVector(
        normMatrices, 
        sampleReplicateGroups,
        unique(sampleReplicateGroups),
        "pearson"
    )))

    expect_true(
        all.equal(
            round(as.matrix(expected_out), 5),
            round(out, 5)
        )
    )
})

test_that("calculateReplicateCV", {
    
    expected_out <- avgcvmem(regression_test_ner)
    out <- calculateReplicateCV(normMatrices, sampleReplicateGroups)

    expect_true(
        all.equal(
            expected_out,
            out
        )
    )
})

test_that("calculateFeatureCV", {
    
    expected_out <- featureCVPerMethod(regression_test_ner)
    out <- calculateFeatureCV(normMatrices)
    
    expect_true(
        all.equal(
            expected_out,
            out
        )
    )
})

test_that("calculateAvgMadMem", {
    
    expected_out <- avgmadmem(regression_test_ner)
    out <- calculateAvgMadMem(normMatrices, sampleReplicateGroups)

    expect_true(
        all.equal(
            expected_out,
            out
        )
    )
})

test_that("calculateAvgReplicateVariation", {
    
    expected_out <- avgvarmem(regression_test_ner)
    out <- calculateAvgReplicateVariation(normMatrices, sampleReplicateGroups)

    expect_true(
        all.equal(
            expected_out,
            out
        )
    )
})


test_that("calculateANOVAPValues", {
    
    # expected_out <- anovaP(regression_test_ner)
    expected_out <- read.csv(system.file(package="NormalyzerDE", "testdata", "anova.csv"), check.names = FALSE)
    anova_pvalues_copy <- anova_pvalues

    expect_true(
        all.equal(
            round(as.matrix(expected_out), 4),
            round(anova_pvalues_copy, 4)
        )
    )
})

test_that("findLowlyVariableFeaturesCVs", {
    
    # expected_out <- lowVarFeaturesCVs(regression_test_ner)
    expected_out <- c(
        "log2" = 1.153562, 
        "VSN" = 1.089077, 
        "GI" = 1.159375, 
        "median" = 1.320822, 
        "mean" = 1.163794, 
        "Quantile" = 1.120080, 
        "CycLoess" = 1.032822, 
        "RLR" = 1.048589,
        "RT-median" = 1.320822, 
        "RT-mean" = 1.163794, 
        "RT-Loess" = 1.032822, 
        "RT-VSN" = 1.089077
    )
    
    anova_pvalues_contrast_nonna <- !is.na(anova_pvalues[, 1])
    log2AnovaFDR <- rep(NA, length(anova_pvalues_contrast_nonna))
    log2AnovaFDR[anova_pvalues_contrast_nonna] <- stats::p.adjust(
        anova_pvalues[, 1][anova_pvalues_contrast_nonna], 
        method="BH")
    
    out <- findLowlyVariableFeaturesCVs(log2AnovaFDR, normMatrices)

    expect_true(
        all.equal(
            round(expected_out, 6),
            round(out, 6)
        )
    )
})

test_that("calculatePercentageAvgDiffInMat_small_test", {
    
    test_mat <- data.frame(c(1,3), c(1,5), c(1,7))
    expect_out <- c(100, 150, 200)
    out <- calculatePercentageAvgDiffInMat(test_mat)
    
    expect_true(
        all.equal(
            out,
            expect_out
        )
    )
})

test_that("calculatePercentageAvgDiffInMat_MAD", {
    
    expected_out <- avgmadmempdiff(regression_test_ner)
    avgMadMemMat <- calculateAvgMadMem(normMatrices, sampleReplicateGroups)
    out <- calculatePercentageAvgDiffInMat(avgMadMemMat)
    
    expect_true(
        all.equal(
            expected_out,
            out
        )
    )
})

test_that("calculatePercentageAvgDiffInMat_CV", {
    
    expected_out <- avgcvmempdiff(regression_test_ner)
    avgCVMat <- calculateReplicateCV(normMatrices, sampleReplicateGroups)
    out <- calculatePercentageAvgDiffInMat(avgCVMat)
    
    expect_true(
        all.equal(
            expected_out,
            out
        )
    )
})

#' Print meta information for Normalyzer plot page
#' ! Needs refactoring to reduce redundancy in code
#' ! Needs double check of functionality
#' 
#' @param plotname Name of current plot.
#' @param pageno Current page number.
#' @param jobname Name of ongoing job.
#' @param currentLayout Custom viewport layout.
#' @return None
#' @keywords internal
printMeta <- function(plotname, pageno, jobname, currentLayout) {

    ncol <- currentLayout$ncol
    nrow <- currentLayout$nrow
        
    gp <- grid::gpar(fontsize=11, fontfamily="Helvetica", 
                     col="black", fontface="bold")
    gpfill <- grid::gpar(fill="gray90", lwd=0, lty=0)
    
    for (pos in seq_len((ncol - 1) * (nrow - 1))) {
        
        if (pos <= ncol) {
            row <- 1
        }
        else {
            row <- nrow
        }
        
        col <- (pos - 1) %% ncol + 1
        grid::grid.rect(vp=grid::viewport(layout.pos.row=row, 
                                          layout.pos.col=col), gp=gpfill)
    }
    
    grid::grid.text(plotname, 
                    vp=grid::viewport(layout.pos.row=1, layout.pos.col=1), 
                    just=c("left", "center"), 
                    gp=grid::gpar(fontface="bold", fontsize=14, fontfamily="Helvetica", col="darkBlue"))
    grid::grid.text("NormalyzerDE Report", 
                    vp=grid::viewport(layout.pos.row=1, layout.pos.col=ncol),
                    just=c("right", "center"), gp=gp)
    grid::grid.text(paste("Page ", pageno,sep=""), 
                    vp=grid::viewport(layout.pos.row=nrow, layout.pos.col=ncol),
                    just=c("right", "center"), gp=gp)
    grid::grid.text(paste("Project: ", jobname, sep=""), 
                    vp=grid::viewport(layout.pos.row=nrow, layout.pos.col=1), just=c("left", "center"), gp=gp)
}

#' Generates a number of visualizations for the performance measures calculated
#' for the normalized matrices. These contain both general measures and
#' direct comparisons for different normalization approaches.
#' 
#' They include:
#' 
#' "Total intensity" 
#' Barplot showing the summed intensity in each sample for thelog2-transformed 
#' data
#' 
#' "Total missing" 
#' Barplot showing the number of missing values found in each sample for the 
#' log2-tranformed data
#' 
#' Log2-MDS plot: MDS plot where data is reduced to two dimensions allowing 
#' inspection of the main global changes in the data
#' 
#' PCV - Intragroup: Mean of intragroup CV of all replicate groups
#' 
#' PMAD - Intragroup: Mean of intragroup median absolute deviation across 
#' replicate groups
#' 
#' PEV - Intragroup: Mean of intragroup pooled estimate of variance across the 
#' replicate groups
#' 
#' Relative PCV, PMAD and PEV compared to log2: The results from PCV, PMAD
#' and PEV from all normalized data compared to the log2 data
#' 
#' Stable variables plot:
#' 5% of least differentially expressed variables are identified by ANOVA
#' analysis of log2 transformed data. Thereafter, global CV of these variables
#' is estimated from different normalized datasets. A plot of global CV of the
#' stable variables from all datsets on the y-axis and PCV-compared to log2 on
#' the x-axis is generated.
#' 
#' CV vs Raw Intensity plots:
#' For the first replicate group in each of the normalized dataset, a plot of
#' PCV of each variable compared to the average intensity of the variable in
#' the replicate group is plotted.
#' 
#' MA plots:
#' Plotted using the plotMA function of the limma package. The first sample in
#' each dataset is plotted against the average of the replicate group that
#' sample belong to.
#' 
#' Scatterplots:
#' The first two samples from each dataset are plotted.
#' 
#' Q-Q plots:
#' QQ-plots are plotted for the first sample in each normalized dataset.
#' 
#' Boxplots:
#' Boxplots for all samples are plotted and colored according to the replicate
#' grouping.
#' 
#' Relative Log Expression (RLE) plots:
#' Relative log expression value plots. Ratio between the expression of the
#' variable and the median expression of this variable across all samples.
#' The samples should be aligned around zero. Any deviation would indicate
#' discrepancies in the data.
#' 
#' Density plots:
#' Density distributions for each sample using the density function. Can 
#' capture outliers (if single densities lies far from the others) and see
#' if there is batch effects in the dataset (if for instance there is two
#' clear collections of lines in the data).
#' 
#' MDS plots
#' Multidimensional scaling plot using the cmdscale() function from the stats
#' package. Is often able to show whether replicates group together, and
#' whether there are any clear outliers in the data.
#' 
#' MeanSDplots
#' Displays the standard deviation values against values ordered according
#' to mean. If no dependency on mean is present (as is desired) a flat red
#' line is shown.
#' 
#' Pearson and Spearman correlation
#' Mean of intragroup Pearson and Spearman correlation values for each method.
#' 
#' Dendograms
#' Generated using the hclust function. Data is centered and scaled prior to
#' analysis. Coloring of replicates is done using as.phylo from the ape package.
#' 
#' P-value histograms
#' Histogram plots of p-values after calculating an ANOVA between different
#' condition groups. If no effect is present in the data a flat distribution
#' is expected. If an effect is present a flat distribution is still expected,
#' but with a sharp peak close to zero. If other effects are present it might
#' indicate that the data doesn't support the assumptions of ANOVA, for
#' instance if there are batch effects present in the data.
#' 
#' @param nr Normalyzer results object.
#' @param jobdir Path to output directory for run.
#' @param plotRows Number of plot rows.
#' @param plotCols Number of plot columns.
#' @param writeAsPngs Output the report as PNG-plots instead of a single PDF
#' @return None
#' @export
#' @examples
#' data(example_summarized_experiment)
#' normObj <- getVerifiedNormalyzerObject("job_name", example_summarized_experiment)
#' normResults <- normMethods(normObj)
#' normResultsWithEval <- analyzeNormalizations(normResults)
#' outputDir <- tempdir()
#' generatePlots(normResultsWithEval, outputDir)
generatePlots <- function(nr, jobdir, plotRows=3, plotCols=4, writeAsPngs=FALSE) {
    
    nds <- nds(nr)
    currentjob <- jobName(nds)
    nrows <- plotRows + 2
    ncols <- plotCols + 2
    
    currentLayout <- grid::grid.layout(
        nrow=nrows, 
        ncol=ncols,
        heights=c(0.1, rep(3 / (nrows - 2), (nrows-  2)), 0.1), 
        widths=c(0.1, rep(4 / (ncols - 2), (ncols - 2)), 0.1), 
        default.units=c('null', 'null')
    )
    
    pageno <- 1
    nextPageNo <- function(iter=TRUE) {
        if (iter) {
          pageno <<- pageno + 1
        }
        pageno
    }
    
    currentFont <- "Helvetica"
    setupPlotting(currentjob, jobdir, "Norm-report")
    
    if (writeAsPngs) {
        pngDir <- sprintf("%s/pngs", jobdir)
        dir.create(pngDir)
    }
    else {
      pngDir <- NULL
    }
    
    writePage(plotFrontPage, jobdir, writeAsPngs, sprintf("%s/%s_%s", pngDir, nextPageNo(iter=FALSE), "front_page.png"), currentjob, currentFont)
    isLimitedRun <- singleReplicateRun(nds) || isTinyRun(nds)
    writePage(plotSampleMappingPage, jobdir, writeAsPngs, sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "sample_setup.png"), nr, currentFont, currentLayout, currentjob, nextPageNo(iter=FALSE))
    writePage(plotSampleOutlierSummary, jobdir, writeAsPngs, sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "outlier_detection.png"), nr, currentLayout, nextPageNo(iter=FALSE))
    
    # If running with single-replicates, skip replicate dependent quality plots
    if (!isLimitedRun) {
        writePage(plotReplicateVariance, jobdir, writeAsPngs, sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "replicate_variation.png"), nr, currentLayout, nextPageNo(iter=FALSE))
        writePage(plotReplicateVarAndStableVariables, jobdir, writeAsPngs, sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "replicate_variation_rel_to_log2.png"), nr, currentLayout, nextPageNo(iter=FALSE))
        writePage(plotCVvsIntensity, jobdir, writeAsPngs, sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "cv_vs_raw_intensity.png"), nr, currentLayout, nextPageNo(iter=FALSE))
        writePage(plotMA, jobdir, writeAsPngs, sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "ma.png"), nr, currentLayout, nextPageNo(iter=FALSE))
    }

    writePage(plotScatter, jobdir, writeAsPngs, sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "scatter.png"), nr, currentLayout, nextPageNo(iter=FALSE))
    writePage(plotQQ, jobdir, writeAsPngs, sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "qqplots.png"), nr, currentLayout, nextPageNo(iter=FALSE))
    writePage(plotBoxPlot, jobdir, writeAsPngs, sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "boxplot.png"), nr, currentLayout, nextPageNo(iter=FALSE))
    writePage(plotRLE, jobdir, writeAsPngs, sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "rel_log_expr.png"), nr, currentLayout, nextPageNo(iter=FALSE))
    writePage(plotDensity, jobdir, writeAsPngs, sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "density.png"), nr, currentLayout, nextPageNo(iter=FALSE))
    writePage(plotMDS, jobdir, writeAsPngs, sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "mds.png"), nr, currentLayout, nextPageNo(iter=FALSE))
    
    if (!isLimitedRun) {   
        writePage(plotMeanSD, jobdir, writeAsPngs, sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "mean_sd.png"), nr, currentLayout, nextPageNo(iter=FALSE))
        writePage(plotCorrelation, jobdir, writeAsPngs, sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "correlation.png"), nr, currentLayout, nextPageNo(iter=FALSE))
    }
     
    writePage(plotDendrograms, jobdir, writeAsPngs, sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "dendrogram.png"), nr, currentLayout, nextPageNo(iter=FALSE))
    
    if (!isLimitedRun) {
        writePage(plotPHist, jobdir, writeAsPngs, sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "sample_setup.png"), nr, currentLayout, nextPageNo(iter=FALSE))
    }
    
    grDevices::dev.off()
}

writePage <- function(outputFunc, jobdir, writeAsPngs, pngPath=NULL, ...) {
  if (!is.null(pngPath) && writeAsPngs) {
    grDevices::png(filename=pngPath, width = 11.7, height = 8.3, units="in", res=300)
    outputFunc(...)
    grDevices::dev.off()
  }
  else {
    outputFunc(...)
  }
}

#' Setup PDF report settings by initializing the color palette, format
#' for the PDF report and the graphical device
#' 
#' @param currentJob Name of current run.
#' @param jobDir Path to output directory for run.
#' @param suffix Text to add to output filename.
#' @return None
#' @keywords internal
setupPlotting <- function(currentJob, jobDir, suffix) {
    
    grDevices::palette(c(
        "red", "green", "blue", "orange", "darkgray", "blueviolet", 
        "darkslateblue", "darkviolet", "gray", "bisque4", "brown", 
        "cadetblue4", "darkgreen", "darkcyan", "darkmagenta", "darkgoldenrod4", 
        "coral1"
    ))

    grDevices::pdf(
      file=paste(jobDir, "/", suffix, "-", currentJob, ".pdf", sep=""),
      paper="a4r", width=0, height=0
    )
    
    themeNorm <- ggplot2::theme_set(ggplot2::theme_bw())
    themeNorm <- ggplot2::theme_update(
        panel.grid.minor=ggplot2::element_blank(), 
        axis.text=ggplot2::element_text(size=7), 
        axis.title=ggplot2::element_text(size=8), 
        plot.title=ggplot2::element_text(size=8), 
        plot.margin=ggplot2::unit(c(1, 1, 1, 1), "mm")
    )
    # def.par <- graphics::par(no.readonly=TRUE)
}

#' Generate first page in output report and write to viewport
#' 
#' @param currentjob Name of current run.
#' @param currentFont Font used for output document.
#' @return None
#' @keywords internal
plotFrontPage <- function(currentjob, currentFont) {
    
    graphics::par(mfrow=c(4, 1))
    # TODO: Re-insert nice illustration (figure?)
    # data(data4pdftitle)
    graphics::plot(1, type="n", axes=FALSE, xlab="", ylab="")
    
    if ("NormalyzerDE" %in% rownames(utils::installed.packages())) {
      version <- utils::packageVersion("NormalyzerDE")
    }
    else {
      version <- "(version not found)"
    }
    
    la1 <- grid::grid.layout(
        nrow=7, 
        ncol=1, 
        heights=c(0.2, 1, 0.1, 0.2, 0.1, 0.2, 0.2), 
        default.units=c('null','null')
    )
    gpfill <- grid::gpar(fill="gray90", lwd=0, lty=0)
    grid::pushViewport(grid::viewport(layout=la1))
    grid::grid.rect(vp=grid::viewport(layout.pos.row=1), gp=gpfill)
    grid::grid.rect(vp=grid::viewport(layout.pos.row=7), gp=gpfill)
    
    grid::grid.text(
        paste("Project Name: ", currentjob, sep=""), 
        vp=grid::viewport(layout.pos.row=3), 
        just=c("center","center"), 
        gp=grid::gpar(fontsize=12, fontfamily=currentFont, col="black"))
    
    grid::grid.text(
        paste0("NormalyzerDE (ver ", version, " )"), 
        vp=grid::viewport(layout.pos.row=4), 
        just=c("center", "center"), 
        gp=grid::gpar(
            fontface="bold", 
            fontsize=32, 
            fontfamily=currentFont, 
            col="darkblue")
        )
    
    grid::grid.text(
        paste("Report created on: ", Sys.Date(), sep=""), 
        vp=grid::viewport(layout.pos.row=5), 
        just=c("center","center"),
        gp=grid::gpar(fontsize=12, fontfamily=currentFont, col="black"))
    
    citationText <- paste(
        "Citation: NormalyzerDE: Online Tool for Improved Normalization of",
        "Omics Expression Data and High-Sensitivity Differential Expression Analysis\n",
        "Journal of Proteome Research (2018), 10.1021/acs.jproteome.8b00523"
    )
    
    grid::grid.text(
        citationText, 
        vp=grid::viewport(layout.pos.row=6), 
        just=c("center","center"), 
        gp=grid::gpar(fontsize=10, fontfamily=currentFont, col="black")
    )
    
    grid::grid.text(
        paste("Documentation for analyzing this report can be found at",
              "http://quantitativeproteomics.org/normalyzer/help.php"),
        vp=grid::viewport(layout.pos.row=7), 
        just=c("center", "center"), 
        gp=grid::gpar(fontsize=10, fontfamily=currentFont, col="black")
    )
}


#' Write page with sample mapping
#' 
#' @param nr Normalyzer results object.
#' @param currentLayout Layout used for document.
#' @param pageno Current page number.
#' @return None
#' @keywords internal
plotSampleMappingPage <- function(nr, currentFont, currentLayout, currentjob, pageno) {
  
  nds <- nds(nr)
  currentjob <- jobName(nds)
  
  tout <- rbind(c(1, 2), c(3, 4))
  graphics::layout(tout)
  graphics::par(mar=c(4, 4, 2, 1), oma=c(2, 2, 3, 2), xpd=NA)        
  
  grid::grid.newpage()
  groupColName <- groupNameCol(nds)
  groupCol <- designMatrix(nds)[[groupColName]]
  
  groupLevels <- levels(as.factor(groupCol))
  condCounts <- table(groupCol)
  
  groupList <- lapply(
    seq_len(length(groupLevels)), 
    function(lv, groups, groupCounts) { 
      list(groupNbr=lv, groupString=groups[[lv]], groupCount=groupCounts[[lv]]) 
    }, 
    groups=groupLevels, groupCounts=condCounts)
  
  groupStringsWHead <- c(list(list(groupNbr="Group nbr.", groupString="Design group", groupCount="Nbr. samples in cond."), list(groupNbr="", groupString="", groupCount="")), groupList)

  lapply(seq_len(length(groupStringsWHead)), function(i, stringsToShow) {
    yPos <- 0.9-i*0.02
    xDelta <- 0.15
    grid::grid.text(stringsToShow[[i]]$groupNbr, x=0.01, y=yPos, hjust=0)
    grid::grid.text(stringsToShow[[i]]$groupString, x=0.01+xDelta, y=yPos, hjust=0)
    grid::grid.text(stringsToShow[[i]]$groupCount, x=0.01+2*xDelta, y=yPos, hjust=0)
  }, groupStringsWHead)
  
  grid::grid.text(
    "Please note that the grouping only impacts evaluation measures and visuals seen in the report, they do not impact the performed normalizations.",
    x=0.01, y=0.1, hjust=0
  )
  
  grid::pushViewport(grid::viewport(layout=currentLayout))
  printMeta(
    "Sample setup",
    pageno,
    currentjob,
    currentLayout
  )
}

#' Write page containing sample summary of intensities, missing values and 
#' MDS plot to the viewport
#' 
#' @param nr Normalyzer results object.
#' @param currentLayout Layout used for document.
#' @param pageno Current page number.
#' @return None
#' @keywords internal
plotSampleOutlierSummary <- function(nr, currentLayout, pageno) {
    
    nds <- nds(nr)
    methodlist <- normalizations(nr)
    filterED <- sampleReplicateGroups(nds)
    filterrawdata <- filterrawdata(nds)
    currentjob <- jobName(nds)
    
    tout <- rbind(c(1, 2), c(3, 4))
    graphics::layout(tout)
    graphics::par(mar=c(4, 4, 2, 1), oma=c(2, 2, 3, 2), xpd=NA)        

    datacoltotal <- colSums(filterrawdata, na.rm=TRUE)
    
    graphics::barplot(
        datacoltotal, 
        las=2, 
        main="Total intensity", 
        cex.names=0.5, 
        names.arg=substr(names(datacoltotal), 1, 10)
    )
    
    datamissingcol <- apply(filterrawdata, 2, function(x) { sum(is.na(x)) })
    graphics::barplot(
        datamissingcol, 
        las=2, 
        main="Total missing", 
        cex.names=0.5, 
        names.arg=substr(names(datamissingcol), 1, 10)
    )
    
    datastore <- methodlist[[1]]
    
    d <- stats::dist(
        scale(t(stats::na.omit(datastore)), center=TRUE, scale=TRUE)
    )
    fit <- stats::cmdscale(d, eig=TRUE, k=2)
    x <- fit$points[, 1]
    y <- fit$points[, 2]
    
    graphics::plot(x, y, type="n", main="Log2-MDS plot", xlab="", ylab="")
    graphics::text(
        fit$points[, 1], fit$points[, 2], col=filterED, labels=filterED
    )
    grid::pushViewport(grid::viewport(layout=currentLayout))
    
    printMeta(
        "Data Summary - Outlier detection", 
        pageno, 
        currentjob, 
        currentLayout
    )
}

#' Generate normalization replicate variance summary by displaying
#' CV (coefficient of variance), MAD (mean of intragroup median absolute
#' deviation) and PEV (Pooled Estimate of Variance) as mean of intragroups
#' 
#' @param nr Normalyzer results object.
#' @param currentLayout Layout used for document.
#' @param pageno Current page number.
#' @return None
#' @keywords internal
plotReplicateVariance <- function(nr, currentLayout, pageno) {
    
    nds <- nds(nr)
    methodnames <- names(normalizations(nr))
    currentjob <- jobName(nds)
    
    ner <- ner(nr)
    avgCVMem <- avgcvmem(ner)
    avgMADMem <- avgmadmem(ner)
    avgVarMem <- avgvarmem(ner)
    
    tout <- rbind(c(1, 2, 3), c(4))
    graphics::layout(tout)
    graphics::par(mar=c(2, 2, 2, 1), oma=c(2, 2, 3, 2), xpd=NA)
    
    graphics::boxplot(
        avgCVMem, 
        main="PCV - Intragroup", 
        names=c(methodnames), 
        border="red", 
        density=20, 
        cex=0.3, 
        cex.axis=0.9, 
        las=2, 
        frame.plot=FALSE
    )
    
    graphics::stripchart(
        as.data.frame(avgCVMem), 
        vertical=TRUE, 
        cex=0.4, 
        las=2, 
        pch=20, 
        add=TRUE, 
        col="darkgray"
    )
    
    graphics::boxplot(
        avgMADMem, 
        main="PMAD - Intragroup", 
        names=c(methodnames), 
        border="red", 
        density=20, 
        cex=0.3, 
        cex.axis=0.9, 
        las=2, 
        frame.plot=FALSE
    )
    
    graphics::stripchart(
        as.data.frame(avgMADMem), 
        vertical=TRUE, 
        cex=0.4, 
        las=2, 
        pch=20, 
        add=TRUE, 
        col="darkgray"
    )
    
    graphics::boxplot(
        avgVarMem, 
        main="PEV - Intragroup", 
        names=c(methodnames), 
        border="red", 
        density=20, 
        cex=0.3, 
        cex.axis=0.9, 
        las=2, 
        frame.plot=FALSE
    )
    
    graphics::stripchart(
        as.data.frame(avgVarMem), 
        vertical=TRUE, 
        cex=0.4, 
        las=2, 
        pch=20, 
        add=TRUE, 
        col="darkgray"
    )
    
    grid::pushViewport(grid::viewport(layout=currentLayout))
    printMeta("Replicate variation", pageno, currentjob, currentLayout)
}

#' Write figures displaying pooled coefficient of variance, median absolute 
#' deviation and pooled estimate of variance percentage compared to 
#' log2-transformed and stable variables plot displaying CV of stable variables
#' against pooled CV measure. The stable variables are calculated by an 
#' ANOVA comparison across sample conditions and selecting features with the 
#' least clear difference.
#'  
#' @param nr Normalyzer results object.
#' @param currentLayout Layout used for document.
#' @param pageno Current page number.
#' @return None
#' @keywords internal
plotReplicateVarAndStableVariables <- function(nr, currentLayout, pageno) {
    
    nds <- nds(nr)
    methodnames <- names(normalizations(nr))
    currentjob <- jobName(nds)
    
    ner <- ner(nr)
    lowVarFeaturesCVsPercDiff <- lowVarFeaturesCVsPercDiff(ner)
    avgcvmempdiff <- avgcvmempdiff(ner)
    avgmadmempdiff <- avgmadmempdiff(ner)
    avgvarmempdiff <- avgvarmempdiff(ner)
    
    lowBound <- min(c(avgcvmempdiff, avgmadmempdiff, avgvarmempdiff)) - 10
    highBound <- max(c(avgcvmempdiff, avgmadmempdiff, avgvarmempdiff)) + 5
    
    tout <- rbind(c(1, 2, 3), c(4, 5, 5))
    graphics::layout(tout)
    graphics::par(mar=c(6, 6, 3, 1), oma=c(2, 3, 3, 2), xpd=NA)
    
    abc <- graphics::barplot(
        avgcvmempdiff, 
        main="PCV compared to log2 ", 
        names.arg=c(methodnames), 
        border="red", 
        ylim=c(lowBound, highBound),
        # ylim=c(min(avgcvmempdiff) - 10, 
        #        max(avgmadmempdiff) + 5), 
        density=20, 
        cex=0.9, 
        cex.axis=0.7, 
        las=2, 
        xpd=FALSE
    )
    
    graphics::axis(1, at=c(0.2, (max(abc) + 0.5)), labels=FALSE, lwd.ticks=0)
    graphics::axis(1, at=abc, labels=FALSE, lwd=0, lwd.ticks=1)
    graphics::text(abc,avgcvmempdiff, labels=round(avgcvmempdiff, digits=0), 
                   pos=3, las=2)
    
    abc <- graphics::barplot(
        avgmadmempdiff, 
        main="PMAD compared to log2", 
        names.arg=c(methodnames), 
        border="red", 
        ylim=c(lowBound, highBound),
        # ylim=c(min(avgmadmempdiff) - 10, 
        #        max(avgmadmempdiff) + 5), 
        density=20, 
        cex=0.9, 
        cex.axis=0.7, 
        las=2, 
        xpd=FALSE
    )
    
    graphics::axis(1, at=c(0.2, max(abc) + 0.5), labels=FALSE, lwd.ticks=0)
    graphics::axis(1, at=abc, labels=FALSE, lwd=0, lwd.ticks=1)
    graphics::text(
        abc, 
        avgmadmempdiff, 
        labels=round(avgmadmempdiff, digits=0), 
        pos=3, 
        las=2
    )
    
    abc <- graphics::barplot(
        avgvarmempdiff, 
        main="%PEV - compared to log2", 
        names.arg=c(methodnames), 
        border="red", 
        ylim=c(lowBound, highBound),
        # ylim=c(min(avgvarmempdiff) - 10, 
        #        max(avgmadmempdiff) + 5),
        density=20, 
        cex=0.9, 
        cex.axis=0.7, 
        las=2, 
        xpd=FALSE
    )
    
    graphics::axis(
        1, at = c(0.2, max(abc) + 0.5), labels = FALSE, lwd.ticks = 0
    )
    graphics::axis(
        1, at=abc, labels=FALSE, lwd=0, lwd.ticks=1
    )
    graphics::text(
        abc, 
        avgvarmempdiff, 
        labels=round(avgvarmempdiff, digits=0), 
        pos=3, 
        las=2
    )
    
    if (!all(is.na(lowVarFeaturesCVsPercDiff))) {
        
        # if (min(avgcvmempdiff) < 0 || 
        #     max(avgcvmempdiff) > 100 || 
        #     min(lowVarFeaturesCVsPercDiff) < 0 || 
        #     max(lowVarFeaturesCVsPercDiff) > 100) {
        #     
        #     graphics::plot(
        #         avgcvmempdiff, lowVarFeaturesCVsPercDiff, pch=18, 
        #         xlim=c(0, 100), ylim=c(0, 100), 
        #         main="Stable variables plot", 
        #         xlab="PCV (intragroup) compared to Log2", 
        #         ylab="% Global CV of stable variables compared to Log2"
        #     )
        #     car::showLabels(
        #         avgcvmempdiff, 
        #         lowVarFeaturesCVsPercDiff, 
        #         labels=methodnames, 
        #         id.method="mahal", 
        #         id.cex=0.7, 
        #         id.col="black"
        #     )
        # }
        # else {
        graphics::plot(
            avgcvmempdiff, 
            lowVarFeaturesCVsPercDiff, 
            pch=18, 
            main="Stable variables plot", 
            xlab="PCV (Intragroup) compared to Log2", 
            ylab="% Global CV of stable variables compared to Log2"
        )
        car::showLabels(
            avgcvmempdiff, 
            lowVarFeaturesCVsPercDiff, 
            labels=methodnames, 
            id.method="mahal", 
            id.cex=0.7, 
            id.col="black"
        )
        # }
    }
    
    grid::pushViewport(grid::viewport(layout=currentLayout))
    printMeta(
        "Replicate variation (Relative to Log2)", 
        pageno, 
        currentjob, 
        currentLayout
    )
}

#' Plots page displaying coefficient of variance (CV) against raw intensity
#' for features across the performed normalizations
#' 
#' @param nr Normalyzer results object.
#' @param currentLayout Layout used for document.
#' @param pageno Current page number.
#' @return None
#' @keywords internal
plotCVvsIntensity <- function(nr, currentLayout, pageno) {
    
    nds <- nds(nr)
    methodnames <- names(normalizations(nr))
    methodlist <- normalizations(nr)
    currentjob <- jobName(nds)
    sampleReplicateGroups <- sampleReplicateGroups(nds)
    filterrawdata <- filterrawdata(nds)
    
    log2Mat <- methodlist[[1]]
    tempcvmat1 <- matrix(
        nrow=nrow(log2Mat), 
        ncol=length(methodlist), 
        byrow=TRUE
    )
    tempavgmat1 <- matrix(
        nrow=nrow(log2Mat), 
        ncol=length(methodlist), 
        byrow=TRUE
    )
    maxtempcv <- 0
    
    for (j in seq_along(methodlist)) {
        
        log2Mat <- methodlist[[j]]
        log2Mat <- log2Mat[, sampleReplicateGroups == min(sampleReplicateGroups)]

        for (i in seq_len(nrow(log2Mat))) {
            
            tempcv <- stats::sd(log2Mat[i, ], na.rm=TRUE) / mean(log2Mat[i, ], na.rm=TRUE)
            tempavg <- mean(filterrawdata[i, ], na.rm=TRUE)
            
            tempcvmat1[i, j] <- 100 * tempcv
            tempavgmat1[i, j] <- tempavg 
        }
        
        if (maxtempcv < max(tempcvmat1, na.rm=TRUE)) {
            maxtempcv <- max(tempcvmat1, na.rm=TRUE)
        }
    }
    
    tout <- matrix(
        seq_len((currentLayout$nrow - 2) * (currentLayout$ncol - 2)), 
        ncol=(currentLayout$ncol - 2), 
        byrow=TRUE
    )
    graphics::layout(tout)
    graphics::par(mar=c(4, 4, 2, 1), oma=c(2, 2, 3, 2), xpd=NA)
    
    for (i in seq_len(ncol(tempcvmat1))) {
        graphics::plot(
            tempavgmat1[, i], 
            tempcvmat1[, i], 
            main=methodnames[i], 
            xlab="Raw intensity", 
            ylab="CV", 
            cex=0.3, 
            ylim=c(0, maxtempcv)
        )
    }
    
    grid::pushViewport(grid::viewport(layout=currentLayout))
    printMeta("CV vs Raw Intensity plots", pageno, currentjob, currentLayout)
}

#' Produces a page containing expression vs. fold-change figures (MA plots)
#' The visualized fold is between the first sample in each group and the
#' average of the replicate to which that sample belongs
#' 
#' @param nr Normalyzer results object.
#' @param currentLayout Layout used for document.
#' @param pageno Current page number.
#'
#' @return None
#' @keywords internal
plotMA <- function(nr, currentLayout, pageno) {
    
    nds <- nds(nr)
    methodNames <- names(normalizations(nr))
    normalizedDataList <- normalizations(nr)
    currentjob <- jobName(nds)
    sampleReplicateGroups <- sampleReplicateGroups(nds)
    filterrawdata <- filterrawdata(nds)
    
    Malist <- list()
    for (i in seq_along(normalizedDataList)) {
        
        methodData <- as.matrix(normalizedDataList[[i]])
        methodDataFirstCond <- methodData[, sampleReplicateGroups == min(sampleReplicateGroups), drop=FALSE]
        firstColWoNA <- methodDataFirstCond[!is.na(methodDataFirstCond[, 1]), ]
        avgExpr <- rowMeans(firstColWoNA)
        fold <- apply(cbind(firstColWoNA[, 1], avgExpr), 1, function(x) x[1] - x[2])
        plotDf <- as.data.frame(cbind(avgExpr, fold))
        
        Malist[[i]] <- ggplot2::ggplot(plotDf, ggplot2::aes(avgExpr, fold)) + 
            ggplot2::geom_point(color="darkgray", size=0.7, na.rm=TRUE) + 
            ggplot2::labs(x=("Replicate group mean"), 
                          y=("Replicate-1 Fold Change"),
                          title=methodNames[i]) + 
            ggplot2::stat_smooth(method="loess", se=FALSE, colour="red", na.rm=TRUE, formula='y ~ x') + 
            ggplot2::geom_abline(intercept=0, slope=0, linewidth=0.3)
    } 
    
    grid::grid.newpage()
    grid::pushViewport(grid::viewport(layout=currentLayout))
    
    printPlots(Malist, "MA plots", pageno, currentjob, currentLayout)
}

#' Produces page containing scatter plot plotting the first two samples from 
#' each dataset against each other for each normalization method
#' 
#' @param nr Normalyzer results object.
#' @param currentLayout Layout used for document.
#' @param pageno Current page number.
#' @return None
#' @keywords internal
plotScatter <- function(nr, currentLayout, pageno) {
    
    nds <- nds(nr)
    methodnames <- names(normalizations(nr))
    methodlist <- normalizations(nr)
    currentjob <- jobName(nds)
    
    tout <- matrix(
        seq_len((currentLayout$nrow - 2) * (currentLayout$ncol - 2)), 
        ncol=(currentLayout$ncol - 2), 
        byrow=TRUE
    )
    graphics::layout(tout)
    graphics::par(mar=c(2, 2, 2, 1), oma=c(3, 2, 3, 2), xpd=NA)
    
    for (i in seq_along(methodlist)) {
        methodData <- methodlist[[i]]
        fit <- stats::lm(methodData[, 1]~methodData[, 2])
        graphics::plot(
            methodData[, 1], 
            methodData[, 2], 
            xlab="", 
            ylab="", 
            main=methodnames[i], 
            pch=19, 
            cex=0.2
        )
        
        graphics::legend(
            "topleft", 
            bty="n", 
            legend=paste("R2 ", format(summary(fit)$adj.r.squared, digits=2)), cex=0.7)
    }
    
    grid::pushViewport(grid::viewport(layout=currentLayout))
    printMeta("Scatterplots", pageno, currentjob, currentLayout)
}

#' Produces page showing QQ-plots for the first sample for each normalization
#' method. This plot can be used to assess whether the data follows a normal
#' distribution.
#' 
#' @param nr Normalyzer results object.
#' @param currentLayout Layout used for document.
#' @param pageno Current page number.
#' @return None
#' @keywords internal
plotQQ <- function(nr, currentLayout, pageno) {
    
    nds <- nds(nr)
    methodnames <- names(normalizations(nr))
    methodlist <- normalizations(nr)
    currentjob <- jobName(nds)
    
    qqlist <- list()
    
    for (i in seq_along(methodlist)) {  
        methodData <- methodlist[[i]]
        tempcolname <- colnames(methodData)
        qqlist[[i]] <- ggplot2::ggplot(
            data.frame(sample=methodData[, 1]), 
            ggplot2::aes(sample=sample)) + 
            ggplot2::stat_qq(na.rm=TRUE) + 
            ggplot2::stat_qq_line(na.rm=TRUE) +
            ggplot2::labs(x="", y="", title=methodnames[i])
    }
    
    grid::grid.newpage()
    grid::pushViewport(grid::viewport(layout=currentLayout))
    printPlots(qqlist, "Q-Q plots", pageno, currentjob, currentLayout)
}

#' Boxplots showing distribution of values after different normalizations
#' 
#' @param nr Normalyzer results object.
#' @param currentLayout Layout used for document.
#' @param pageno Current page number.
#' @return None
#' @keywords internal
plotBoxPlot <- function(nr, currentLayout, pageno) {
    
    nds <- nds(nr)
    methodnames <- names(normalizations(nr))
    methodlist <- normalizations(nr)
    currentjob <- jobName(nds)
    filterED <- sampleReplicateGroups(nds)
    filterrawdata <- filterrawdata(nds)
    
    tout <- matrix(
        seq_len((currentLayout$nrow - 2) * (currentLayout$ncol - 2)), 
        ncol=(currentLayout$ncol - 2), 
        byrow=TRUE
    )
    graphics::layout(tout)
    graphics::par(mar=c(2, 2, 2, 1), oma=c(3, 2, 3, 2), xpd=NA)
    mindata <- 1000
    maxdata <- 0
    
    for (i in seq_along(methodlist)) { 
        tempmin <- min(methodlist[[i]], na.rm=TRUE)
        tempmax <- max(methodlist[[i]], na.rm=TRUE)
        if (tempmin < mindata) {
            mindata <- tempmin
        }
        if (tempmax > maxdata) {
            maxdata <- tempmax
        }
    }
    
    for (i in seq_along(methodlist)) {   
        graphics::par(mar=c(5, 1, 1, 1))
        graphics::boxplot(
            methodlist[[i]], 
            cex=0.1, 
            cex.axis=0.7, 
            las=2, 
            main=methodnames[i], 
            col=(filterED), 
            outcol="lightgray", 
            ylim=c(mindata - 1, maxdata + 1), 
            names=substr(colnames(methodlist[[i]]), 1, 10))
    }
    
    grid::pushViewport(grid::viewport(layout=currentLayout))
    printMeta("Boxplots", pageno, currentjob, currentLayout)
}

#' Boxplots showing relative log expression after normalizations
#' 
#' @param nr Normalyzer results object.
#' @param currentLayout Layout used for document.
#' @param pageno Current page number.
#' @return None
#' @keywords internal
plotRLE <- function(nr, currentLayout, pageno) {
    
    nds <- nds(nr)
    methodnames <- names(normalizations(nr))
    methodlist <- normalizations(nr)
    currentjob <- jobName(nds)
    filterED <- sampleReplicateGroups(nds)
    
    tout <- matrix(
        seq_len((currentLayout$nrow - 2) * (currentLayout$ncol - 2)), 
        ncol=(currentLayout$ncol - 2), 
        byrow=TRUE
    )
    graphics::layout(tout)
    graphics::par(mar=c(2, 2, 2, 1), oma=c(3, 2, 3, 2), xpd=NA)
    
    for (i in seq_along(methodlist)) {
        deviations = methodlist[[i]] - matrixStats::rowMedians(methodlist[[i]], na.rm=TRUE)
        graphics::boxplot(
            deviations, 
            outcol="lightgray", 
            cex=0.1, 
            cex.axis=0.7, 
            las=2,
            main=methodnames[i], 
            col=(filterED), 
            names=substr(colnames(methodlist[[i]]), 
                         1, 
                         6))
    }
    
    grid::pushViewport(grid::viewport(layout=currentLayout))
    printMeta(
        "Relative Log Expression (RLE) plots", 
        pageno, 
        currentjob, 
        currentLayout
    )
}

#' Density plots showing value distributions after normalizations
#' 
#' @param nr Normalyzer results object.
#' @param currentLayout Layout used for document.
#' @param pageno Current page number.
#' @return None
#' @keywords internal
plotDensity <- function(nr, currentLayout, pageno) {
    
    nds <- nds(nr)
    methodnames <- names(normalizations(nr))
    methodlist <- normalizations(nr)
    currentjob <- jobName(nds)
    
    tout <- matrix(
        seq_len((currentLayout$nrow-2)*(currentLayout$ncol-2)), 
        ncol=(currentLayout$ncol-2), 
        byrow=TRUE
    )
    graphics::layout(tout)
    graphics::par(mar=c(3, 2, 3, 1), oma=c(3, 2, 3, 2), xpd=NA)
    
    for (i in seq_along(methodlist)) {
        
        methodData <- methodlist[[i]]
        tempd <- stats::density(methodData[, 1], na.rm=TRUE)
        graphics::plot(stats::density(methodData[, 1], na.rm=TRUE), xlab="", 
                       ylab="", ylim=c(min(tempd$y), max(tempd$y) * 1.5), 
                       main=methodnames[i], lty=2, lwd=1, col="darkgray")
        
        for (j in 2:ncol(methodData)) {
            
            graphics::lines(stats::density(methodData[, j], na.rm=TRUE), 
                            , lty=2, lwd=1, col="darkgray")
        }
    }
    
    grid::pushViewport(grid::viewport(layout=currentLayout))
    printMeta("Density plots", pageno, currentjob, currentLayout)
}

#' MDS plots showing grouping of samples after normalizations
#' 
#' @param nr Normalyzer results object.
#' @param currentLayout Layout used for document.
#' @param pageno Current page number.
#' @return None
#' @keywords internal
plotMDS <- function(nr, currentLayout, pageno) {
    
    nds <- nds(nr)
    methodnames <- names(normalizations(nr))
    methodlist <- normalizations(nr)
    currentjob <- jobName(nds)
    filterED <- sampleReplicateGroups(nds)
    
    tout <- matrix(
        seq_len((currentLayout$nrow-2)*(currentLayout$ncol-2)), 
        ncol=(currentLayout$ncol-2), 
        byrow=TRUE
    )
    graphics::layout(tout)
    graphics::par(mar=c(2, 2, 2, 1), oma=c(3, 2, 3, 2), xpd=NA)
    
    for (i in seq_along(methodlist)) {
        
        datastore <- methodlist[[i]]
        d <- stats::dist(
            scale(t(stats::na.omit(datastore)), 
                  center=TRUE, 
                  scale=TRUE)
            )
        fit <- stats::cmdscale(d, eig=TRUE, k=2)
        x <- fit$points[, 1]
        y <- fit$points[, 2]
        graphics::plot(x, y, type="n", main=methodnames[i], xlab="", ylab="")
        graphics::text(
            fit$points[, 1], 
            fit$points[, 2], 
            col=filterED, 
            labels=filterED
        )
    }
    
    grid::pushViewport(grid::viewport(layout=currentLayout))
    printMeta(
        paste("MDS plots - Built from", 
              ncol(d), 
              "variables with non-missing data", sep=" "), 
        pageno,
        currentjob, 
        currentLayout
    )
}

#' Visualize standard deviation over (expression?) for different values
#' 
#' @param nr Normalyzer results object.
#' @param currentLayout Layout used for document.
#' @param pageno Current page number.
#' @return None
#' @keywords internal
plotMeanSD <- function(nr, currentLayout, pageno) {
    
    nds <- nds(nr)
    methodnames <- names(normalizations(nr))
    methodlist <- normalizations(nr)
    currentjob <- jobName(nds)
    
    sdPlots <- list()
    
    for (i in seq_along(methodlist)) {
        
        methodData <- methodlist[[i]]
        msd <- vsn::meanSdPlot(
            methodData, 
            xlab="", 
            ylab="", 
            plot=FALSE, 
            na.rm=TRUE
        )
        
        sdPlots[[i]] <- msd$gg + ggplot2::ggtitle(methodnames[i]) +
            ggplot2::theme(legend.position="none", plot.margin=ggplot2::unit(c(1,0,0,0), "cm"))
    }
    
    grid::grid.newpage()
    grid::pushViewport(grid::viewport(layout=currentLayout))
    printPlots(sdPlots, "MeanSDplots", pageno, currentjob, currentLayout)  
}

#' Visualize within-replicates correlations
#' 
#' @param nr Normalyzer results object.
#' @param currentLayout Layout used for document.
#' @param pageno Current page number.
#' @return None
#' @keywords internal
plotCorrelation <- function(nr, currentLayout, pageno) {
    
    ner <- ner(nr)
    repCorPear <- repCorPear(ner)
    repCorSpear <- repCorSpear(ner)
    
    nds <- nds(nr)
    methodnames <- names(normalizations(nr))
    methodlist <- normalizations(nr)
    currentjob <- jobName(nds)
    filterED <- sampleReplicateGroups(nds)
    filterrawdata <- filterrawdata(nds)
    
    tout <- rbind(c(1, 2), c(3))
    graphics::layout(tout)
    graphics::par(mar=c(2, 2, 2, 1), oma=c(2, 2, 3, 2), xpd=NA)
    
    perdf <- data.frame(repCorPear)
    
    abc <- graphics::boxplot(perdf, main="Pearson correlation - Intragroup", 
                             names=c(methodnames), border="red", 
                             density=20, cex=0.3, cex.axis=0.9, las=2)
    
    graphics::stripchart(
        as.data.frame(perdf), 
        vertical=TRUE, 
        cex=0.4, 
        las=2, 
        pch=20, 
        add=TRUE, 
        col="darkgreen"
    )
    
    spedf <- data.frame(repCorSpear)
    
    abc <- graphics::boxplot(spedf, main="Spearman correlation - Intragroup", 
                             names=c(methodnames), border="red", 
                             density=20, cex=0.3, cex.axis=0.9, las=2)
    
    graphics::stripchart(
        as.data.frame(spedf), 
        vertical=TRUE, 
        cex=0.4, 
        las=2, 
        pch=20, 
        add=TRUE, 
        col="darkgreen"
    )
    grid::pushViewport(grid::viewport(layout=currentLayout))
    printMeta("Correlation plots", pageno, currentjob, currentLayout)
}

#' Visualize dendrogram grouping of samples
#' 
#' @param nr Normalyzer results object.
#' @param currentLayout Layout used for document.
#' @param pageno Current page number.
#' @return None
#' @keywords internal
plotDendrograms <- function(nr, currentLayout, pageno) {
    
    nds <- nds(nr)
    methodnames <- names(normalizations(nr))
    methodlist <- normalizations(nr)
    currentjob <- jobName(nds)
    filterED <- sampleReplicateGroups(nds)
    
    tout <- matrix(
        seq_len((currentLayout$nrow - 2) * (currentLayout$ncol - 2)), 
        ncol=(currentLayout$ncol - 2), 
        byrow=TRUE
    )
    graphics::layout(tout)
    graphics::par(mar=c(2, 2, 2, 1), oma=c(2, 2, 3, 2), xpd=NA)
    
    colorVect <- c("red", "green", "blue", "orange", "darkgray", "blueviolet", 
                    "darkslateblue", "darkviolet", "gray", "bisque4", "brown", 
                    "cadetblue4", "darkgreen", "darkcyan", "darkmagenta", 
                    "darkgoldenrod4", "coral1")
    colt <- rep(colorVect, ceiling(length(filterED) / length(colorVect)))
    
    for (j in seq_along(methodlist)) {
        
        dataMatrix <- stats::na.omit(methodlist[[j]])
        colnames(dataMatrix) <- filterED
        scaledTransposedMatrix <- scale(t(dataMatrix), center=TRUE, scale=TRUE)

        hc <- stats::hclust(stats::dist(scaledTransposedMatrix), "ave")
        
        graphics::plot(
            ape::as.phylo(hc), 
            main=methodnames[j], 
            cex=0.5, 
            tip.color=colt[filterED]
        )
        ape::axisPhylo(side=1)
    }
    
    grid::pushViewport(grid::viewport(layout=currentLayout))
    printMeta(paste("Dendrograms - Built from", ncol(scaledTransposedMatrix), 
                    "variables containing non-missing data", sep=" "), 
              pageno, currentjob, currentLayout)
}

#' Generate P-histograms for ANOVA calculated after each normalization
#' 
#' @param nr Normalyzer results object.
#' @param currentLayout Layout used for document.
#' @param pageno Current page number.
#' @return None
#' @keywords internal
plotPHist <- function(nr, currentLayout, pageno) {
    
    nds <- nds(nr)
    ner <- ner(nr)
    methodnames <- names(normalizations(nr))
    currentjob <- jobName(nds)
    anovaP <- anovaP(ner)
    histPlots <- list()
    
    for (i in seq_along(methodnames)) {
        
        anovaPVals <- anovaP[, i]
        df <- data.frame(anovaPVals=anovaPVals)
        histPlots[[i]] <- ggplot2::ggplot(df) + 
            ggplot2::geom_histogram(ggplot2::aes(anovaPVals), na.rm=TRUE, binwidth=0.01) +
            ggplot2::ggtitle(methodnames[i])
    }
    
    grid::grid.newpage()
    grid::pushViewport(grid::viewport(layout=currentLayout))
    printPlots(histPlots, "HistPlots", pageno, currentjob, currentLayout)  
}

#' NormalyzerDE pipeline entry point
#' 
#' This function is the main execution point for the normalization part of
#' the NormalyzerDE analysis pipeline. When executed it performs the following
#' steps:
#' 
#' 1: Loads the data matrix containing expression values and optional
#' annotations, as well as the design matrix containing the experimental setup
#' 2: Performs input data verification to validate that the data is in correct
#' format. This step captures many common formatting errors. It returns an
#' instance of the NormalyzerDataset class representing the unprocessed data.
#' 3: Calculate a range of normalizations for the dataset. The result is
#' provided as a NormalyzerResults object containing the resulting data matrices
#' from each normalization.
#' 4: Analyze the normalizations and generate performance measures for each
#' of the normalized datasets. This result is provided as a 
#' NormalyzerEvaluationResults object.
#' 5: Output the matrices containing the normalized datasets to files.
#' 6: Generate visualizations overviewing the performance measures and
#' write them to a PDF report.
#' 
#' @param jobName Give the current run a name.
#' @param designPath Path to file containing design matrix.
#' @param dataPath Specify an output directory for generated files.
#'  Defaults to current working directory.
#' @param experimentObj SummarizedExperiment object, can be provided as input
#'  as alternative to 'designPath' and 'dataPath'
#' @param outputDir Directory where results folder is created.
#' @param forceAllMethods Debugging function. Run all normalizations even if
#'  they aren't in the recommended range of number of values
#' @param omitLowAbundSamples Automatically remove samples with fewer non-NA
#'  values compared to threshold given by sampleAbundThres. 
#'  Will otherwise stop with error message if such sample is encountered.
#' @param sampleAbundThres Threshold for omitting low-abundant
#'  samples. Is by default set to 15.
#' @param tinyRunThres If total number of features is less than this, a limited
#'  run is performed.
#' @param requireReplicates Require multiple samples per condition to pass input 
#'  validation.
#' @param normalizeRetentionTime Perform normalizations over retention time.
#' @param plotRows Number of plot-rows in output documentation.
#' @param plotCols Number of plot-columns in output documentation.
#' @param zeroToNA Convert zero values to NA.
#' @param sampleColName Column name in design matrix containing sample IDs.
#' @param groupColName Column name in design matrix containing condition IDs.
#' @param inputFormat Type of input format.
#' @param skipAnalysis Only perform normalization steps.
#' @param quiet Omit status messages printed during run.
#' @param noLogTransform Don't log-transform the input.
#' @param writeReportAsPngs Output the evaluation report as PNG files instead of
#'  a single PDF
#' 
#' @param rtStepSizeMinutes Retention time normalization window size.
#' @param rtWindowMinCount Minimum number of datapoints in each retention-time
#'   segment.
#' @param rtWindowShifts Number of layered retention time normalized windows.
#' @param rtWindowMergeMethod Merge approach for layered retention time windows.
#' 
#' @return None
#' @export
#' @import MASS limma preprocessCore methods
#' @examples
#' \dontrun{
#' data_path <- system.file(package="NormalyzerDE", "extdata", "tiny_data.tsv")
#' design_path <- system.file(package="NormalyzerDE", "extdata", "tiny_design.tsv")
#' out_dir <- tempdir()
#' normalyzer(
#'     jobName="my_jobname", 
#'     designPath=design_path, 
#'     dataPath=data_path, 
#'     outputDir=out_dir)
#' normalyzer(
#'     "my_jobname", 
#'     designMatrix="design.tsv", 
#'     "data.tsv", 
#'     outputDir="path/to/output", 
#'     normalizeRetentionTime=TRUE, 
#'     retentionTimeWindow=2)
#' normalyzer(
#'     "my_jobname", 
#'     designMatrix="design.tsv", 
#'     "data.tsv", 
#'     outputDir="path/to/output", 
#'     inputFormat="maxquantprot")
#' }
normalyzer <- function(
        jobName,
        designPath=NULL, 
        dataPath=NULL,
        experimentObj=NULL,
        outputDir=".",
        forceAllMethods=FALSE,
        omitLowAbundSamples=FALSE,
        sampleAbundThres=5,
        tinyRunThres=50,
        requireReplicates=TRUE,
        normalizeRetentionTime=TRUE,
        plotRows=3,
        plotCols=4,
        zeroToNA=FALSE,
        sampleColName="sample",
        groupColName="group",
        inputFormat="default",
        skipAnalysis=FALSE,
        quiet=FALSE,
        noLogTransform=FALSE,
        writeReportAsPngs=FALSE,
        
        rtStepSizeMinutes=1,
        rtWindowMinCount=100,
        rtWindowShifts=1,
        rtWindowMergeMethod="mean"
    ) {
    
    if (!quiet) message("You are running version ", utils::packageVersion("NormalyzerDE"), " of NormalyzerDE")

    if (is.null(experimentObj) && (is.null(designPath) || is.null(dataPath))) {
        stop("Either options 'designPath' plus 'dataPath' or 'summarizedExp' need to be provided")
    }
    
    startTime <- Sys.time()
    
    if (!quiet) message("[Step 1/5] Load data and verify input")

    if (is.null(experimentObj)) {
        experimentObj <- setupRawDataObject(dataPath, designPath, inputFormat, 
                                            zeroToNA, sampleColName, groupColName)
    }
    else {
        verifySummarizedExperiment(experimentObj, sampleColName)
        SummarizedExperiment::colData(experimentObj)[[sampleColName]] <- 
            as.character(SummarizedExperiment::colData(experimentObj)[[sampleColName]])
        SummarizedExperiment::metadata(experimentObj) <- list(
            sample=sampleColName, 
            group=groupColName)
    }

    normObj <- getVerifiedNormalyzerObject(
        jobName=jobName,
        summarizedExp=experimentObj,
        threshold=sampleAbundThres,
        omitSamples=omitLowAbundSamples,
        requireReplicates=requireReplicates,
        quiet=quiet,
        noLogTransform=noLogTransform,
        tinyRunThres=tinyRunThres
    )
        
    jobDir <- setupJobDir(jobName, outputDir)
    if (!quiet) message(
        "[Step 1/5] Input verified, job directory prepared at:", 
        jobDir
    )
    
    if (!quiet) message("[Step 2/5] Performing normalizations")
    normalyzerResultsObject <- normMethods(
        normObj,
        forceAll=forceAllMethods,
        normalizeRetentionTime=normalizeRetentionTime,
        rtStepSizeMinutes=rtStepSizeMinutes,
        rtWindowMinCount=rtWindowMinCount,
        rtWindowShifts=rtWindowShifts,
        rtWindowMergeMethod=rtWindowMergeMethod,
        quiet=quiet,
        noLogTransform=noLogTransform
    )
    if (!quiet) message("[Step 2/5] Done!")
    
    if (!skipAnalysis) {
        if (!quiet) message("[Step 3/5] Generating evaluation measures...")
        normalyzerResultsObject <- analyzeNormalizations(normalyzerResultsObject)
        if (!quiet) message("[Step 3/5] Done!")
    }
    else {
        if (!quiet) message("[Step 3/5] skipAnalysis flag set so no analysis 
                          performed")
    }

    if (!quiet) message("[Step 4/5] Writing matrices to file")
    writeNormalizedDatasets(normalyzerResultsObject, jobDir)
    if (!quiet) message("[Step 4/5] Matrices successfully written")
    
    if (!skipAnalysis) {
        if (!quiet) message("[Step 5/5] Generating plots...")
        generatePlots(normalyzerResultsObject, jobDir, plotRows=plotRows, 
                      plotCols=plotCols, writeAsPngs=writeReportAsPngs)
        if (!quiet) message("[Step 5/5] Plots successfully generated")
    }
    else {
        if (!quiet) message("[Step 5/5] skipAnalysis flag set so no plots 
                          generated")
    }
    
    endTime <- Sys.time()
    totTime <- difftime(endTime, startTime, units="mins")
    if (!quiet) message("All done! Results are stored in: ", jobDir, 
                        ", processing time was ", round(totTime, 1), 
                        " minutes")
}

#' NormalyzerDE differential expression
#' 
#' Performs differential expression analysis on a normalization matrix.
#' This command executes a pipeline processing the data and generates an
#' annotated normalization matrix and a report containing p-value histograms
#' for each of the performed comparisons.
#' 
#' When executed, it performs the following steps:
#' 
#' 1: Read the data and the design matrices into dataframes.
#' 2: Generate an instance of the NormalyzerStatistics class representing the
#' data and their statistical comparisons.
#' 3: Optionally reduce technical replicates in both the data matrix and the 
#' design matrix
#' 4: Calculate statistical contrats between supplied groups
#' 5: Generate an annotated version of the original dataframe where columns
#' containing statistical key measures have been added
#' 6: Write the table to file
#' 7: Generate a PDF report displaying p-value histograms for each calculated
#' contrast
#' 
#' @param jobName Name of job
#' @param designPath File path to design matrix
#' @param dataPath File path to normalized matrix
#' @param experimentObj SummarizedExperiment object, can be provided as input
#'  as alternative to 'designPath' and 'dataPath'
#' @param comparisons Character vector containing target contrasts. 
#'   If comparing condA with condB, then the vector would be c("condA-condB")
#' @param outputDir Path to output directory
#' @param logTrans Log transform the input (needed if providing non-logged 
#'   input)
#' @param type Type of statistical comparison, "limma", "limma_intensity" or 
#'  "welch", where "limma_intensity" allows the prior to be fit according to
#'  intensity rather than using a flat prior
#' @param sampleCol Design matrix column header for column containing sample IDs
#' @param condCol Design matrix column header for column containing sample 
#'   conditions
#' @param batchCol Provide an optional column for inclusion of possible batch 
#'   variance in the model
#' @param techRepCol Design matrix column header for column containing technical 
#'   replicates
#' @param leastRepCount Minimum required replicate count
#' @param quiet Omit status messages printed during run
#' 
#' @param sigThres Significance threshold use for illustrating significant hits
#'   in diagnostic plots
#' @param sigThresType Type of significance threshold, "fdr" or "p". "fdr" is
#'   strongly recommended (Benjamini-Hochberg corrected p-values)
#' @param log2FoldThres Fold-size cutoff for being considered significant in
#'   diagnostic plots
#' @param writeReportAsPngs Output report as separate PNG files instead of a
#'   single PDF
#' 
#' @return None
#' @export
#' @examples 
#' data_path <- system.file(package="NormalyzerDE", "extdata", "tiny_data.tsv")
#' design_path <- system.file(package="NormalyzerDE", "extdata", "tiny_design.tsv")
#' out_dir <- tempdir()
#' normalyzerDE(
#'   jobName="my_jobname", 
#'   comparisons=c("4-5"), 
#'   designPath=design_path, 
#'   dataPath=data_path,
#'   outputDir=out_dir,
#'   condCol="group")
#' @export
normalyzerDE <- function(jobName, comparisons, designPath=NULL, dataPath=NULL, experimentObj=NULL, 
                         outputDir=".", logTrans=FALSE, type="limma", sampleCol="sample", condCol="group", 
                         batchCol=NULL, techRepCol=NULL, leastRepCount=1, quiet=FALSE, 
                         sigThres=0.1, sigThresType="fdr", log2FoldThres=0, writeReportAsPngs=FALSE) {

    if (!quiet) message("You are running version ", utils::packageVersion("NormalyzerDE"), " of NormalyzerDE")
    
    if (is.null(experimentObj) && (is.null(designPath) || is.null(dataPath))) {
        stop("Either options 'designPath' plus 'dataPath' or 'summarizedExp' need to be provided")
    }
    
    if (is.null(comparisons)) {
        stop("Argument 'comparisons' must be provided. Specify one or more comparisons as a vector.\n",
             "Example, one comparison between group 1 and 2: c('1-2')\n",
             "Example, two comparisons between groups 1 and 2, and groups 2 and 3: c('1-2', '2-3')")
    }
    
    startTime <- Sys.time()
    jobDir <- setupJobDir(jobName, outputDir)

    if (!quiet) print("Setting up statistics object")
    if (is.null(experimentObj)) {
        experimentObj <- setupRawContrastObject(dataPath, designPath, sampleCol)
    }
    else {
        verifySummarizedExperiment(experimentObj, sampleCol)
    }
    
    if (!is.null(techRepCol)) {
        if (!quiet) print("Reducing technical replicates")
        experimentObj <- reduceTechnicalReplicates(experimentObj, techRepCol, sampleCol)
    }

    nst <- NormalyzerStatistics(
        experimentObj, 
        logTrans=logTrans
    )
        
    if (!quiet) print("Calculating statistical contrasts...")
    nst <- calculateContrasts(nst, comparisons, type=type, condCol=condCol, batchCol=batchCol, leastRepCount=leastRepCount)
    if (!quiet) print("Contrast calculations done!")
    
    annotDf <- generateAnnotatedMatrix(nst)
    outPath <- paste0(jobDir, "/", jobName, "_stats.tsv")

    if (!quiet) print(paste("Writing", nrow(annotDf), "annotated rows to", outPath))
    utils::write.table(annotDf, file=outPath, sep="\t", row.names = FALSE, quote=FALSE)
    if (!quiet) print(paste("Writing statistics report"))
    generateStatsReport(nst, jobName, jobDir, sigThres, sigThresType, log2FoldThres, writeAsPngs=writeReportAsPngs)
    
    endTime <- Sys.time()
    totTime <- difftime(endTime, startTime, units="mins")
    if (!quiet) print(paste0("All done! Results are stored in: ", jobDir, ", processing time was ", round(totTime, 1), " minutes"))
}

#' Create empty directory for run
#' 
#' Creates a directory at provided path named to the jobname.
#' 
#' @param jobName Name of the run.
#' @param outputDir Path to directory where to create the output directory.
#' @return Path to newly created directory.
#' @export
#' @examples
#' setupJobDir("job_name", "path/to/outdir")
setupJobDir <- function(jobName, outputDir) {
    
    if (is.null(outputDir)) {
        jobDir <- paste(getwd(), "/", jobName[1], sep="")
    }
    else {
        jobDir <- paste(outputDir, "/", jobName[1], sep="")
    }
    createDirectory(jobDir)
    
    jobDir
}

#' Get dataframe with raw data column sorted on replicates
#' 
#' @param rawDataOnly Dataframe with unparsed input data matrix.
#' @param groups Vector containing condition levels.
#' @return rawData sorted on replicate
#' @keywords internal
getReplicateSortedData <- function(rawDataOnly, groups) {

    indexList <- getIndexList(groups)
    orderedIndices <- unlist(indexList[sort(names(indexList))])
    
    repSortDf <- vapply(
        orderedIndices,
        function(index) {
            rawDataOnly[, index]
        },
        rawDataOnly[, 1]
    )
    
    colnames(repSortDf) <- colnames(rawDataOnly)[orderedIndices]
    
    repSortDf
}

#' Create directory, or return error if already present
#' 
#' @param targetPath Path where to attempt to create directory
#' @return None
#' @keywords internal
createDirectory <- function(targetPath) {
    
    if (file.exists(targetPath)) {
        errorHandler <- "Directory already exists"
        class(errorHandler) <- "try-error"
        if (inherits(errorHandler, "try-error")) {
            return(errorHandler)
        }
        stop("Directory already exists")
    } 
    else {
        dir.create(targetPath, recursive=TRUE)
    }
}

#' Get number of seconds between two Sys.time() objects
#' 
#' @param start Start-time object
#' @param end End-time object
#' @return None
#' @keywords internal
elapsedSecondsBetweenSystimes <- function(start, end) {
    
    startSecond <- strtoi(format(start, "%s"))
    endSecond <- strtoi(format(end, "%s"))
    elapsed <- end - start
    elapsed
}

#' Return list containing vector positions of values in string
#' 
#' @param targetVector 
#' @return indexList List where key is condition level and values are indices
#'   for the condition
#' @keywords internal
getIndexList <- function(targetVector) {
    
    indexList <- list()
    uniqVals <- unique(targetVector)
    for (val in uniqVals) {
        indexList[[toString(val)]] <- which(targetVector == val)
    }
    indexList
}

#' Get contrast vector (TRUE/FALSE-values) indicating whether both at least
#' half values are present, and each sample has at least one non-NA value
#' 
#' @param dataMatrix Matrix with expression values for entities in replicate 
#'  samples.
#' @param replicateHeader Header showing how samples in matrix are replicated.
#' @param minCount Minimum number of required values present in samples.
#' @return Contrast vector
#' @keywords internal
getRowNAFilterContrast <- function(dataMatrix, 
                                   replicateHeader, 
                                   minCount=1) {
    
    replicatesHaveData <- rep(TRUE, nrow(dataMatrix))
    indexList <- getIndexList(replicateHeader)
    
    for (sampleIndex in seq_along(names(indexList))) {
        
        repVal <- names(indexList)[sampleIndex]
        cols <- indexList[[repVal]]
        
        nbrNAperReplicate <- rowSums(is.na(dataMatrix[, cols, drop=FALSE]))
        nbrReplicates <- length(cols)
        nbrNonNA <- nbrReplicates - nbrNAperReplicate
        replicatesHaveData <- (nbrNonNA >= minCount & replicatesHaveData)
    }
    
    replicatesHaveData
}

#' Generate a random test dataset with features, sample values and retention times
#' 
#' @param nSamples Number of samples
#' @param nFeatures Number of features
#' @param rtMin Minimum retention time
#' @param rtMax Maximum retention time
#' @param mean Mean value for sample intensities
#' @param sd Standard deviation for sample intensities
#' @return Test dataset
#' @export
#' @examples
#' df <- setupTestData(6, 20)
#' df <- setupTestData(6, 20, mean=15, sd=1)
#' @keywords internal
setupTestData <- function(nSamples, nFeatures, rtMin=40, rtMax=80, mean=20, sd=4) {

    featureNames <- paste0("feature_", seq(1, nFeatures))
    sampleData <- matrix(stats::rnorm(nSamples * nFeatures, mean, sd), nFeatures, nSamples)
    rtData <- stats::runif(nFeatures, rtMin, rtMax)

    df <- data.frame(feature=featureNames, 
                     RT=rtData, 
                     as.data.frame(sampleData))

    colnames(df) <- c("feature", "RT", paste0("S", seq(1, nSamples)))
    df
}

#' General function for calculating percentage difference of average column
#' means in matrix
#' 
#' @param targetMat Matrix for which column means should be compared
#' @return percDiffVector Vector with percentage difference, where first element
#'   always will be 100
#' @keywords internal
calculatePercentageAvgDiffInMat <- function(targetMat) {
    
    calculatePercDiff <- function (sampleIndex, mat) {
        mean(mat[, sampleIndex]) * 100 / mean(mat[, 1])
    }
    
    percDiffVector <- vapply(
        seq_len(ncol(targetMat)), 
        calculatePercDiff,
        0,
        mat=targetMat)
    
    percDiffVector
}

#' Filter rows with lower than given number of replicates for any condition
#' 
#' @param df Dataframe with expression data to filter
#' @param groups Condition groups header
#' @param leastRep Minimum number of replicates in each group
#'   to retain
#' @return collDesignDf Reduced design matrix
#' @keywords internal
filterLowRep <- function(df, groups, leastRep = 2) {
    
    allReplicatesHaveValuesContrast <- function(row, groups, minCount) {
        names(row) <- groups
        repCounts <- table(names(stats::na.omit(row)))
        length(repCounts) == length(unique(groups)) && min(repCounts) >= minCount || minCount == 0
    }
    
    rowMeetThresContrast <- apply(
        df, 
        1,
        allReplicatesHaveValuesContrast, 
        groups = groups, 
        minCount = leastRep)
    
    filteredDf <- df[rowMeetThresContrast, ]
    filteredDf
}

context("utils.R")

test_that("getIndexList", {
    
    out <- getIndexList(c(1,2,3))
    expect_true(all.equal(out, list("1"=1, "2"=2, "3"=3)))
    
    out <- getIndexList(c(1,2,1,2,3))
    expect_true(all.equal(out, list("1"=c(1, 3), "2"=c(2, 4), "3"=5)))
})

test_that("getRowNAFilterContrast", {

    data("example_stat_data")
    data("example_design")
    levels <- example_design$group
    out <- getRowNAFilterContrast(head(example_stat_data), levels, minCount=3)
    expect_true(
        all.equal(
            out,
            c("1"=TRUE, "2"=TRUE, "3"=FALSE, "4"=FALSE, "5"=TRUE, "6"=TRUE))
    )
})

test_that("getReplicateSortedData_constant", {

    rawMat <- as.matrix(data.frame(
        "A"=c(1, 1, 1), 
        "A"=c(2, 2, 2),
        "B"=c(3, 3, 3),
        "B"=c(4, 4, 4)
    ))
    
    groups <- c("A", "A", "B", "B")
    sortedMat <- getReplicateSortedData(rawMat, groups)
            
    expect_true(
        all.equal(
            rawMat,
            sortedMat)
    )
})

test_that("getReplicateSortedData_reordering", {
    
    rawMat <- as.matrix(data.frame(
        "B"=c(3, 3, 3),
        "A"=c(1, 1, 1), 
        "B"=c(4, 4, 4),
        "A"=c(2, 2, 2)
    ))
    
    expectedMat <- as.matrix(data.frame(
        "A"=c(1, 1, 1), 
        "A"=c(2, 2, 2),
        "B"=c(3, 3, 3),
        "B"=c(4, 4, 4)
    ))
    
    groups <- c("B", "A", "B", "A")
    sortedMat <- getReplicateSortedData(rawMat, groups)
    
    expect_true(
        all.equal(
            sortedMat,
            expectedMat)
    )
})

#' Perform normalizations on Normalyzer dataset
#' 
#' @param nds Normalyzer dataset object.
#' @param forceAll Force all methods to run despite not qualifying for thresholds.
#' @param normalizeRetentionTime Perform retention time based normalization methods.
#' @param quiet Prevent diagnostic output
#' 
#' @param rtStepSizeMinutes Retention time normalization window size.
#' @param rtWindowMinCount Minimum number of datapoints in each retention-time
#'   segment.
#' @param rtWindowShifts Number of layered retention time normalized windows.
#' @param rtWindowMergeMethod Merge approach for layered retention time windows.
#' @param noLogTransform Per default NormalyzerDE performs a log-transformation
#'   on the input data. If not needed, specify this option
#' 
#' @return Returns Normalyzer results object with performed analyzes assigned
#'  as attributes
#' @export
#' @examples
#' data(example_summarized_experiment)
#' normObj <- getVerifiedNormalyzerObject("job_name", example_summarized_experiment)
#' normResults <- normMethods(normObj)
normMethods <- function(nds, forceAll=FALSE, normalizeRetentionTime=TRUE, 
                        quiet=FALSE, rtStepSizeMinutes=1, rtWindowMinCount=100, 
                        rtWindowShifts=1, rtWindowMergeMethod="mean", noLogTransform=FALSE) {
    
    nr <- NormalyzerResults(nds=nds)
    # nr <- initializeResultsObject(nr)
    nr <- performNormalizations(
        nr, 
        forceAll=forceAll, 
        rtNorm=normalizeRetentionTime, 
        rtStepSizeMinutes=rtStepSizeMinutes, 
        rtWindowMinCount=rtWindowMinCount, 
        rtWindowShifts=rtWindowShifts, 
        rtWindowMergeMethod=rtWindowMergeMethod, 
        noLogTransform=noLogTransform,
        quiet=quiet
    )
    
    return(nr)
}

#' The normalization divides the intensity of each variable in a sample
#' with the sum of intensities of all variables in the sample and multiplies
#' with the median of sum of intensities of all variables in all samples.
#' The normalized data is then log2-transformed.
#' 
#' @param rawMatrix Target matrix to be normalized
#' @param noLogTransform Assumes no need for log transformation
#' @return Normalized and log-transformed matrix
#' @export
#' @examples
#' data(example_data_only_values_small)
#' normMatrix <- globalIntensityNormalization(example_data_only_values)
globalIntensityNormalization <- function(rawMatrix, noLogTransform=FALSE) {

    if (noLogTransform) {
        rawMatrix <- 2 ** rawMatrix
    }
        
    colSums <- colSums(rawMatrix, na.rm=TRUE)
    colSumsMedian <- stats::median(colSums)
    normMatrix <- matrix(nrow=nrow(rawMatrix), ncol=ncol(rawMatrix), byrow=TRUE)
    normFunc <- function(colIndex) { (rawMatrix[rowIndex, colIndex] / colSums[colIndex]) * colSumsMedian }
    
    for (rowIndex in seq_len(nrow(rawMatrix))) {
        normMatrix[rowIndex, ] <- vapply(seq_len(ncol(rawMatrix)), normFunc, 0)
    }
    
    normLog2Matrix <- log2(normMatrix)
    colnames(normLog2Matrix) <- colnames(rawMatrix)
    normLog2Matrix
}

#' Intensity of each variable in a given sample is divided by the median of
#' intensities of all variables in the sample and then multiplied by the mean
#' of median of sum of intensities of all variables in all samples.
#' The normalized data is then log2-transformed.
#' 
#' @param rawMatrix Target matrix to be normalized
#' @param noLogTransform Assumes no need for log transformation
#' @return Normalized and log-transformed matrix
#' @export
#' @examples
#' data(example_data_only_values_small)
#' normMatrix <- medianNormalization(example_data_only_values)
medianNormalization <- function(rawMatrix, noLogTransform=FALSE) {

    if (noLogTransform) {
        rawMatrix <- 2 ** rawMatrix
    }
    
    colMedians <- matrixStats::colMedians(rawMatrix, na.rm=TRUE)
    meanColMedian <- mean(colMedians, na.rm=TRUE)
    normMatrix <- matrix(nrow=nrow(rawMatrix), ncol=ncol(rawMatrix), byrow=TRUE)
    normFunc <- function(colIndex) { 
        (rawMatrix[rowIndex, colIndex] / colMedians[colIndex]) * meanColMedian 
    }
    
    for (rowIndex in seq_len(nrow(rawMatrix))) {
        normMatrix[rowIndex, ] <- vapply(seq_len(ncol(rawMatrix)), normFunc, 0)
    }
    
    normLog2Matrix <- log2(normMatrix)
    colnames(normLog2Matrix) <- colnames(rawMatrix)
    normLog2Matrix
}

#' Intensity of each variable in a given sample is divided by the mean of sum of
#' intensities of all variables in the sample and then multiplied by the mean
#' of sum of intensities of all variables in all samples. The normalized data
#' is then transformed to log2.
#' 
#' @param rawMatrix Target matrix to be normalized
#' @param noLogTransform Assumes no need for log transformation
#' @return Normalized and log-transformed matrix
#' @export
#' @examples
#' data(example_data_only_values_small)
#' normMatrix <- meanNormalization(example_data_only_values)
meanNormalization <- function(rawMatrix, noLogTransform=FALSE) {
    
    if (noLogTransform) {
        rawMatrix <- 2 ** rawMatrix
    }
    
    colMeans <- colMeans(rawMatrix, na.rm=TRUE)
    avgColMean <- mean(colMeans, na.rm=TRUE)
    normMatrix <- matrix(nrow=nrow(rawMatrix), ncol=ncol(rawMatrix), byrow=TRUE)
    
    normFunc <- function(colIndex) { (rawMatrix[rowIndex, colIndex] / colMeans[colIndex]) * avgColMean }
    
    for (rowIndex in seq_len(nrow(rawMatrix))) {
        normMatrix[rowIndex, ] <- vapply(seq_len(ncol(rawMatrix)), normFunc, 0)
    }
    
    normLog2Matrix <- log2(normMatrix)
    colnames(normLog2Matrix) <- colnames(rawMatrix)
    normLog2Matrix
}

#' Log2 transformed data is normalized using the function "justvsn" from the
#' VSN package. 
#' 
#' The VSN (Variance Stabilizing Normalization) attempts to transform the data 
#' in such a way that the variance remains nearly constant over the intensity 
#' spectrum
#' 
#' @param rawMatrix Target matrix to be normalized
#' @return Normalized matrix
#' @export
#' @examples
#' data(example_data_only_values_small)
#' normMatrix <- performVSNNormalization(example_data_only_values)
performVSNNormalization <- function(rawMatrix) {
    
    normMatrix <- suppressMessages(vsn::justvsn(rawMatrix, ))
    colnames(normMatrix) <- colnames(rawMatrix)
    normMatrix
}

#' Quantile normalization is performed by the function "normalize.quantiles"
#' from the package preprocessCore.
#' 
#' It makes the assumption that the data in different samples should originate
#' from an identical distribution. It does this by generating a reference
#' distribution and then scaling the other samples accordingly.
#' 
#' @param rawMatrix Target matrix to be normalized
#' @param noLogTransform Assumes no need for log transformation
#' @return Normalized matrix
#' @export
#' @examples
#' data(example_data_only_values_small)
#' normMatrix <- performQuantileNormalization(example_data_only_values)
performQuantileNormalization <- function(rawMatrix, noLogTransform=FALSE) {
    
    if (!noLogTransform) {
        log2Matrix <- log2(rawMatrix)
    }
    else {
        log2Matrix <- rawMatrix
    }
    
    normMatrix <- preprocessCore::normalize.quantiles(log2Matrix, copy=TRUE)
    colnames(normMatrix) <- colnames(rawMatrix)
    normMatrix
}

#' Median absolute deviation normalization
#' Normalization subtracts the median and divides the data by the
#' median absolute deviation (MAD).
#' 
#' @param rawMatrix Target matrix to be normalized
#' @param noLogTransform Assumes no need for log transformation
#' @return Normalized matrix
#' @export
#' @examples
#' data(example_data_only_values_small)
#' normMatrix <- performSMADNormalization(example_data_only_values)
performSMADNormalization <- function(rawMatrix, noLogTransform=FALSE) {
    
    if (!noLogTransform) {
        log2Matrix <- log2(rawMatrix)
    }
    else {
        log2Matrix <- rawMatrix
    }
    
    sampleLog2Median <- matrixStats::colMedians(log2Matrix, na.rm=TRUE)
    sampleMAD <- matrixStats::colMads(log2Matrix, na.rm=TRUE)
    madMatrix <- t(apply(log2Matrix, 1, function(row) ((row - sampleLog2Median) / sampleMAD)))
    
    madPlusMedianMatrix <- madMatrix + mean(sampleLog2Median)
    colnames(madPlusMedianMatrix) <- colnames(rawMatrix)
    
    madPlusMedianMatrix
}

#' Cyclic Loess normalization
#' 
#' Log2 transformed data is normalized by Loess method using the function
#' "normalizeCyclicLoess". Further information is available for the function
#' "normalizeCyclicLoess" in the Limma package.
#' 
#' @param rawMatrix Target matrix to be normalized
#' @param noLogTransform Assumes no need for log transformation
#' @return Normalized matrix
#' @export
#' @examples
#' data(example_data_only_values_small)
#' normMatrix <- performCyclicLoessNormalization(example_data_only_values)
performCyclicLoessNormalization <- function(rawMatrix, noLogTransform=FALSE) {
    
    if (!noLogTransform) {
        log2Matrix <- log2(rawMatrix)
    }
    else {
        log2Matrix <- rawMatrix
    }
    
    normMatrix <- limma::normalizeCyclicLoess(
        log2Matrix,
        method="fast",
        adaptive.span=FALSE
    )
    colnames(normMatrix) <- colnames(rawMatrix)
    
    normMatrix
}

#' Global linear regression normalization
#' 
#' Log2 transformed data is normalized by robust linear regression using
#' the function "rlm" from the MASS package. 
#' 
#' @param rawMatrix Target matrix to be normalized
#' @param noLogTransform Assumes no need for log transformation
#' @return Normalized matrix
#' @export
#' @examples
#' data(example_data_only_values_small)
#' normMatrix <- performGlobalRLRNormalization(example_data_only_values)
performGlobalRLRNormalization <- function(rawMatrix, noLogTransform=FALSE) {
    
    if (!noLogTransform) {
        log2Matrix <- log2(rawMatrix)
    }
    else {
        log2Matrix <- rawMatrix
    }
    
    sampleLog2Median <- matrixStats::rowMedians(log2Matrix, na.rm=TRUE)
    
    calculateRLMForCol <- function(colIndex, sampleLog2Median, log2Matrix) {
        
        lrFit <- MASS::rlm(as.matrix(log2Matrix[, colIndex])~sampleLog2Median, na.action=stats::na.exclude)
        coeffs <- lrFit$coefficients
        coefIntercept <- coeffs[1]
        coefSlope <- coeffs[2]
        globalFittedRLRCol <- (log2Matrix[, colIndex] - coefIntercept) / coefSlope
        globalFittedRLRCol
    }
    
    globalFittedRLR <- vapply(
        seq_len(ncol(log2Matrix)),
        calculateRLMForCol,
        rep(0, nrow(log2Matrix)),
        sampleLog2Median=sampleLog2Median,
        log2Matrix=log2Matrix
    )
    
    colnames(globalFittedRLR) <- colnames(rawMatrix)
    
    globalFittedRLR
}

#' Do no normalization (For debugging purposes)
#' 
#' @param rawMatrix Target matrix to be normalized
#' @return Normalized matrix
#' @keywords internal
performNoNormalization <- function(rawMatrix) {
    rawMatrix
}

context("normMethods.R")

data("example_design")
data("example_data_only_values")

# Subset the data to only look at first three conditions
test_design <- example_design[example_design$group %in% c("1", "2", "3"), ]
test_data <- example_data_only_values[, as.character(test_design$sample)]

# Remove rows with only NA-values
test_data <- test_data[rowSums(is.na(test_data)) != ncol(test_data), ]

test_that("globalIntensityNormalization", {
    
    expect_dim <- c(98, 9)
    expect_colsum <- c(
        2481.86278, 2420.0029, 2493.85063, 
        2464.60255, 2422.10488, 2473.12488, 
        2516.12334, 2432.26214, 2390.14505
    )
    
    out <- globalIntensityNormalization(test_data)
    out_dim <- dim(out)
    out_colsum <- round(colSums(out, na.rm=TRUE), 5)
    
    expect_true(
        all.equal(
            expect_dim,
            out_dim
        )
    )
    
    expect_true(
        all(
            expect_colsum == out_colsum
        )
    )
})

test_that("medianNormalization", {
    
    expect_dim <- c(98, 9)
    expect_colsum <- c(
        2476.44406, 2416.74862, 2514.76548, 
        2480.75296, 2421.68804, 2460.60737, 
        2515.09438, 2403.50676, 2382.08166
    )
    
    out <- medianNormalization(test_data)
    out_dim <- dim(out)
    out_colsum <- round(colSums(out, na.rm=TRUE), 5)
    
    expect_true(
        all.equal(
            expect_dim,
            out_dim
        )
    )
    
    expect_true(
        all(
            expect_colsum == out_colsum
        )
    )
})

test_that("meanNormalization", {
    
    expect_dim <- c(98, 9)
    expect_colsum <- c(
        2480.02877, 2415.35575, 2493.44705, 
        2462.76854, 2417.45773, 2471.29087, 
        2517.16589, 2427.61499, 2384.11555
    )
    
    out <- meanNormalization(test_data)
    out_dim <- dim(out)
    out_colsum <- round(colSums(out, na.rm=TRUE), 5)
    
    expect_true(
        all.equal(
            expect_dim,
            out_dim
        )
    )
    
    expect_true(
        all(
            expect_colsum == out_colsum
        )
    )
})

test_that("performVSNNormalization", {
    
    expect_dim <- c(98, 9)
    expect_colsum <- c(
        2470.56447, 2407.87325, 2496.00488, 
        2461.14702, 2421.64088, 2477.71483, 
        2520.74195, 2413.61712, 2397.58554
    )
    
    out <- performVSNNormalization(test_data)
    out_dim <- dim(out)
    out_colsum <- round(colSums(out, na.rm=TRUE), 5)
    
    expect_true(
        all.equal(
            expect_dim,
            out_dim
        )
    )
    
    expect_true(
        all(
            expect_colsum == out_colsum
        )
    )
})

test_that("performQuantileNormalization", {
    
    expect_dim <- c(98, 9)
    expect_colsum <- c(
        2469.98486, 2415.69332, 2497.12467, 
        2469.98486, 2415.69332, 2469.98486, 
        2524.27466, 2415.69332, 2388.56021
    )
    
    out <- performQuantileNormalization(test_data)
    out_dim <- dim(out)
    out_colsum <- round(colSums(out, na.rm=TRUE), 5)
    
    expect_true(
        all.equal(
            expect_dim,
            out_dim
        )
    )
    
    expect_true(
        all(
            expect_colsum == out_colsum
        )
    )
})

test_that("performSMADNormalization", {
    
    expect_dim <- c(98, 9)
    expect_colsum <- c(
        2474.668, 2417.49851, 2507.3658, 
        2476.69406, 2419.97969, 2466.17639, 
        2520.90251, 2411.10946, 2386.40614
    )
    
    out <- performSMADNormalization(test_data)
    out_dim <- dim(out)
    out_colsum <- round(colSums(out, na.rm=TRUE), 5)
    
    expect_true(
        all.equal(
            expect_dim,
            out_dim
        )
    )
    
    expect_true(
        all(
            expect_colsum == out_colsum
        )
    )
})

test_that("performCyclicLoessNormalization", {
    
    expect_dim <- c(98, 9)
    expect_colsum <- c(
        2478.31304, 2410.73689, 2494.80824, 
        2454.38212, 2418.7555, 2471.09178, 
        2510.40413, 2402.29372, 2389.24695
    )
    
    out <- performCyclicLoessNormalization(test_data)
    out_dim <- dim(out)
    out_colsum <- round(colSums(out, na.rm=TRUE), 5)
    
    expect_true(
        all.equal(
            expect_dim,
            out_dim
        )
    )
    
    expect_true(
        all(
            expect_colsum == out_colsum
        )
    )
})

test_that("performGlobalRLRNormalization", {
    
    expect_dim <- c(98, 9)
    expect_colsum <- c(
        2481.46193, 2416.7949, 2499.26657, 
        2461.07399, 2424.5646, 2474.91485, 
        2517.3423, 2408.12205, 2393.68206
    )
    
    out <- performGlobalRLRNormalization(test_data)
    out_dim <- dim(out)
    out_colsum <- round(colSums(out, na.rm=TRUE), 5)
    
    expect_true(
        all.equal(
            expect_dim,
            out_dim
        )
    )
    
    expect_true(
        all(
            expect_colsum == out_colsum
        )
   )
})

test_that("performNoNormalization", {
    
    expect_dim <- c(98, 9)
    expect_colsum <- c(
        35539172278, 35144510958, 36832468728, 
        36954497268, 38286168846, 38114456004, 
        37366276438, 30257069174, 37091357660
    )
    
    out <- performNoNormalization(test_data)
    out_dim <- dim(out)
    out_colsum <- round(colSums(out, na.rm=TRUE), 5)
    
    expect_true(
        all.equal(
            expect_dim,
            out_dim
        )
    )
    
    expect_true(
        all(
            expect_colsum == out_colsum
        )
    )
})

#' Represents raw input data together with basic annotation information
#' 
#' Takes a job name, a data matrix, a design matrix as well as specification
#' of the group and sample columns in the design matrix. Provides the
#' basic representation of a dataset in the NormalyzerDE normalization part.
#' 
#' @slot jobName Name of the job represented by the dataset.
#' @slot rawData Matrix with raw values.
#' @slot sampleNameCol Name column for sample.
#' @slot groupNameCol Name column for groups.
#' @slot designMatrix Data frame containing design.
#' @slot sampleNames Vector containing sample names.
#' @slot filterrawdata Reduced raw data matrix where low abundance rows are 
#'  removed
#' @slot sampleReplicateGroups Vector with sample replicate information
#' @slot samplesGroupsWithReplicates Vector with replicated sample replicate information
#' @slot annotationValues Annotation part of original dataframe.
#' @slot retentionTimes Vector of retention time values.
#' @slot singleReplicateRun Conditional whether run is single replicate.
NormalyzerDataset <- setClass("NormalyzerDataset",
                              representation(
                                  
                                  jobName = "character",
                                  rawData = "matrix",
                                  
                                  sampleNameCol = "character",
                                  groupNameCol = "character",
                                  
                                  designMatrix = "data.frame",
                                  sampleNames = "character",

                                  filterrawdata = "matrix",
                                  sampleReplicateGroups = "numeric",
                                  samplesGroupsWithReplicates = "numeric",
                                  
                                  annotationValues = "matrix",
                                  retentionTimes = "numeric",
                                  
                                  isTinyRun = "logical",
                                  singleReplicateRun = "logical"
                              ))

#' Constructor for NormalyzerDataset
#' 
#' @param jobName Name of the NormalyzerDE processing run
#' @param designMatrix Matrix containing sample conditions
#' @param rawData Matrix containing raw input data
#' @param annotationData Matrix containing annotation information for each
#'   input feature. Is expected to contain the same number of rows as the data
#'   but can contain any number of features
#' @param sampleNameCol Name of column in design matrix containing sample
#'   information
#' @param groupNameCol Name of column in design matrix containing condition
#'   information
#' @param tinyRunThres If fewer features than this is present in the input
#'   a limited run will be performed to avoid some steps requiring a more
#'   extensive number of features.
#' @param quiet If set to TRUE no information messages will be printed
#' @return nds Generated NormalyzerDataset instance
#' @keywords internal
NormalyzerDataset <- function(jobName, designMatrix, rawData, annotationData,
                                sampleNameCol, groupNameCol, tinyRunThres=50, quiet=FALSE) {
              
              sampleReplicateGroups <- as.numeric(
                  as.factor(designMatrix[, groupNameCol]))
              samplesGroupsWithReplicates <- as.numeric(
                  names(table(sampleReplicateGroups)[table(sampleReplicateGroups) > 1]))
              sampleNames <- as.character(designMatrix[, sampleNameCol])
              
              # annotationValues <- rawData[, !(colnames(rawData) %in% sampleNames), drop=FALSE]
              filterrawdata <- rawData[, sampleNames]
              class(filterrawdata) <- "numeric"
              
              if (nrow(filterrawdata) < tinyRunThres) {
                  isTinyRun <- TRUE
                  if (!quiet) {
                      message("Number of features (", nrow(filterrawdata), 
                              ") is less than the theshold for normal run (", 
                              tinyRunThres, "), a limited run is performed ",
                              "\nExact settings can be adjusted using the 'tinyRunThreshold' setting")
                  }
              }
              else {
                  isTinyRun <- FALSE
              }
              
              nds <- new(
                  "NormalyzerDataset",
                  jobName=jobName,
                  designMatrix=designMatrix,
                  rawData=rawData,
                  sampleNameCol=sampleNameCol,
                  groupNameCol=groupNameCol,
                  filterrawdata=filterrawdata,
                  sampleReplicateGroups=sampleReplicateGroups,
                  samplesGroupsWithReplicates=samplesGroupsWithReplicates,
                  annotationValues=annotationData,
                  isTinyRun=isTinyRun
              )
              
              rtColumn <- getRTColumn(annotationData, quiet=quiet)
              if (!is.null(rtColumn)) {
                  retentionTimes(nds) <- rtColumn
              }
              
              singleReplicateRun(nds) <- checkSingleReplicateRun(nds, quiet=quiet)
              
              nds
          }

setGeneric("jobName", function(object) { standardGeneric("jobName") })
setMethod("jobName", signature(object="NormalyzerDataset"), 
          function(object) { slot(object, "jobName") })

setGeneric("rawData", function(object) { standardGeneric("rawData") })
setMethod("rawData", signature(object="NormalyzerDataset"), 
          function(object) { slot(object, "rawData") })

setGeneric("sampleNameCol", function(object) { standardGeneric("sampleNameCol") })
setMethod("sampleNameCol", signature(object="NormalyzerDataset"), 
          function(object) { slot(object, "sampleNameCol") })

setGeneric("groupNameCol", function(object) { standardGeneric("groupNameCol") })
setMethod("groupNameCol", signature(object="NormalyzerDataset"), 
          function(object) { slot(object, "groupNameCol") })

setGeneric("designMatrix", function(object) { standardGeneric("designMatrix") })
setMethod("designMatrix", signature(object="NormalyzerDataset"), 
          function(object) { slot(object, "designMatrix") })

setGeneric("sampleNames", function(object) { standardGeneric("sampleNames") })
setMethod("sampleNames", signature(object="NormalyzerDataset"), 
          function(object) { slot(object, "sampleNames") })

setGeneric("filterrawdata", function(object) { standardGeneric("filterrawdata") })
setMethod("filterrawdata", signature(object="NormalyzerDataset"), 
          function(object) { slot(object, "filterrawdata") })

setGeneric("sampleReplicateGroups", function(object) { standardGeneric("sampleReplicateGroups") })
setMethod("sampleReplicateGroups", signature(object="NormalyzerDataset"), 
          function(object) { slot(object, "sampleReplicateGroups") })

setGeneric("samplesGroupsWithReplicates", function(object) { standardGeneric("samplesGroupsWithReplicates") })
setMethod("samplesGroupsWithReplicates", signature(object="NormalyzerDataset"), 
          function(object) { slot(object, "samplesGroupsWithReplicates") })

setGeneric("annotationValues", function(object) { standardGeneric("annotationValues") })
setMethod("annotationValues", signature(object="NormalyzerDataset"), 
          function(object) { slot(object, "annotationValues") })

setGeneric("retentionTimes", function(object) { standardGeneric("retentionTimes") })
setMethod("retentionTimes", signature(object="NormalyzerDataset"), 
          function(object) { slot(object, "retentionTimes") })
setGeneric("retentionTimes<-", function(object, value) { standardGeneric("retentionTimes<-") })
setReplaceMethod("retentionTimes", signature(object="NormalyzerDataset"), 
                 function(object, value) { 
                     slot(object, "retentionTimes") <- value
                     validObject(object)
                     object
                 })

setGeneric("isTinyRun", function(object) { standardGeneric("isTinyRun") })
setMethod("isTinyRun", signature(object="NormalyzerDataset"), 
          function(object) { slot(object, "isTinyRun") })

setGeneric("singleReplicateRun", function(object) { standardGeneric("singleReplicateRun") })
setMethod("singleReplicateRun", signature(object="NormalyzerDataset"), 
          function(object) { slot(object, "singleReplicateRun") })
setGeneric("singleReplicateRun<-", function(object, value) { standardGeneric("singleReplicateRun<-") })
setReplaceMethod("singleReplicateRun", signature(object="NormalyzerDataset"), 
                 function(object, value) { 
                     slot(object, "singleReplicateRun") <- value
                     validObject(object)
                     object
                 })

#' Detect single replicate, and assign related logical
#'
#' @param nds Normalyzer dataset
#' @param quiet Don't give non-error output
#' @return bool on whether sample contains only one sample group
#' @rdname detectSingleReplicate
#' @keywords internal
setGeneric(name="detectSingleReplicate", 
           function(nds, quiet) standardGeneric("detectSingleReplicate"))

#' @rdname detectSingleReplicate
setMethod("detectSingleReplicate", "NormalyzerDataset",
          function(nds, quiet=FALSE) {
              
              headerCounts <- table(sampleReplicateGroups(nds))
              nonReplicatedSamples <- names(headerCounts[headerCounts == 1])
              
              if (length(nonReplicatedSamples) > 0) {
                  singleReplicateRun <- TRUE
                  
                  if (!quiet) {
                      message("Non replicated samples in dataset: ",
                              paste(nonReplicatedSamples, collapse=" "),
                              " performing limited single-replicate run\n")
                  }
              }
              else {
                  singleReplicateRun <- FALSE
              }
              
              singleReplicateRun
          }
)

#' Detect single sample group 
#'
#' @param nds Normalyzer dataset.
#' @param quiet Only print error messages
#' @return None
#' @rdname detectSingletonSample
#' @keywords internal
setGeneric(name="detectSingletonSample", 
           function(nds, quiet) standardGeneric("detectSingletonSample"))

#' @rdname detectSingletonSample
setMethod("detectSingletonSample", "NormalyzerDataset",
          function(nds, quiet=FALSE) {
              
              groups <- sampleReplicateGroups(nds)
              distinctSamples <- unique(groups)
              singletonSamplePresent <- FALSE
              
              if (length(distinctSamples) == 1) {
                  singletonSamplePresent <- TRUE
                  if (!quiet) {
                      message("Only one replicate group present. ",
                              paste("Group: ", distinctSamples[1]),
                              " Proceeding with limited processing\n")
                  }
              }
              else if (length(distinctSamples) == 0) {
                  stop("No sample replicate groups found - Aborting.\n")
              }
              
              singletonSamplePresent
          }
)

setGeneric(name="checkSingleReplicateRun", 
           function(nds, quiet) standardGeneric("checkSingleReplicateRun"))
setMethod("checkSingleReplicateRun", "NormalyzerDataset",
          function(nds, quiet=FALSE) {
              
              singleReplicateRun <- detectSingleReplicate(nds, quiet=quiet)
              singletonSamplePresent <- detectSingletonSample(nds, quiet=quiet)
              
              if (singleReplicateRun || singletonSamplePresent) {
                  singleReplicateRun <- TRUE
              }
              else {
                  singleReplicateRun <- FALSE
              }
              singleReplicateRun
          })

getRTColumn <- function(annotData, quiet=FALSE) {
    rtColumns <- grep("\\bRT\\b", colnames(annotData), ignore.case = TRUE)
    
    if (length(rtColumns) > 1) {
        errorMessage <- paste(
            "Only able to handle single RT column (name containing RT standing by itself) ",
            "Please change name or remove unwanted RT columns\n")
        stop(errorMessage)
    }
    else if (length(rtColumns) == 1) {
        
        if (!quiet) message("RT annotation column found (", rtColumns, ")")
        
        rtValues <- annotData[, rtColumns]
        retentionTimes <- as.numeric(rtValues)
        return(retentionTimes)
    }
    else {
        if (!quiet) message("No RT column found, skipping RT processing\n")
        return(NULL)
    }
}

#' Generate PDF grid page filling it with provided list of plots
#' 
#' @param plotlist List of target plots to display.
#' @param plotname List of names corresponding to the provided plot list.
#' @param pageno Current page number.
#' @param jobname Name of ongoing job.
#' @param currentLayout Custom viewport layout.
#' @return None
#' @keywords internal
printPlots <- function(plotlist, plotname, pageno, jobname, currentLayout) {
    
    ncol <- currentLayout$ncol
    nrow <- currentLayout$nrow
    
    gp <- grid::gpar(fontsize=11, fontfamily="Helvetica", col="black", fontface="bold")
    gpfill <- grid::gpar(fill="gray90", lwd=0, lty=0)
    
    drawRectangles(nrow, ncol, gpfill, gp)
    
    grid::grid.text(plotname, 
                    vp=grid::viewport(layout.pos.row=1, layout.pos.col=1),
                    just=c("left","center"), gp=grid::gpar(fontface="bold", fontsize=14, 
                                                           fontfamily="Helvetica", col="darkBlue"))
    grid::grid.text("NormalyzerDE Report", 
                    vp=grid::viewport(layout.pos.row=1, layout.pos.col=ncol),
                    just=c("right", "center"), gp=gp)
    grid::grid.text(paste("Page ", pageno, sep=""), 
                    vp=grid::viewport(layout.pos.row=nrow, layout.pos.col=ncol), just=c("right", "center"), gp=gp)
    grid::grid.text(paste("Project: ", jobname, sep=""),
                    vp=grid::viewport(layout.pos.row=nrow, layout.pos.col=1), just=c("left", "center"), gp=gp)
    
    gridRows <- nrow - 2
    gridCols <- ncol - 2
    gridSize <- gridRows * gridCols

    row <- 2  # Start value
    col <- 2  # Start value

    posCounter <- 0
    for (i in seq_along(plotlist)) {
        
        if (gridSize == 1 || (i != 1 && i %% gridSize == 1)) {
             grid::grid.newpage()
            grid::pushViewport(grid::viewport(layout=currentLayout))
            
            grid::grid.rect(vp=grid::viewport(layout.pos.row=row, layout.pos.col=col), gp=gpfill)
            posCounter <- 0
        }
        
        posCounter <- posCounter + 1
        
        row <- (posCounter - 1) %/% gridCols + 2
        col <- (posCounter - 1) %% gridCols + 2
        
        print(plotlist[[i]], vp=grid::viewport(layout.pos.row=row, layout.pos.col=col))
    }
}

drawRectangles <- function(nrow, ncol, gpfill, gp) {
    
    for (pos in seq_len((ncol - 1) * (nrow - 1))) {
        if (pos <= ncol) row <- 1
        else row <- nrow
        
        col <- (pos - 1) %% ncol + 1
        grid::grid.rect(vp=grid::viewport(layout.pos.row=row, layout.pos.col=col), gp=gpfill)
    }
}

## ----setup, echo=FALSE, results="hide"----------------------------------------
# knitr::opts_chunk$set(tidy=FALSE, cache=TRUE, dev="png", message=FALSE,
# error=FALSE, warning=TRUE)

## -----------------------------------------------------------------------------
library(NormalyzerDE)
outDir <- tempdir()
designFp <- system.file(package="NormalyzerDE", "extdata", "tiny_design.tsv")
dataFp <- system.file(package="NormalyzerDE", "extdata", "tiny_data.tsv")
normalyzer(jobName="vignette_run", designPath=designFp, dataPath=dataFp, 
           outputDir=outDir)

## -----------------------------------------------------------------------------
normMatrixPath <- paste(outDir, "vignette_run/CycLoess-normalized.txt", sep="/")
normalyzerDE("vignette_run", 
             comparisons=c("4-5"),
             designPath=designFp, 
             dataPath=normMatrixPath,
             outputDir=outDir, 
             condCol="group")

## -----------------------------------------------------------------------------
dataMatrix <- read.table(dataFp, sep="\t", header = TRUE)
designMatrix <- read.table(designFp, sep="\t", header = TRUE)
designMatrix$sample <- as.character(designMatrix$sample)
dataOnly <- dataMatrix[, designMatrix$sample]
annotOnly <- dataMatrix[, !(colnames(dataMatrix) %in% designMatrix$sample)]

sumExpObj <- SummarizedExperiment::SummarizedExperiment(
    as.matrix(dataOnly),
    colData=designMatrix,
    rowData=annotOnly
)

normalyzer(jobName="sumExpRun", experimentObj = sumExpObj, outputDir=outDir)

## -----------------------------------------------------------------------------
fullDf <- read.csv(dataFp, sep="\t")
designDf <- read.csv(designFp, sep="\t")
head(fullDf, 1)
head(designDf, 1)

## -----------------------------------------------------------------------------
sampleNames <- as.character(designDf$sample)
typeof(sampleNames)

## -----------------------------------------------------------------------------
dataMat <- as.matrix(fullDf[, sampleNames])
retentionTimes <- fullDf$Average.RT

head(dataMat, 1)

## -----------------------------------------------------------------------------
typeof(dataMat)

print("Rows and columns of data")
dim(dataMat)

print("Number of retention times")
length(retentionTimes)

## -----------------------------------------------------------------------------
performCyclicLoessNormalization <- function(rawMatrix) {
    log2Matrix <- log2(rawMatrix)
    normMatrix <- limma::normalizeCyclicLoess(log2Matrix, method="fast")
    colnames(normMatrix) <- colnames(rawMatrix)
    normMatrix
}

## -----------------------------------------------------------------------------
rtNormMat <- getRTNormalizedMatrix(dataMat, 
                                   retentionTimes, 
                                   performCyclicLoessNormalization, 
                                   stepSizeMinutes=1, 
                                   windowMinCount=100)

## -----------------------------------------------------------------------------
globalNormMat <- performCyclicLoessNormalization(dataMat)
dim(rtNormMat)
dim(globalNormMat)
head(rtNormMat, 1)
head(globalNormMat, 1)

## -----------------------------------------------------------------------------
layeredRtNormMat <- getSmoothedRTNormalizedMatrix(
    dataMat, 
    retentionTimes, 
    performCyclicLoessNormalization, 
    stepSizeMinutes=1, 
    windowMinCount=100, 
    windowShifts=3, 
    mergeMethod="mean")

dim(layeredRtNormMat)
head(layeredRtNormMat, 1)

## -----------------------------------------------------------------------------
jobName <- "vignette_run"
experimentObj <- setupRawDataObject(dataFp, designFp, "default", TRUE, "sample", "group")
normObj <- getVerifiedNormalyzerObject(jobName, experimentObj)

## -----------------------------------------------------------------------------
normResults <- normMethods(normObj)

## -----------------------------------------------------------------------------
normResultsWithEval <- analyzeNormalizations(normResults)

## -----------------------------------------------------------------------------
jobDir <- setupJobDir("vignette_run", tempdir())
writeNormalizedDatasets(normResultsWithEval, jobDir)

## -----------------------------------------------------------------------------
generatePlots(normResultsWithEval, jobDir)

## -----------------------------------------------------------------------------
bestNormMatPath <- paste(jobDir, "RT-Loess-normalized.txt", sep="/")
experimentObj <- setupRawContrastObject(bestNormMatPath, designFp, "sample")
nst <- NormalyzerStatistics(experimentObj, logTrans=FALSE)

## -----------------------------------------------------------------------------
comparisons <- c("4-5")
nst <- calculateContrasts(nst, comparisons, condCol="group", leastRepCount=2)

## -----------------------------------------------------------------------------
annotDf <- generateAnnotatedMatrix(nst)
utils::write.table(annotDf, file=paste(jobDir, "stat_table.tsv", sep="/"))
generateStatsReport(nst, "Vignette stats", jobDir)

## -----------------------------------------------------------------------------
sessionInfo()

proteiosToNormalyzer <- function(proteiosFp, sep="\t") {
    
    valuesDf <- as.matrix(
        utils::read.table(proteiosFp, 
        header=FALSE, 
        sep="\t", 
        stringsAsFactors=FALSE,
        quote=""))
    fullMat <- as.matrix(valuesDf)
    fullMat
}

maxQuantToNormalyzer <- function(maxQuantFp, protLevel, sep="\t") {
    
    pepIntensityPattern <- "Intensity\\."
    
    if (!protLevel) {
        annotCols <- c("Sequence", "Mass", "Proteins", "Leading.razor.protein", "PEP", "Charges")
        matrixType <- "peptide.txt"
    }
    else {
        annotCols <- c("Protein.IDs", "Majority.protein.IDs", "Fasta.headers")
        matrixType <- "proteinGroups.txt"
    }
    
    fullDf <- utils::read.csv(maxQuantFp, sep=sep, stringsAsFactors=FALSE, comment.char="", quote="", header=TRUE)
    cnames <- colnames(fullDf)
    intensityCols <- cnames[grepl(pepIntensityPattern, cnames)]
    
    headerNames <- c(annotCols, intensityCols)
    headerNamesTrimmed <- gsub("Intensity.", "", headerNames)

    if (!(all(annotCols %in% colnames(fullDf)))) {
        stop(
            "Performing MaxQuant processing for the ", matrixType, " matrix\n", 
            "Didn't find all of the following expected columns (with spaces instead of dots): \n",
            paste(annotCols, collapse=", "),
            "\nColumns in the input data:\n",
            paste(colnames(fullDf), collapse=", ")
        )
    }
    
    valuesDf <- fullDf[, c(annotCols, intensityCols)]
    rawDf <- rbind(headerNamesTrimmed, valuesDf)
    rawMat <- as.matrix(rawDf)
    
    return(rawMat)
}

#' Write normalization matrices to file
#' 
#' Outputs each of the normalized datasets to the specified directory.
#' 
#' @param nr Results object.
#' @param jobdir Path to output directory.
#' @param includePairwiseComparisons Include p-values for pairwise comparisons.
#' @param includeCvCol Include CV column in output.
#' @param includeAnovaP Include ANOVA p-value in output.
#' @param normSuffix String used to name output together with normalization names.
#' @param rawdataName Name of output raw data file.
#' @return None
#' @export
#' @examples
#' data(example_summarized_experiment)
#' normObj <- getVerifiedNormalyzerObject("job_name", example_summarized_experiment)
#' normResults <- normMethods(normObj)
#' normResultsWithEval <- analyzeNormalizations(normResults)
#' outputDir <- tempdir()
#' writeNormalizedDatasets(normResultsWithEval, outputDir)
writeNormalizedDatasets <- function(nr, jobdir, includePairwiseComparisons=FALSE, 
                                    includeCvCol=FALSE, includeAnovaP=FALSE,
                                    normSuffix="-normalized.txt",
                                    rawdataName="submitted_rawdata.txt") {
    
    nds <- nds(nr)
    ner <- ner(nr)

    methodlist <- normalizations(nr)
    methodnames <- names(methodlist)
    annotationColumns <- annotationValues(nds)
    if (ncol(annotationColumns) == 0) {
        annotationColumns <- NULL
    }
    
    for (sampleIndex in seq_along(methodnames)) {
        
        currentMethod <- methodnames[sampleIndex]
        filePath <- paste(jobdir, "/", currentMethod, normSuffix, sep="")
        outputTable <- cbind(annotationColumns, methodlist[[sampleIndex]])

        if (includeAnovaP) {
            anovaP <- anovaP(ner)[,sampleIndex]
            
            if (nrow(outputTable) != length(anovaP)) {
                stop("Table row count: ", nrow(outputTable), 
                     " must match p-value vector length for anova: ", 
                     length(anovaP))
            }
            
            outputTable <- cbind(outputTable, anovaP=anovaP)
        }
        
        if (includePairwiseComparisons) {
            
            ner <- ner(nr)
            for (comp in names(pairwiseCompsP(ner))) {
                
                compColP <- pairwiseCompsP(ner)[[comp]][, sampleIndex]
                compColFdr <- pairwiseCompsFdr(ner)[[comp]][, sampleIndex]
                
                newColnames <- c(
                    colnames(outputTable), 
                    paste("comp", comp, "p", sep="_"), 
                    paste("comp", comp, "fdr", sep="_")
                )
                outputTable <- cbind(outputTable, compColP, compColFdr)
                colnames(outputTable) <- newColnames
            }
        }
        
        if (includeCvCol) {
            cvCol <- featureCVPerMethod(ner)[, sampleIndex]
            outputTable <- cbind(outputTable, CV=cvCol)
        }

        utils::write.table(
            outputTable, file=filePath, sep="\t", row.names=FALSE, quote=FALSE)
    }
    
    rawFilePath <- paste(jobdir, "/", rawdataName, sep="")
    rawOutputTable <- cbind(annotationColumns, filterrawdata(nds))
    
    utils::write.table(
        rawOutputTable, 
        file=rawFilePath, 
        sep="\t", 
        row.names=FALSE, 
        quote=FALSE
    )
}

#' Small example dataset used to demonstrate code consistency in testing and
#' as dummy data in the vignette.
#' 
#' @format A data frame containing annotation and expression data
#' @keywords internal
"example_data"

#' Same data as in "example_data", but omitting the annotation meaning that it
#' only contains the expression data.
#'  
#' @format A data frame containing expression data
#' @keywords internal
"example_data_only_values"

#' Design matrix corresponding to the small example datasets.
#' 
#' @format A design matrix corresponding to the dataset "example_data"
#' @keywords internal
"example_design"

#' Same data as in "example_data", but normalized and ready for statistical
#' processing.
#' 
#' @format A normalized data frame ready for statistical processing
#' @keywords internal
"example_stat_data"

#' SummarizedExperiment object prepared with design-matrix, data-matrix
#' and annotation columns loaded for raw data
#' 
#' @format An instance of the class SummarizedExperiment
#' @keywords internal
"example_summarized_experiment"

#' SummarizedExperiment object prepared with design-matrix, data-matrix
#' and annotation columns for normalized data
#' 
#' @format An instance of the class SummarizedExperiment with stats data
#' @keywords internal
"example_stat_summarized_experiment"

#' Full raw NormalyzerDE matrix used for internal testing
#' 
#' @format A data table ready for analysis in NormalyzerDE
#' @keywords internal
"example_wide_data"

#' Design matrix belonging together with example_wide_data. Used for
#' internal testing.
#' 
#' @format A design table ready for analysis in NormalyzerDE
#' @keywords internal
"example_wide_design"

#' Perform RT-segmented normalization by performing the supplied normalization
#' over retention-time sliced data
#' 
#' The function orders the retention times and steps through them using the
#' supplied step size (in minutes). If smaller than a fixed lower boundary 
#' the window is expanded to ensure a minimum amount of data in each 
#' normalization step. An offset can be specified which can be used to perform
#' multiple RT-segmentations with partial overlapping windows.
#' 
#' @param rawMatrix Target matrix to be normalized
#' @param retentionTimes Vector of retention times corresponding to rawMatrix
#' @param normMethod The normalization method to apply to the time windows
#' @param stepSizeMinutes Size of windows to be normalized
#' @param windowMinCount Minimum number of values for window to not be expanded.
#' @param offset Whether time window should shifted half step size
#' @param noLogTransform Don't log-transform the data
#' @return Normalized matrix
#' @export
#' @examples
#' data(example_data_small)
#' data(example_design_small)
#' data(example_data_only_values)
#' dataMat <- example_data_only_values
#' retentionTimes <- as.numeric(example_data[, "Average.RT"])
#' performCyclicLoessNormalization <- function(rawMatrix) {
#'     log2Matrix <- log2(rawMatrix)
#'     normMatrix <- limma::normalizeCyclicLoess(log2Matrix, method="fast")
#'     colnames(normMatrix) <- colnames(rawMatrix)
#'     normMatrix
#' }
#' rtNormMat <- getRTNormalizedMatrix(dataMat, retentionTimes, 
#' performCyclicLoessNormalization, stepSizeMinutes=1, windowMinCount=100)
getRTNormalizedMatrix <- function(rawMatrix, retentionTimes, normMethod, 
                                  stepSizeMinutes=1, windowMinCount=100, 
                                  offset=0, noLogTransform=FALSE) {
    
    # Key variables:
    #    targetSliceIndices 
    #        Indices in raw matrix for rows falling within retention time 
    #        interval
    #    normalizationSliceIndices 
    #        Rows used for normalization, can include wider interval than 
    #        target slice if target slice isn't containing enough data
    #    indicesOfInterest
    #        Target indices within the normalization slice window
    
    if (!is(rawMatrix, "matrix")) {
        stop("Type of rawMatrix is expected to be matrix, received: ", 
             class(rawMatrix)
        )
    }
    
    sortedRT <- sort(retentionTimes)
    
    startVal <- min(retentionTimes, na.rm=TRUE)
    endVal <- max(retentionTimes, na.rm=TRUE)
    rowNumbers <- c()

    if (offset) {
        startVal <- startVal - stepSizeMinutes * offset
    }
    
    processedRows <- matrix(, ncol=ncol(rawMatrix), nrow=0)

    for (windowStart in seq(startVal, endVal, stepSizeMinutes)) {
        
        windowEnd <- windowStart + stepSizeMinutes
        normalizationStartRT <- windowStart
        normalizationEndRT <- windowEnd
        targetSliceIndices <- which(
            retentionTimes >= windowStart & retentionTimes < windowEnd
        )
        
        if (length(targetSliceIndices) == 0) {
            next
        }
        else if (length(targetSliceIndices) < windowMinCount) {
            
            normalizationRange <- getWidenedRTRange(
                windowStart, 
                windowEnd, 
                windowMinCount, 
                retentionTimes
            )
            normalizationStartRT <- normalizationRange[1]
            normalizationEndRT <- normalizationRange[2]
        }
        
        normalizationSliceIndices <- which(
            retentionTimes >= normalizationStartRT & 
                retentionTimes <= normalizationEndRT
        )
        normalizationRows <- rawMatrix[normalizationSliceIndices,, drop=FALSE]
        
        if (noLogTransform) {
            processedNormalizationRows <- normMethod(normalizationRows, noLogTransform=noLogTransform)
        }
        else {
            processedNormalizationRows <- normMethod(normalizationRows)
        }
        
        rownames(processedNormalizationRows) <- rownames(normalizationRows)
        
        if (length(targetSliceIndices) < length(normalizationSliceIndices)) {
            
            indicesOfInterest <- which(
                normalizationSliceIndices %in% targetSliceIndices
            )
            normalizedTargetRows <- processedNormalizationRows[indicesOfInterest,]
            # normalizedTargetRows <- processedNormalizationRows[indicesOfInterest,, drop=FALSE]
        }
        else {
            normalizedTargetRows <- processedNormalizationRows
        }
        
        rowNumbers <- c(rowNumbers, targetSliceIndices)
        processedRows <- rbind(processedRows, normalizedTargetRows)
    }
    
    orderedProcessedRows <- processedRows[order(rowNumbers), ]
    orderedProcessedRows
}

#' Pick datapoints before and after window until a minimum number is reached
#' Expects the start and end retention times to match actual retention times
#' present in the data
#' 
#' @param rtStart Original retention time start point
#' @param rtEnd Original retention time end point
#' @param minimumDatapoints Required number of datapoints to fulfill
#' @param retentionTimes Vector with all retention times
#' @return Vector with start and end of new RT range
#' @keywords internal
getWidenedRTRange <- function(rtStart, rtEnd, minimumDatapoints, retentionTimes,
                              allowTooWideData=FALSE) {
    
    sortedRts <- sort(retentionTimes)
    currentRTSlice <- sortedRts[sortedRts >= rtStart & sortedRts < rtEnd] 
    
    if (length(currentRTSlice) == 0) {
        stop("Selected retention time slice doesn't contain any data")
    }
    
    if (length(currentRTSlice) > minimumDatapoints) {
        if (allowTooWideData) {
            return(c(rtStart, rtEnd))
        }
        else {
            stop("Number of datapoints exceed minimum, add option ",
                 "'allowTooWideData' to process anyway")
        }
    }
    
    # Get single element if multiple with exactly same RT
    startIndex <- utils::head(which(sortedRts == min(currentRTSlice)), 1)
    endIndex <- utils::tail(which(sortedRts == max(currentRTSlice)), 1)

    currentCount <- length(currentRTSlice)
    remainingCount <- minimumDatapoints - currentCount
    
    pickBefore <- floor(remainingCount / 2)
    pickAfter <- ceiling(remainingCount / 2)
    
    totalBefore <- length(sortedRts[sortedRts < rtStart])
    totalAfter <- length(sortedRts[sortedRts >= rtEnd])
    
    stopifnot(remainingCount == pickBefore + pickAfter)
    stopifnot(totalBefore + totalAfter + length(currentRTSlice) == length(retentionTimes))
    
    if (pickBefore > totalBefore && pickAfter > totalAfter) {
        stop("Not enough values in dataset to do RT normalization with current 
             minimum datapoints setting - Please adjust settings")
    }
    else if (pickBefore > totalBefore) {
        diff <- pickBefore - totalBefore
        pickAfter <- pickAfter + diff
        pickBefore <- pickBefore - diff
    }
    else if (pickAfter > totalAfter) {
        diff <- pickAfter - totalAfter
        pickBefore <- pickBefore + diff
        pickAfter <- pickAfter - diff
    }
    
    newStartRtIndex <- startIndex - pickBefore
    newEndRtIndex <- endIndex + pickAfter
    
    if (newEndRtIndex - newStartRtIndex + 1 > length(sortedRts)) {
        stop("Requested minimum window size (", 
             newEndRtIndex - newStartRtIndex + 1, 
             ") exceeds total number of datapoints (", 
             length(sortedRts),
             ")")
    }
    
    widenedSlice <- sortedRts[newStartRtIndex:newEndRtIndex]

    if (length(widenedSlice) != minimumDatapoints) {
        stop("Widened slice should equal to minimum number of data points")
    }
    
    stopifnot(length(widenedSlice) == minimumDatapoints)

    widenedStartRt <- min(widenedSlice)
    widenedEndRt <- max(widenedSlice)
            
    c(widenedStartRt, widenedEndRt)
}


#' Generate multiple RT time-window normalized matrices where one is shifted.
#' Merge them using a specified method (mean or median) and return the result.
#' 
#' Uses the function getRTNormalizedMatrix to generate multiple normalized
#' matrices which are shifted respective to each other and finally merged into
#' a single matrix. This could potentially reduce effect of fluctuations
#' within individual windows.
#' 
#' @param rawMatrix Target matrix to be normalized
#' @param retentionTimes Vector of retention times corresponding to rawMatrix
#' @param normMethod The normalization method to apply to the time windows
#' @param stepSizeMinutes Size of windows to be normalized
#' @param windowShifts Number of frame shifts.
#' @param windowMinCount Minimum number of features within window.
#' @param mergeMethod Layer merging approach. Mean or median.
#' @param noLogTransform Don't log transform the input
#' @return Normalized matrix
#' @export
#' @examples
#' 
#' data(example_data_small)
#' data(example_data_only_values)
#' data(example_design_small)
#' retentionTimes <- as.numeric(example_data[, "Average.RT"])
#' dataMat <- example_data_only_values
#' performCyclicLoessNormalization <- function(rawMatrix) {
#'     log2Matrix <- log2(rawMatrix)
#'     normMatrix <- limma::normalizeCyclicLoess(log2Matrix, method="fast")
#'     colnames(normMatrix) <- colnames(rawMatrix)
#'     normMatrix
#' }
#' rtNormMat <- getSmoothedRTNormalizedMatrix(dataMat, retentionTimes, 
#'     performCyclicLoessNormalization, stepSizeMinutes=1, windowMinCount=100, 
#'     windowShifts=2, mergeMethod="median")
getSmoothedRTNormalizedMatrix <- function(
    rawMatrix, retentionTimes, normMethod, stepSizeMinutes, 
    windowShifts=2, windowMinCount=100, mergeMethod="mean", noLogTransform=FALSE) {
    
    matrices <- list()
    
    for (i in seq_len(windowShifts)) {
        
        fracShift <- (i - 1) * 1 / windowShifts
        matrices[[i]] <- getRTNormalizedMatrix(
            rawMatrix, 
            retentionTimes, 
            normMethod,
            stepSizeMinutes=stepSizeMinutes, 
            windowMinCount=windowMinCount, 
            offset=fracShift,
            noLogTransform=noLogTransform
        )
    }

    if (mergeMethod == "mean") {
        combinedMatrices <- getCombinedMatrix(matrices, mean)
    }
    else if (mergeMethod == "median") {
        combinedMatrices <- getCombinedMatrix(matrices, stats::median)
    }
    else {
        stop("Unknown merge method: ", mergeMethod)
    }
    
    colnames(combinedMatrices) <- colnames(rawMatrix)
    combinedMatrices
}

#' Merge multiple dataframes using provided function
#' 
#' @param mList List containing dataframes of same shape
#' @param combFunc Function performing elementwise merge of matrices
#' @return combinedMatrix A single dataframe with combined data
#' @keywords internal
getCombinedMatrix <- function(mList, combFunc) {
    
    matrixCount <- length(mList)
    rows <- nrow(mList[[1]])
    cols <- ncol(mList[[1]])
    mLength <- rows * cols
    combinedMatrix <- matrix(0, nrow=rows, ncol=cols)
    
    # Iterate over each element position
    for (i in seq_len(mLength)) {
        elemVals <- vapply(mList, function(mat) {mat[[i]]}, 0)
        targetVal <- combFunc(elemVals)
        combinedMatrix[i] <- targetVal
    }
    
    colnames(combinedMatrix) <- colnames(mList[[1]])
    combinedMatrix
}

context("higherOrderNormMethods.R")

data("example_design")
data("example_data_only_values")
data("example_data")
data("example_stat_data")

# Subset the data to only look at first three conditions
test_design <- example_design[example_design$group %in% c("1", "2", "3"), ]
test_data <- example_data_only_values[, as.character(test_design$sample)]

# Remove rows with only NA-values
non_na_rows <- which(rowSums(is.na(test_data)) != ncol(test_data))
test_data <- test_data[non_na_rows, ]
real_rt_vals <- round(example_stat_data$Average.RT, 5)[non_na_rows]


test_that("getRTNormalizedMatrix_global_intensity", {
    
    expect_dim <- c(98, 9)
    expect_colsum <- c(
        2482.77553, 2420.75378, 2497.33571, 
        2464.09042, 2424.41947, 2472.58042, 
        2515.43627, 2419.77311, 2393.76557
    )
    
    out <- getRTNormalizedMatrix(
        rawMatrix=as.matrix(test_data), 
        retentionTimes=real_rt_vals, 
        normMethod=globalIntensityNormalization, 
        stepSizeMinutes=1, 
        windowMinCount=10)
    
    out_dim <- dim(out)
    out_colsum <- round(colSums(out, na.rm=TRUE), 5)
    
    expect_true(
        all.equal(
            expect_dim,
            out_dim
        )
    )
    
    expect_true(
        all(
            expect_colsum == out_colsum
        )
    )
})

test_that("getRTNormalizedMatrix_loess", {
    
    expect_dim <- c(98, 9)
    expect_colsum <- c(
        2472.8547, 2412.4723, 2496.2553, 
        2458.90508, 2419.86546, 2471.89521, 
        2515.73799, 2413.87385, 2392.8269
    )
    
    out <- getRTNormalizedMatrix(
        rawMatrix=as.matrix(test_data), 
        retentionTimes=real_rt_vals, 
        normMethod=performCyclicLoessNormalization, 
        stepSizeMinutes=1, 
        windowMinCount=10)
    
    out_dim <- dim(out)
    out_colsum <- round(colSums(out, na.rm=TRUE), 5)
    
    expect_true(
        all.equal(
            expect_dim,
            out_dim
        )
    )
    
    expect_true(
        all(
            expect_colsum == out_colsum
        )
    )
})

test_that("getSmoothedRTNormalizedMatrix_global_intensity", {
    
    expect_dim <- c(98, 9)
    expect_colsum <- c(
        2483.01579, 2420.61195, 2497.10932, 
        2463.82244, 2424.36461, 2472.49291, 
        2515.79797, 2421.82197, 2393.95915
    )
    
    out <- getSmoothedRTNormalizedMatrix(
        rawMatrix=as.matrix(test_data), 
        retentionTimes=real_rt_vals, 
        normMethod=globalIntensityNormalization, 
        stepSizeMinutes=1, 
        windowMinCount=10,
        mergeMethod="median",
        windowShifts=4
        )
    
    out_dim <- dim(out)
    out_colsum <- round(colSums(out, na.rm=TRUE), 5)
    
    expect_true(
        all.equal(
            expect_dim,
            out_dim
        )
    )
    
    expect_true(
        all(
            expect_colsum == out_colsum
        )
    )
})

test_that("getSmoothedRTNormalizedMatrix_loess", {
    
    expect_dim <- c(98, 9)
    expect_colsum <- c(
        2473.03148, 2412.86092, 2496.37835, 
        2458.6256, 2419.13168, 2471.7457, 
        2514.70862, 2413.37041, 2392.00512
    )
    
    out <- getSmoothedRTNormalizedMatrix(
        rawMatrix=as.matrix(test_data), 
        retentionTimes=real_rt_vals, 
        normMethod=performCyclicLoessNormalization, 
        stepSizeMinutes=1, 
        windowMinCount=10,
        mergeMethod="mean",
        windowShifts=3
        )
    
    out_dim <- dim(out)
    out_colsum <- round(colSums(out, na.rm=TRUE), 5)
    
    expect_true(
        all.equal(
            expect_dim,
            out_dim
        )
    )
    
    expect_true(
        all(
            expect_colsum == out_colsum
        )
    )
})

test_that("getWidenedRTRange_natural_numbers", {
    
    rt_times <- seq_len(10)
    
    expect_true(
        all.equal(
            getWidenedRTRange(4, 5, 2, rt_times),
            c(4, 5)
        )
    )
    
    expect_true(
        all.equal(
            getWidenedRTRange(4, 5, 3, rt_times),
            c(3, 5)
        )
    )
    
    expect_true(
        all.equal(
            getWidenedRTRange(4, 5, 4, rt_times),
            c(3, 6)
        )
    )
})

test_that("getWidenedRTRange_float_numbers", {
    
    rt_times <- c(1.1, 2.2, 3.3, 4.2, 5.4, 6.7, 7.1, 8.2, 9.2, 10.3)
    
    expect_true(
        all.equal(
            getWidenedRTRange(2.2, 3.3, 2, rt_times),
            c(2.2, 3.3)
        )
    )
    
    expect_true(
        all.equal(
            getWidenedRTRange(2.2, 3.3, 6, rt_times),
            c(1.1, 6.7)
        )
    )
    
    expect_true(
        all.equal(
            getWidenedRTRange(2.2, 3.3, 10, rt_times),
            c(1.1, 10.3)
        )
    )
    
    expect_error(
        getWidenedRTRange(2.2, 3.3, 11, rt_times)
    )
})

test_that("getWidenedRTRange_real_numbers", {
    
    expect_true(
        all.equal(
            getWidenedRTRange(34.497, 45.081, 20, round(real_rt_vals, 3)),
            c(30.937, 45.335)
        )
    )
    
    expect_true(
        all.equal(
            getWidenedRTRange(34.497, 45.082, 30, round(real_rt_vals, 3)),
            c(28.211, 48.111)
        )
    )
    
    expect_true(
        all.equal(
            getWidenedRTRange(34.49672, 45.08198, 98, round(real_rt_vals, 3)),
            c(9.024, 128.702)
        )
    )
    
    expect_error(
        getWidenedRTRange(34.49672, 45.08198, 100, round(real_rt_vals, 3))
    )
    
    expect_error(
        getWidenedRTRange(34.49672, 45.08198, 10, round(real_rt_vals, 3))
    )
    
    expect_true(
        all.equal(
            getWidenedRTRange(34.497, 45.082, 10, round(real_rt_vals, 3), allowTooWideData=TRUE),
            c(34.497, 45.082)
        )
    )
})

test_that("getCombinedMatrix_minimal_symmetric", {
    
    expect_out <- as.matrix(data.frame(
        a=c(1.5, 1.5), 
        b=c(1.5, 1.5)))
    
    df1 <- as.matrix(data.frame(a=c(1, 1), b=c(1, 1)))
    df2 <- as.matrix(data.frame(a=c(2, 2), b=c(2, 2)))
    l <- list(df1, df2)
    out <- getCombinedMatrix(l, mean)
    
    expect_true(
        all.equal(
            expect_out,
            out
        )
    )
})

test_that("getCombinedMatrix_minimal_asymmetric", {
    
    expect_out <- as.matrix(data.frame(
        a=c(0.5, 1, 1.5), 
        b=c(2, 2.5, 3)))
    
    df1 <- as.matrix(data.frame(a=c(0, 0, 0), b=c(0, 0, 0)))
    df2 <- as.matrix(data.frame(a=c(1, 2, 3), b=c(4, 5, 6)))
    l <- list(df1, df2)
    out <- getCombinedMatrix(l, mean)
    
    expect_true(
        all.equal(
            expect_out,
            out
        )
    )
})

test_that("getCombinedMatrix_three_layer_mean", {
    
    expect_out <- as.matrix(data.frame(
        a=c(1.00000, 3.33333, 5.66667), 
        b=c(8.00000, 10.33333, 12.66667)))
    
    df1 <- as.matrix(data.frame(a=c(1, 2, 3), b=c(4, 5, 6)))
    df2 <- as.matrix(data.frame(a=c(1, 3, 5), b=c(7, 9, 11)))
    df3 <- as.matrix(data.frame(a=c(1, 5, 9), b=c(13, 17, 21)))
    
    l <- list()
    l[[1]] <- df1
    l[[2]] <- df2
    l[[3]] <- df3
    
    out <- round(getCombinedMatrix(l, mean), 5)
    
    expect_true(
        all.equal(
            expect_out,
            out
        )
    )
})

test_that("getCombinedMatrix_three_layer_median", {
    
    expect_out <- as.matrix(data.frame(a=c(1, 3, 5), b=c(7, 9, 11)))
    
    df1 <- as.matrix(data.frame(a=c(1, 2, 3), b=c(4, 5, 6)))
    df2 <- as.matrix(data.frame(a=c(1, 3, 5), b=c(7, 9, 11)))
    df3 <- as.matrix(data.frame(a=c(1, 5, 9), b=c(13, 17, 21)))
    
    l <- list()
    l[[1]] <- df1
    l[[2]] <- df2
    l[[3]] <- df3
    
    out <- getCombinedMatrix(l, median)
    
    expect_true(
        all.equal(
            expect_out,
            out
        )
    )
})

#' Remove technical replicates from data and design
#' 
#' Collapses sample values into their average. If only one value is present
#' due to NA-values in other technical replicates, then that value is used.
#' 
#' Takes a SummarizedExperiment where the data is present as the assay
#' and the colData contains the design conditions. In the design conditions
#' there should be one column with the technical replicate groups and one
#' column containing the sample names
#' 
#' @param se Summarized experiment where the assay contains the data to be
#'   reduced, and the colData the data frame
#' @param techRepColName Technical replicates column name in colData
#' @param sampleColName Sample names column name in colData
#' @return reducedSe Summarized experiment with reduced data
#' @export
#' @examples
#' testData <- as.matrix(data.frame(
#'     c(1,1,1), 
#'     c(1,2,1), 
#'     c(7,7,7), 
#'     c(7,9,7)))
#' colnames(testData) <- c("a1", "a2", "b1", "b2")
#' designDf <- data.frame(
#'     sample=c("a1", "a2", "b1", "b2"), 
#'     techrep=c("a", "a", "b", "b"))
#' se <- SummarizedExperiment::SummarizedExperiment(
#'     assay=testData,
#'     colData=designDf
#' )
#' statObj <- reduceTechnicalReplicates(se, "techrep", "sample")
reduceTechnicalReplicates <- function(se, techRepColName, sampleColName) {
    
    designDf <- data.frame(SummarizedExperiment::colData(se))
    dataMat <- SummarizedExperiment::assay(se)
        
    techRepGroups <- as.character(designDf[[techRepColName]])
    indices <- lapply(unique(techRepGroups), function(i) { which(techRepGroups %in% i) })
    
    # Reduce design
    collDesignMatList <- lapply(indices, function(inds) { designDf[inds[1],] })
    collDesignDf <- do.call(rbind.data.frame, collDesignMatList)
    collDesignDf <- droplevels(collDesignDf)
    rownames(collDesignDf) <- seq_len(nrow(collDesignDf))
    
    collapsedNames <- unlist(lapply(indices, function(inds) { paste(designDf[[sampleColName]][inds], collapse=".") }))
    collDesignDf[[sampleColName]] <- collapsedNames
    
    # Reduce data
    collDataMat <- as.matrix(data.frame(lapply(
        indices, 
        function(inds) {rowMeans(dataMat[, inds, drop=FALSE], na.rm = TRUE)})))
    colnames(collDataMat) <- collapsedNames
    
    reducedSe <- SummarizedExperiment::SummarizedExperiment(
        assay=collDataMat,
        colData=collDesignDf,
        rowData=SummarizedExperiment::rowData(se)
    )
    reducedSe
}

#' Generate an annotated data frame from statistics object
#' 
#' Extracts key values (p-value, adjusted p-value, log2-fold change and
#' average expression values) from an NormalyzerStatistics instance
#' and appends these to the annotation- and data-matrices
#' 
#' @param nst NormalyzerDE statistics object.
#' @param prefixSep Character string for separating the prefix names from the statistics suffix
#' @param compLabels Vector containing strings to use as prefix for statistical comparisons
#' @return outDf Annotated statistics matrix
#' @export
#' @examples
#' data(example_stat_summarized_experiment)
#' statObj <- NormalyzerStatistics(example_stat_summarized_experiment)
#' statObj <- calculateContrasts(statObj, comparisons=c("1-2", "2-3"), condCol="group", type="limma")
#' annotDf <- generateAnnotatedMatrix(statObj)
generateAnnotatedMatrix <- function(nst, prefixSep="_", compLabels=NULL) {
    
    if (is.null(compLabels)) {
        contrastBase <- names(pairwiseCompsP(nst))
    }
    else {
        if (length(compLabels) != length(pairwiseCompsP(nst))) {
            stop(sprintf(
                "Number of assigned labels compLabels (found %s) needs to be same as number of contrasts (found %s)",
                length(compLabels), length(pairwiseCompsP(nst))
            ))
        }
        contrastBase <- compLabels
    }
        
    pairwiseHead <- paste(contrastBase, "PValue", sep=prefixSep)
    pMat <- data.frame(pairwiseCompsP(nst))
    colnames(pMat) <- pairwiseHead
    
    pairwiseHeadFdr <- paste(contrastBase, "AdjPVal", sep=prefixSep)
    fdrMat <- data.frame(pairwiseCompsFdr(nst))
    colnames(fdrMat) <- pairwiseHeadFdr
    
    pairwiseHeadFold <- paste(contrastBase, "log2FoldChange", sep=prefixSep)
    foldMat <- data.frame(pairwiseCompsFold(nst))
    colnames(foldMat) <- pairwiseHeadFold
    
    aveMat <- data.frame(pairwiseCompsAve(nst))
    outDf <- data.frame(cbind(annotMat(nst), pMat, fdrMat, foldMat, 
                   featureAvg=aveMat[, 1], dataMat(nst)), check.names=FALSE)
    outDf
}

#' Generate full output report plot document. Plots p-value histograms
#' for each contrast in the NormalyzerStatistics instance and writes these
#' to a PDF report.
#' 
#' @param nst NormalyzerDE statistics object.
#' @param jobName Name of processing run.
#' @param jobDir Path to output directory.
#' @param sigThres Significance threshold for indicating as significant
#' @param sigThresType Type of significance threshold (FDR or p)
#' @param log2FoldThres log2 fold-change required for being counted as significant
#' @param plotRows Number of plot rows.
#' @param plotCols Number of plot columns.
#' @param writeAsPngs Output the report as separate PNG files instead of a 
#'   single PDF file
#' @return None
#' @export
#' @examples
#' data(example_stat_summarized_experiment)
#' statObj <- NormalyzerStatistics(example_stat_summarized_experiment)
#' statObj <- calculateContrasts(statObj, comparisons=c("1-2", "2-3"), 
#'   condCol="group", type="limma")
#' outputDir <- tempdir()
#' generateStatsReport(statObj, "jobName", outputDir)
generateStatsReport <- function(nst, jobName, jobDir,
                                sigThres=0.1, sigThresType="fdr", log2FoldThres=0,
                                plotRows=3, plotCols=4, writeAsPngs=FALSE) {
    
    nrows <- plotRows + 1
    ncols <- plotCols + 1
    
    currentLayout <- grid::grid.layout(
        nrow=nrows, 
        ncol=ncols,
        heights=c(0.1, rep(3 / (nrows - 2), (nrows - 2)), 0.1), 
        widths=c(0.1, rep(4 / (ncols - 2), (ncols - 2)), 0.1), 
        default.units=c("null", "null"))
    
    currentFont <- "Helvetica"
    setupPlotting(jobName, jobDir, "Norm-stats-report")
    
    pageNo <- 1
    nextPageNo <- function(iter=TRUE) {
        if (iter) {
            pageNo <<- pageNo + 1
        }
        pageNo
    }
    
    if (writeAsPngs) {
        pngDir <- sprintf("%s/de_pngs", jobDir)
        dir.create(pngDir)
    }
    else {
        pngDir <- NULL
    }
    
    writePage(
        plotFrontPage, jobDir, writeAsPngs, 
        sprintf("%s/%s_%s", pngDir, nextPageNo(iter=FALSE), "front.png"), 
        jobName, currentFont)
    
    writePage(
        plotContrastPHists, jobDir, writeAsPngs, 
        sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "pvalue_histograms.png"), 
        nst, jobName, currentLayout, nextPageNo(iter=FALSE))
    
    writePage(
        plotSigScatter, jobDir, writeAsPngs, 
        sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "ma_scatter.png"), 
        nst, jobName, currentLayout, nextPageNo(iter=FALSE), type="MA", 
        sigThres=sigThres, sigThresType=sigThresType, log2FoldThres=log2FoldThres
    )
    
    writePage(
        plotSigScatter, jobDir, writeAsPngs, 
        sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "volcano_scatter.png"), 
        nst, jobName, currentLayout, nextPageNo(iter=FALSE), type="Volcano",
        sigThres=sigThres, sigThresType=sigThresType, log2FoldThres=log2FoldThres
    )
    
    writePage(
        plotContrastPCA, jobDir, writeAsPngs, 
        sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "contrast_pca_pc12.png"), 
        nst, jobName, currentLayout, nextPageNo(iter=FALSE), pcs=c(1,2))
    
    writePage(
        plotContrastPCA, jobDir, writeAsPngs, 
        sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "contrast_pca_pc34.png"), 
        nst, jobName, currentLayout, nextPageNo(iter=FALSE), pcs=c(3,4))
    
    if (length(comparisons(nst)) > 1) {
        
        writePage(
            plotComparisonVenns, jobDir, writeAsPngs, 
            sprintf("%s/%s_%s", pngDir, nextPageNo(iter=TRUE), "venn_comparison.png"), 
            nst, jobName, currentLayout, nextPageNo(iter=FALSE),
            sigThres=sigThres, sigThresType=sigThresType, log2FoldThres=log2FoldThres
        )
    }
    
    grDevices::dev.off()
}

getSigs <- function(nst, sigThresType, sigThres, log2FoldThres) {

    contrastPLists <- pairwiseCompsP(nst)
    contrastFDRLists <- pairwiseCompsFdr(nst)
    contrastFoldLists <- pairwiseCompsFold(nst)

    sigLists <- list()
    
    getSigList <- function(contrast, contrastPLists, contrastFDRLists, contrastFoldLists) {
        
        pVals <- contrastPLists[[contrast]]
        fdrVals <- contrastFDRLists[[contrast]]
        fold <- contrastFoldLists[[contrast]]
        
        if (sigThresType == "fdr") {
            statSig <- (fdrVals < sigThres)
        }
        else if (sigThresType == "p") {
            statSig <- (pVals < sigThres)
        }
        else {
            stop("Unknown significance threshold type: ", sigThresType)
        }
        
        if (log2FoldThres != 0) {
            foldSig <- abs(fold) >= log2FoldThres
            sig <- statSig & foldSig
        }
        else {
            sig <- statSig
        }
        
        sig
    }

    sigLists <- lapply(names(contrastPLists), getSigList, 
           contrastPLists=contrastPLists,
           contrastFDRLists=contrastFDRLists,
           contrastFoldLists=contrastFoldLists
    )

    sigLists
}

#' If multiple comparisons - Show overlap in Venn diagrams
#' 
#' @param nst NormalyzerDE statistics object.
#' @param jobName Name of processing run.
#' @param currentLayout Layout used for document.
#' @param pageno Current page number.
#' @param sigThres Cutoff value for significance theshold
#' @param sigThresType Type of significance cutoff
#' @param log2FoldThres Log2-fold based cutoff threshold
#' @param maxContrasts Maximum contrasts to show pairwise comparisons for
#' @return None
#' @keywords internal
plotComparisonVenns <- function(nst, jobName, currentLayout, pageno,
                                sigThres=0.1, sigThresType="fdr", 
                                log2FoldThres=0, maxContrasts=4) {

    contrastSig <- getSigs(nst, sigThresType, sigThres, log2FoldThres)
    contrasts <- comparisons(nst)
    compCount <- min(length(contrasts), maxContrasts)
    folds <- pairwiseCompsFold(nst)
    plts <- list()
    index <- 0
    
    for (indOut in seq_len(compCount-1)) {
        for (indIn in (indOut+1):compCount) {

            # Credit goes to:
            # https://scriptsandstatistics.wordpress.com/2018/04/26/how-to-plot-venn-diagrams-using-r-ggplot2-and-ggforce/
            
            sigInds <- as.data.frame(contrastSig[c(indIn, indOut)])
            
            bothSigInds <- which(apply(sigInds, 1, all))
            contraCount <- length(which(sign(folds[[indIn]][bothSigInds]) != sign(folds[[indOut]][bothSigInds])))
            
            vdc <- limma::vennCounts(sigInds)
            class(vdc) <- 'matrix'
            df.vdc <- data.frame(vdc[-1, ])
            df.vdc <- rbind(df.vdc, c(-1, -1, contraCount))
            df.vdc$x <- c(-1.5, 1.5, 0, 0)
            df.vdc$y <- c(0, 0, 0, -0.3)
            df.vdc$label <- df.vdc$Counts
            df.vdc$label[3] <- paste0(df.vdc$label[3] - df.vdc$label[4], " same")
            df.vdc$label[4] <- paste0(df.vdc$label[4], " contra")
            
            df.venn <- data.frame(x = c(-0.5, 0.5),
                                  y = c(0, 0),
                                  labels = c(contrasts[indOut], contrasts[indIn]))
            
            plt <- ggplot2::ggplot(data=df.venn) +
                ggforce::geom_circle(
                    ggplot2::aes_string(x0 = "x", y0 = "y", r = 1.5, fill = "labels"), 
                    alpha = .3, size = 0.5, colour = 'darkgray') +
                ggplot2::coord_fixed() +
                ggplot2::theme_void() +
                ggplot2::theme(legend.position = 'bottom', legend.direction='vertical') +
                ggplot2::scale_fill_manual(values = c('cornflowerblue', 'firebrick',  'gold')) +
                ggplot2::scale_colour_manual(values = c('cornflowerblue', 'firebrick', 'gold'), guide = "none") +
                ggplot2::labs(fill = NULL) +
                ggplot2::annotate("text", x = df.vdc$x, y = df.vdc$y, label = df.vdc$label, size = 5) +
                ggplot2::ggtitle(paste0(sigThresType, " < ", sigThres, 
                           ", |log2 fold| >= ", log2FoldThres))
            
            index <- index + 1
            plts[[index]] <- plt
        }
    }
    
    grid::grid.newpage()
    grid::pushViewport(grid::viewport(layout=currentLayout))
    printPlots(plts, "Venns", pageno, jobName, currentLayout)  
}


#' Takes an NormalyzerStatistics instance and generates and prints a p-value
#' histogram for each onto the viewport
#' 
#' @param nst NormalyzerDE statistics object.
#' @param jobName Name of processing run.
#' @param currentLayout Layout used for document.
#' @param pageno Current page number.
#' @return None
#' @keywords internal
plotContrastPHists <- function(nst, jobName, currentLayout, pageno) {

    contrastPLists <- pairwiseCompsP(nst)
    histPlots <- list()
    
    for (i in seq_along(contrastPLists)) {
        contrast <- names(contrastPLists)[i]
        pVals <- contrastPLists[[contrast]]
        df <- data.frame(pVals=pVals)
        histPlots[[i]] <- ggplot2::ggplot(df, ggplot2::aes(pVals)) + 
            ggplot2::theme_classic() + 
            ggplot2::geom_histogram(na.rm=TRUE, binwidth=0.01, color="#268BD2", fill="#268BD2") +
            ggplot2::ggtitle(paste("Contrast:", contrast)) + 
            ggplot2::xlab("P-values") + 
            ggplot2::ylab("Count") +
            ggplot2::theme(
                axis.text.x = ggplot2::element_text(angle = 90, hjust = 1, vjust=0.5), 
                legend.position="none"
            )
    }
    
    grid::grid.newpage()
    grid::pushViewport(grid::viewport(layout=currentLayout))
    printPlots(histPlots, "HistPlots", pageno, jobName, currentLayout)  
}

#' Takes an NormalyzerStatistics instance and generates and prints a 
#' volcano plot
#' 
#' @param nst NormalyzerDE statistics object.
#' @param jobName Name of processing run.
#' @param currentLayout Layout used for document.
#' @param pageno Current page number.
#' @param type Specify whether to plot 'Volcano' or 'MA'.
#' @param sigThres FDR threshold for DE coloring.
#' @return None
#' @keywords internal
plotSigScatter <- function(nst, jobName, currentLayout, pageno, type="Volcano",
                           sigThres=0.1, sigThresType="fdr", log2FoldThres=0) {
    
    contrastPLists <- pairwiseCompsP(nst)
    contrastFDRLists <- pairwiseCompsFdr(nst)
    contrastFoldLists <- pairwiseCompsFold(nst)
    contrastAveLists <- pairwiseCompsAve(nst)
    plots <- list()
    
    for (i in seq_along(contrastPLists)) {
        
        contrast <- names(contrastPLists)[i]
        pVals <- contrastPLists[[contrast]]
        fdrVals <- contrastFDRLists[[contrast]]
        fold <- contrastFoldLists[[contrast]]
        expression <- contrastAveLists[[contrast]]
        
        if (sigThresType == "fdr") {
            stat_sig <- (fdrVals < sigThres)
            legend_label <- paste("FDR <", sigThres)
        }
        else if (sigThresType == "p") {
            stat_sig <- (pVals < sigThres)
            legend_label <- paste("P <", sigThres)
        }
        else {
            stop("Unknown significance threshold type: ", sigThresType)
        }
        
        if (log2FoldThres != 0) {
            fold_sig <- abs(fold) >= log2FoldThres
            sig <- stat_sig & fold_sig
            legend_label <- paste0(legend_label, "\nFold > ", log2FoldThres)
        }
        else {
            sig <- stat_sig
        }
        
        df <- data.frame(pVals=pVals, fold=fold, sig=sig, expression=expression)
        df <- df[stats::complete.cases(df),]
        
        if (type == "Volcano") {
            plots[[i]] <- ggplot2::ggplot(df, ggplot2::aes(fold, -log10(pVals), color=sig)) + 
                ggplot2::geom_point(size=1, alpha=0.5, na.rm=TRUE) +
                ggplot2::xlab("Fold (log2)") + 
                ggplot2::ylab("-log10(Pvalue)")
        }
        else if (type == "MA") {
            plots[[i]] <- ggplot2::ggplot(df, ggplot2::aes(expression, fold, color=sig)) + 
                ggplot2::geom_point(size=1, alpha=0.5, na.rm=TRUE) +
                ggplot2::xlab("Expression (log2)") + 
                ggplot2::ylab("Fold (log2)")
        }
        else {
            stop("Unknown plot type: ", type)
        }
        
        plots[[i]] <- plots[[i]] + 
            ggplot2::ggtitle(paste("Contrast:", contrast)) + 
            ggplot2::theme_classic() + 
            ggplot2::labs(color=legend_label)
    }
    
    grid::grid.newpage()
    grid::pushViewport(grid::viewport(layout=currentLayout))
    printPlots(plots, type, pageno, jobName, currentLayout)  
}

#' Show in a PCA plot what samples are compared in statistical contrast
#' This is useful to understand what conditions are compared and for checking
#' for outliers in the contrast
#' 
#' @param nst NormalyzerDE statistics object.
#' @param jobName Name of processing run.
#' @param currentLayout Layout used for document.
#' @param pageno Current page number.
#' @param pcs Principal components to show.
#' @return None
#' @keywords internal
plotContrastPCA <- function(nst, jobName, currentLayout, pageno, pcs=c(1,2)) {
    
    activeLevelsFactor <- function(fullFactor, activeLevelsList) {
        
        activeIndsList <- list()
        for (levelName in names(activeLevelsList)) {
            activeLevel <- activeLevelsList[[levelName]]
            activeInds <- which(fullFactor %in% activeLevel)
            for (ind in activeInds) {
                activeIndsList[[as.character(ind)]] <- levelName
            }
        }
        getLevel <- function(index, activeIndsList) {
            if (index %in% names(activeIndsList)) {
                activeIndsList[[as.character(index)]]
            }
            else {
                "other"
            }
        }
        subLevel <- vapply(seq_len(length(fullFactor)), getLevel, 
                           "", activeIndsList = activeIndsList)
        subLevel
    }
    
    contrasts <- comparisons(nst)
    dataDf <- dataMat(nst)
    dfPCA <- stats::prcomp(t(dataDf[stats::complete.cases(dataDf), ]), scale=TRUE, center=TRUE)
    dfOut <- as.data.frame(dfPCA$x)
    percentageVar <- round(dfPCA$sdev^2 / sum(dfPCA$sdev^2) * 100, 2)
    percentageVar <- paste0(colnames(dfOut), " (", paste0(as.character(percentageVar), "%)"))
    groups <- condCol(nst)
    dfOut$sample <- colnames(dataDf)
    
    pc1 <- paste0("PC", pcs[1])
    pc2 <- paste0("PC", pcs[2])
    
    plots <- list()
    
    for (contrast in contrasts) {
        
        contrastLevels <- unlist(strsplit(contrast, "-"))
        
        dfOut$group <- activeLevelsFactor(
            condCol(nst), 
            list(high=contrastLevels[1], low=contrastLevels[2])
        )
        
        plt <- ggplot2::ggplot(
            dfOut, 
            ggplot2::aes_string(x=pc1, y=pc2, color="group", label="sample")) + 
            ggplot2::geom_text() + 
            ggplot2::theme_classic() +
            ggplot2::scale_color_manual(values=c("#00AAAA", "#AA0000", "#BBBBBB")) +
            ggplot2::ggtitle(paste(contrastLevels, collapse=" vs. ")) +
            ggplot2::xlab(percentageVar[pcs[1]]) +
            ggplot2::ylab(percentageVar[pcs[2]])  
        
        plots[[contrast]] <- plt
    }
    
    
    grid::grid.newpage()
    grid::pushViewport(grid::viewport(layout=currentLayout))
    title <- paste0("PCA, contrast colored (factors", pcs[1], " and ", pcs[2], ")")
    printPlots(plots, title, pageno, jobName, currentLayout)  
}

#' Load raw data into dataframe
#' 
#' General function which allows specifying different types of input data
#' including "proteios", "maxquantpep" (peptide output from MaxQuant) and 
#' "maxquantprot" (protein output from MaxQuant) formats.
#' 
#' @param dataPath File path to design matrix.
#' @param inputFormat If input is given in standard NormalyzerDE format, 
#' Proteios format or in MaxQuant protein or peptide format
#' @return rawData Raw data loaded into data frame
#' @examples \dontrun{
#' df <- loadData("data.tsv")
#' }
loadData <- function(dataPath, inputFormat="default") {
    
    if (inputFormat == "default") {
        rawData <- loadRawDataFromFile(dataPath)
    } else if (inputFormat == "proteios") {
        rawData <- proteiosToNormalyzer(dataPath)
    } else if (inputFormat == "maxquantpep") {
        rawData <- maxQuantToNormalyzer(dataPath, protLevel=FALSE)
    } else if (inputFormat == "maxquantprot") {
        rawData <- maxQuantToNormalyzer(dataPath, protLevel=TRUE)
    } else {
        valids <- c("default", "proteios", "maxquantpep", "maxquantprot")
        stop("Unknown inputFormat: ", 
             inputFormat, 
             " valids are: ", 
             paste(valids, collapse=", ")
        )
    }
    
    rawData
}

#' Load raw design into dataframe
#' 
#' Takes a design path, loads the matrix and ensures that the sample column
#' is in character format and that the group column is in factor format.
#' 
#' @param designPath File path to design matrix.
#' @param sampleCol Column name for column containing sample names.
#' @param groupCol Column name for column containing condition levels.
#' @return designMatrix Design data loaded into data frame
#' @examples \dontrun{
#' df <- loadDesign("design.tsv")
#' }
loadDesign <- function(designPath, sampleCol="sample", groupCol="group") {
    
    designMatrix <- utils::read.table(
        designPath, 
        sep="\t", 
        stringsAsFactors=FALSE, 
        header=TRUE, 
        comment.char=""
    )
    
    if (!(sampleCol %in% colnames(designMatrix)) || !(groupCol %in% colnames(designMatrix))) {
        stop("Both sampleCol value and groupCol value must be present in the design matrix header.",
             "\nsampleCol: ", sampleCol, "\ngroupCol: ", groupCol,
             "\nDesign matrix header: ", paste(colnames(designMatrix), collapse=", "))
    }
    
    designMatrix[, sampleCol] <- as.character(designMatrix[, sampleCol])
    designMatrix[, groupCol] <- as.factor(as.character(designMatrix[, groupCol]))
    designMatrix
}

#' Prepare SummarizedExperiment object for raw data to be normalized containing 
#' data, design and annotation information
#' 
#' @param dataPath File path to data matrix.
#' @param designPath File path to design matrix.
#' @param inputFormat Type of matrix for data, can be either 'default',
#'   'proteios', 'maxquantprot' or 'maxquantpep'
#' @param zeroToNA If TRUE zeroes in the data is automatically converted to
#'   NA values
#' @param sampleColName Column name for column containing sample names
#' @param groupColName Column name for column containing condition levels
#' @return experimentObj SummarizedExperiment object loaded with the data
#' @export
#' @examples 
#' data_path <- system.file(package="NormalyzerDE", "extdata", "tiny_data.tsv")
#' design_path <- system.file(package="NormalyzerDE", "extdata", "tiny_design.tsv")
#' df <- setupRawDataObject(data_path, design_path)
setupRawDataObject <- function(dataPath, designPath, inputFormat="default", zeroToNA=FALSE, 
                               sampleColName="sample", groupColName="group") {
    
    rawDesign <- loadDesign(designPath, sampleCol=sampleColName, groupCol=groupColName)
    rawData <- loadData(dataPath, inputFormat=inputFormat)
    rdf <- rawData[2:nrow(rawData), ]
    colnames(rdf) <- rawData[1, ]
    
    verifyDesignMatrix(rdf, rawDesign, sampleColName)
        
    rawDesign[[sampleColName]] <- as.character(rawDesign[[sampleColName]])
    
    sdf <- rdf[, as.character(rawDesign[[sampleColName]])]
    adf <- rdf[, !(colnames(rdf) %in% as.character(rawDesign[[sampleColName]])), drop=FALSE]
    
    experimentObj <- SummarizedExperiment::SummarizedExperiment(
        assays=list(raw=as.matrix(sdf)),
        rowData=adf,
        colData=rawDesign,
        metadata=list(sample=sampleColName, group=groupColName)
    )
    experimentObj
}

#' Prepare SummarizedExperiment object for statistics data
#' 
#' @param dataPath Path to raw data matrix
#' @param designPath Path to design matrix
#' @param sampleColName Name for column in design matrix containing sample names
#' @return experimentObj Prepared instance of SummarizedExperiment
#' @export
#' @examples 
#' data_path <- system.file(package="NormalyzerDE", "extdata", "tiny_data.tsv")
#' design_path <- system.file(package="NormalyzerDE", "extdata", "tiny_design.tsv")
#' sumExpObj <- setupRawContrastObject(data_path, design_path, "sample")
setupRawContrastObject <- function(dataPath, designPath, sampleColName) {
    
    fullDf <- utils::read.csv(
        dataPath, 
        sep="\t", 
        stringsAsFactors=FALSE, 
        quote="", 
        comment.char="",
        check.names=FALSE
    )
    
    designDf <- utils::read.csv(
        designPath, 
        sep="\t",
        stringsAsFactors=FALSE,
        quote="",
        comment.char="",
        check.names=FALSE
    )
    
    verifyDesignMatrix(fullDf, designDf, sampleColName)
    
    sdf <- fullDf[, designDf[[sampleColName]]]
    adf <- fullDf[, !(colnames(fullDf) %in% as.character(designDf[[sampleColName]])), drop=FALSE]
    
    experimentObj <- SummarizedExperiment::SummarizedExperiment(
        assays=list(raw=as.matrix(sdf)),
        colData=designDf,
        rowData=adf,
        metadata=list(sample=sampleColName)
    )
    experimentObj
}

#' Verify that input data is in correct format, and if so, return a generated
#'  NormalyzerDE data object from that input data
#' 
#' This function performs a number of checks on the input data and provides
#' informative error messages if the data isn't fulfilling the required format.
#' Checks include verifying that the design matrix matches to the data matrix,
#' that the data matrix contains valid numbers and that samples have enough
#' values for analysis
#' 
#' @param jobName Name of ongoing run.
#' @param summarizedExp Summarized experiment input object
#' @param threshold Minimum number of features.
#' @param omitSamples Automatically omit invalid samples from analysis.
#' @param requireReplicates Require there to be at least to samples per
#'        condition
#' @param quiet Don't print output messages during processing 
#' @param noLogTransform Don't log-transform the provided data
#' @param tinyRunThres If less features in run, a limited run is performed
#' 
#' @return Normalyzer data object representing verified input data.
#' @export
#' @examples
#' data(example_summarized_experiment)
#' normObj <- getVerifiedNormalyzerObject("job_name", example_summarized_experiment)
getVerifiedNormalyzerObject <- function(
        jobName, 
        summarizedExp,
        threshold=15, 
        omitSamples=FALSE,
        requireReplicates=TRUE,
        quiet=FALSE,
        noLogTransform=FALSE,
        tinyRunThres=50
    ) {

    SummarizedExperiment::assay(summarizedExp) <- preprocessData(SummarizedExperiment::assay(summarizedExp), quiet=quiet)
    summarizedExp <- filterOnlyNARows(summarizedExp)
    
    # TODO: The getter seems to not be available, check later if temporary issue
    groupCol <- summarizedExp@metadata$group
    sampleCol <- summarizedExp@metadata$sample
    designMatrix <- as.data.frame(SummarizedExperiment::colData(summarizedExp))
    
    if (!groupCol %in% colnames(designMatrix)) {
        stop("Given groupCol: '", groupCol, "' was not present among design matrix columns")
    }

    groups <- SummarizedExperiment::colData(summarizedExp)[[groupCol]]
    samples <- SummarizedExperiment::colData(summarizedExp)[[sampleCol]]
    dataMatrix <- SummarizedExperiment::assay(summarizedExp)
    annotationMatrix <- as.matrix(data.frame(lapply(
        SummarizedExperiment::rowData(summarizedExp),
        as.character
    ), stringsAsFactors=FALSE, check.names=FALSE))

    verifyDesignMatrix(dataMatrix, designMatrix, sampleCol)
    verifyValidNumbers(dataMatrix, groups, noLogTransform=noLogTransform, quiet=quiet)
    
    processedRawData <- getReplicateSortedData(dataMatrix, groups)
    # repSortedRawData <- getReplicateSortedData(dataMatrix, groups)
    # processedRawData <- preprocessData(repSortedRawData, quiet=quiet)
    
    lowCountSampleFiltered <- getLowCountSampleFiltered(
        processedRawData, 
        groups,
        threshold=threshold, 
        stopIfTooFew=!omitSamples
    )
    
    designMatrix <- designMatrix[designMatrix[[sampleCol]] %in% colnames(lowCountSampleFiltered), ]
    
    # If no samples left after omitting, stop
    verifyMultipleSamplesPresent(
        lowCountSampleFiltered, 
        groups, 
        requireReplicates=requireReplicates, 
        quiet=quiet
    )
    
    validateSampleReplication(
        lowCountSampleFiltered, 
        groups, 
        requireReplicates=requireReplicates, 
        quiet=quiet
    )
    
    nds <- NormalyzerDataset(
        jobName=jobName,
        designMatrix=designMatrix,
        rawData=processedRawData,
        annotationData=annotationMatrix,
        sampleNameCol=sampleCol,
        groupNameCol=groupCol,
        tinyRunThres=tinyRunThres,
        quiet=quiet
    )
    
    nds
}


filterOnlyNARows <- function(summarizedExp) {

    dataMatrix <- SummarizedExperiment::assay(summarizedExp)
    
    nonFullNAContr <- rowSums(is.na(SummarizedExperiment::assay(summarizedExp))) != ncol(summarizedExp)
    if (length(which(!nonFullNAContr)) > 0) {
        message(length(which(!nonFullNAContr)), " entries with only NA values omitted")
        summarizedExp <- summarizedExp[nonFullNAContr, ]
    }
    
    summarizedExp
}

#' Try reading raw Normalyzer matrix from provided filepath
#' 
#' @param inputPath Path to Normalyzer data.
#' @return Table containing raw data from input file.
#' @keywords internal
loadRawDataFromFile <- function(inputPath) {
    
    tryCatch(
        rawData <- as.matrix(utils::read.table(inputPath, 
                                               header=FALSE, 
                                               sep="\t", 
                                               stringsAsFactors=FALSE,
                                               quote="",
                                               comment.char="")),
        error=function(e) {
            message("Error encountered for input file:", inputPath, ", error: ", e)
            stop("Please provide a valid input file.\n")
        },
        
        warning=function(w) {
            message("An issue was encountered when attempting to load:", 
                    inputPath,
                    "Warning:",
                    w)
            stop("Please investigate the warning and provide a valid input file.\n")
        }
    )
    
    rawData
}


#' Verify that input fields conform to the expected formats
#' 
#' @param rawDataOnly Dataframe with input data.
#' @param groups Condition levels for comparisons.
#' @return None
#' @keywords internal
verifyValidNumbers <- function(rawDataOnly, groups, noLogTransform=FALSE, quiet=FALSE) {
    
    # Fields expected to contain numbers in decimal or scientific notation, or containing NA or null
    validPatterns <- c("\\d+(\\.\\d+)?", "NA", "\"NA\"", "null", "\\d+(\\.\\d+)?[eE]([\\+\\-])?\\d+$", "")
    
    regexPattern <- sprintf("^(%s)$", paste(validPatterns, collapse="|"))
    nonMatchIndices <- grep(regexPattern, rawDataOnly, perl=TRUE, ignore.case=TRUE, invert = TRUE)
    naIndices <- which(is.na(rawDataOnly))
    invalidNonNAIndices <- nonMatchIndices[!nonMatchIndices %in% naIndices]
    rowsWithIssues <- unique((invalidNonNAIndices-1) %% nrow(rawDataOnly) + 1)
    
    if (length(invalidNonNAIndices) > 0) {
        errorString <- paste(
            "Invalid values encountered in input data.",
            "Only valid data is numeric (dot-decimal, not comma) and NA- or na-fields",
            "Invalid fields: ",
            paste(rawDataOnly[unique(invalidNonNAIndices)], collapse=" "),
            "These were encountered for row numbers (showing max 10 first):",
            paste(utils::head(rowsWithIssues, 10), collapse=", "),
            "Content of the first row with issues: ",
            paste(rawDataOnly[rowsWithIssues[1], ], collapse=", "),
            "Aborting...",
            sep="\n"
        )
        
        stop(errorString)
    }
    
    if (!noLogTransform) {
        belowOnePattern <- c("^0.\\d+$")
        belowOneMatches <- grep(belowOnePattern, rawDataOnly, perl=TRUE)
        rowsWithIssues <- unique((belowOneMatches-1) %% nrow(rawDataOnly) + 1)
        firstIssueRow <- rawDataOnly[rowsWithIssues[1], ]

        if (length(belowOneMatches) > 0) {
            errorString <- paste(
                "Encountered below-one values in raw data. As the data is log-transformed ",
                "during processing this will lead to negative values which in turn will ",
                "crash processing. Consider using the 'noLogTransform' option if your data ",
                "already is normally distributed or scaling all values if appropriate.", 
                "First ten row numbers where this issue was encountered, excluding header row: ",
                paste(utils::head(rowsWithIssues, 10), collapse=", "),
                paste0("Content of first row (row ", rowsWithIssues[1], ") with issues: "),
                paste(firstIssueRow, collapse=", "),
                sep="\n"
            )
            stop(errorString)
        }
    }

    if (!quiet) message("Input data checked. All fields are valid.")
}


#' Verify that SummarizedExperiment setup matches the data matrix
#' 
#' @param summarizedExp SummarizedExperiment object.
#' @param sampleCol Column in the SummarizedExperiment colData containing sample IDs.
#' 
#' @return None
#' @keywords internal
verifySummarizedExperiment <- function(summarizedExp, sampleCol) {
    
    fullMatrix <- cbind(
        data.frame(SummarizedExperiment::rowData(summarizedExp)),
        SummarizedExperiment::assay(summarizedExp)
    )
    
    designMatrix <- data.frame(SummarizedExperiment::colData(summarizedExp))
    
    verifyDesignMatrix(
        fullMatrix,
        designMatrix,
        sampleCol
    )
}


#' Verify that design matrix setup matches the data matrix
#' 
#' @param fullMatrix Dataframe with input data.
#' @param designMatrix Dataframe with design setup.
#' @param sampleCol Column in design matrix containing sample IDs.
#' 
#' @return None
#' @keywords internal
verifyDesignMatrix <- function(fullMatrix, designMatrix, sampleCol) {

    if (!(sampleCol %in% colnames(designMatrix))) {
        stop("Design matrix header must contain sampleCol name. \n", 
             "Provided sampleCol was: ", sampleCol, " \n", 
             "Following header was found in the design matrix: ", paste(colnames(designMatrix), collapse=", "))
    }
    
    designColnames <- designMatrix[, sampleCol]
    
    if (!all(designColnames %in% colnames(fullMatrix))) {
        errorString <- paste(
            "Not all columns present in design matrix are present in data matrix. \n",
            " The following elements were not found in the data matrix header: \n\n",
            paste(base::setdiff(designColnames, colnames(fullMatrix)), collapse=", "), " \n",
            "\n Please carefully check that the column names in your data matrix", 
            "matches the sample column in the design matrix"
        )
        stop(errorString)
    }

    dataMatrix <- fullMatrix[, designMatrix[, sampleCol]]
    dataColumns <- dataMatrix[, designColnames]
    
    if (length(designColnames) != ncol(dataColumns)) {
        errorString <- paste(
            "Number of samples does not match number of selected columns",
            "Found number of columns:",
            ncol(dataColumns),
            "Expected number of columns:",
            length(designColnames),
            "Are all columns in the design matrix present in the data matrix?"
        )
        stop(errorString)
    }
    
    if (length(unique(designColnames)) != length(designColnames)) {
        errorString <- paste(
            "Sample labels must be unique, found: ",
            length(unique(designColnames)),
            "unique, expected:",
            length(designColnames)
        )
        stop(errorString)
    }
}




#' Replace empty values (0 or empty field) with NA in input data
#' 
#' @param dataMatrix Matrix with raw data.
#' @param quiet Don't show diagnostic messages
#' @return Parsed rawdata where 0 values are replaced with NA
#' @keywords internal
preprocessData <- function(dataMatrix, quiet=FALSE) {

    zeroFields <- length(dataMatrix[!is.na(dataMatrix) & dataMatrix == 0])
    emptyFields <- length(dataMatrix[!is.na(dataMatrix) & dataMatrix == ""])
    nullFields <- length(dataMatrix[!is.na(dataMatrix) & dataMatrix == "null"])
    
    if (zeroFields != 0) {
        if (!quiet) {
            message(zeroFields, " fields with '0' were replaced by 'NA'")
        }
        dataMatrix[dataMatrix == 0] <- NA
    }
    
    if (emptyFields != 0) {
        if (!quiet) {
            message(emptyFields, " empty fields were replaced by 'NA'")
        }
        dataMatrix[dataMatrix == ""] <- NA
    }
    
    if (nullFields != 0) {
        if (!quiet) {
            message(nullFields, " 'null' fields were replaced by 'NA'")
        }
        dataMatrix[dataMatrix == "null"] <- NA
    }

    dataMatrix
}

#' Verify that samples contain at least a lowest number of values
#' 
#' @param dataMatrix Dataframe with processed input data.
#' @param groups Vector containing condition levels.
#' @param threshold Lowest number of allowed values in a column.
#' @param stopIfTooFew Abort run if lower than threshold number of values in
#'        column
#' @return None
#' @keywords internal
getLowCountSampleFiltered <- function(dataMatrix, groups, threshold=15, stopIfTooFew=TRUE) {
    
    sampleIndices <- seq_along(groups)
    numberOfValues <- colSums(!is.na(dataMatrix))
    notPassingThreshold <- which(numberOfValues < threshold)
    
    if (length(notPassingThreshold) == length(numberOfValues)) {
        errorString <- paste(
            "None of the samples had enough valid non-NA values",
            "Found number of non-NA values:",
            paste(numberOfValues[notPassingThreshold], collapse=" "),
            "Current threshold: ",
            threshold,
            "You could try lowering the threshold by adjusting \"sampleAbundThres\"",
            "Be aware that this will likely lead to downstream crashes.",
            sep="\n"
        )
        stop(errorString)
    }
    else if (length(notPassingThreshold) > 0) {
        errorString <- paste(
            "Following samples does not contain enough non-NA values:",
            paste(sampleIndices[notPassingThreshold], collapse=" "),
            "Found number of values:",
            paste(numberOfValues[notPassingThreshold], collapse=" "),
            "Current threshold:",
            threshold,
            "You can force processing without this sample by specifying the",
            "option \"omitLowAbundSamples\"",
            sep="\n")
        
        if (stopIfTooFew) {
            stop(errorString)
        }
        else {
            warning(errorString)
        }
    }
    
    if (length(notPassingThreshold > 0)) {
        naSamplesOmittedDf <- dataMatrix[, -sampleIndices[notPassingThreshold]]
    }
    else {
        dataMatrix
    }
}


#' Check whether all samples have replicates
#' 
#' @param dataMatrix Prepared matrix containing expression data.
#' @param groups Vector containing condition levels
#' @param requireReplicates By default stops processing if not all samples
#'  have replicates
#' @return None
#' @keywords internal
validateSampleReplication <- function(dataMatrix, groups, requireReplicates=TRUE, quiet=FALSE) {
    
    headerCounts <- table(groups)
    nonReplicatedSamples <- names(headerCounts[headerCounts == 1])

    if (length(nonReplicatedSamples) > 0) {
        
        errorString <- paste(
            "Following group conditions does not have replicates:",
            paste(nonReplicatedSamples, collapse=" "),
            "By default this is not allowed.",
            "You can force limited processing of non-replicated data by",
            "setting the \"requireReplicates\" option to FALSE",
            sep="\n")
        
        if (requireReplicates) {
            stop(errorString)
        }
        else if (!quiet) {
            warning(errorString)
        }
    }
    else {
        if (!quiet) message("Sample replication check: All samples have replicates")
    }
}


#' Check whether more than one sample is present
#' 
#' @param dataMatrix Prepared dataframe.
#' @param groups Vector containing condition levels
#' @param requireReplicates By default stops processing if not all samples
#'  have replicates
#' @return None
#' @keywords internal
verifyMultipleSamplesPresent <- function(dataMatrix, groups, requireReplicates=TRUE, quiet=FALSE) {
    
    samples <- groups[as.numeric(as.factor(groups)) > 0]
    distinctSamples <- unique(samples)

    if (length(samples) < 2) {
        errorString <- paste(
            "At least two samples are required to run Normalyzer",
            "Here, we found:",
            paste(samples, collapse=" "),
            sep="\n")
        
        stop(errorString)
    }
            
    if (length(distinctSamples) == 1) {
        
        errorString <- paste(
            "Found less than two distinct sample groups. Following was found:",
            paste(distinctSamples, collapse=" "),
            "For full processing two or more sample groups are required",
            "You can force limited processing for one sample group by setting the", 
            "\"requireReplicates\" option to FALSE\n",
            sep="\n")
        
        if (requireReplicates) {
            stop(errorString)
        }
        else if (!quiet) {
            warning(errorString)
        }
    }
    else if (length(distinctSamples) == 0) {
        stop("No replicate groups found. Double check your input file,
             and check that your data haven't been filtered out in preceeding
             input validation steps.")
    }
    else {
        if (!quiet) message("Sample check: More than one sample group found")
    }
}

context("Global test")

designPath <- system.file(package="NormalyzerDE", "extdata", "tiny_design.tsv")
dataPath <- system.file(package="NormalyzerDE", "extdata", "tiny_data.tsv")
tempOut <- tempdir()

design <- read.csv(designPath, sep="\t")
sample_names <- as.character(design$sample)

rdf <- read.csv(
    dataPath, 
    stringsAsFactors=FALSE, 
    quote="", 
    comment.char="", 
    sep="\t",
    check.names=FALSE
)

sdf <- rdf[, sample_names]
adf <- rdf[, !(colnames(rdf) %in% sample_names)]

test_that("Parsed input data is same as raw input", {
    
    se <- setupRawDataObject(
        dataPath, 
        designPath,
        zeroToNA=TRUE,
        sampleColName="sample",
        groupColName="group",
        inputFormat="default"
    )
    
    expect_true(all(design == data.frame(SummarizedExperiment::colData(se))), "colData should be identical to data frame")

    se_data <- SummarizedExperiment::assay(se)
    class(se_data) <- "numeric"
    
    expect_true(all(colSums(se_data, na.rm=TRUE) == colSums(sdf, na.rm=TRUE)), "Sum of data should be same")
    expect_true(all(colnames(se_data) == colnames(sdf)), "Column names should be the same")
    expect_true(all(adf == data.frame(SummarizedExperiment::rowData(se))), "Annotation information should be the same")
})

#' Representation of the results from performing normalization over a dataset
#' 
#' It is linked to a NormalyzerDataset instance representing the raw data
#' which has been processed. After performing evaluation it also links to
#' an instance of NormalyzerEvaluationResults representing the results
#' from the evaluation.
#' 
#' @slot normalizations SummarizedExperiment object containing calculated
#'   normalization results
#' @slot nds Normalyzer dataset representing run data
#' @slot ner Normalyzer evaluation results
#' for running extended normalizations
#' @importClassesFrom SummarizedExperiment SummarizedExperiment
#' @export
NormalyzerResults <- setClass("NormalyzerResults",
                              slots=c(
                                  nds = "NormalyzerDataset",
                                  ner = "NormalyzerEvaluationResults",
                                  normalizations = "list"
                              ))

#' Constructor for NormalyzerResults
#' 
#' @param nds NormalyzerDataset object
#' @return nr Prepared NormalyzerResults object
#' @export
#' @examples
#' data(example_summarized_experiment)
#' normObj <- getVerifiedNormalyzerObject("job_name", example_summarized_experiment)
#' emptyNormResults <- NormalyzerResults(normObj)
NormalyzerResults <- function(nds) {
    nr <- new("NormalyzerResults", nds=nds)
    nr
}

setGeneric("normalizations", function(object) { standardGeneric("normalizations") })
setMethod("normalizations", signature(object="NormalyzerResults"), 
          function(object) { slot(object, "normalizations") })
setGeneric("normalizations<-", function(object, value) { standardGeneric("normalizations<-") })
setReplaceMethod("normalizations", signature(object="NormalyzerResults"), 
                 function(object, value) { 
                     slot(object, "normalizations") <- value
                     validObject(object)
                     object
                 })

setGeneric("nds", function(object) { standardGeneric("nds") })
setMethod("nds", signature(object="NormalyzerResults"), 
          function(object) { slot(object, "nds") })
setGeneric("nds<-", function(object, value) { standardGeneric("nds<-") })
setReplaceMethod("nds", signature(object="NormalyzerResults"), 
                 function(object, value) { 
                     slot(object, "nds") <- value
                     validObject(object)
                     object
                 })

setGeneric("ner", function(object) { standardGeneric("ner") })
setMethod("ner", signature(object="NormalyzerResults"), 
          function(object) { slot(object, "ner") })
setGeneric("ner<-", function(object, value) { standardGeneric("ner<-") })
setReplaceMethod("ner", signature(object="NormalyzerResults"), 
                 function(object, value) { 
                     slot(object, "ner") <- value
                     validObject(object)
                     object
                 })

#' Main function for executing normalizations
#'
#' @param nr Normalyzer results object.
#' @param forceAll Ignore dataset size limits and run all normalizations
#'  (only meant for testing purposes)
#' @param rtNorm Perform retention time based normalizations
#' 
#' @param rtStepSizeMinutes Retention time normalization window size.
#' @param rtWindowMinCount Minimum number of datapoints in each retention-time
#'   segment.
#' @param rtWindowShifts Number of layered retention time normalized windows.
#' @param rtWindowMergeMethod Merge approach for layered retention time windows.
#' @param noLogTransform Prevent log-transforming input
#' @param quiet Don't show regular output messages
#' 
#' @return nr NormalyzerDE results object
#' @rdname performNormalizations
#' @keywords internal
setGeneric(name="performNormalizations",
           function(nr, forceAll=FALSE, rtNorm=FALSE, rtStepSizeMinutes=1, 
                    rtWindowMinCount=100, rtWindowShifts=1, 
                    rtWindowMergeMethod="median", noLogTransform=FALSE, quiet=FALSE) 
               standardGeneric("performNormalizations"))
#' @rdname performNormalizations
setMethod("performNormalizations", "NormalyzerResults",
          function(nr, forceAll=FALSE, rtNorm=FALSE, rtStepSizeMinutes=1, 
                   rtWindowMinCount=100, rtWindowShifts=1, 
                   rtWindowMergeMethod="median", noLogTransform=FALSE, quiet=FALSE) {
              
              nds <- nds(nr)
              rtColPresent <- length(retentionTimes(nds)) > 0
              normResults <- list()
              
              if (!noLogTransform) {
                  normResults[["log2"]] <- log2(filterrawdata(nds))
                  
                  if (!isTinyRun(nds)) {
                      normResults[["VSN"]] <- performVSNNormalization(filterrawdata(nds))
                  }
                  else {
                      if (!quiet) {
                          message(
                              "Skipping VSN normalization due to small number of features, ",
                              nrow(filterrawdata(nds)), " features found"
                          )
                      }
                  }
              }
              else {
                  normResults[["log2"]] <- filterrawdata(nds)
                  if (!quiet) {
                      message("VSN normalization assumes non log-transformed data, as the option ",
                              "noLogTransform is specified it is assumed to not need log2-transformation ",
                              "and thus the VSN normalization is skipped\n")
                  }
              }
              
              normResults[["GI"]] <- globalIntensityNormalization(filterrawdata(nds), noLogTransform=noLogTransform)
              normResults[["median"]] <- medianNormalization(filterrawdata(nds), noLogTransform=noLogTransform)
              normResults[["mean"]] <- meanNormalization(filterrawdata(nds), noLogTransform=noLogTransform)
              normResults[["Quantile"]] <- performQuantileNormalization(filterrawdata(nds), noLogTransform=noLogTransform)
              # normResults[["SMAD"]] <- performSMADNormalization(filterrawdata(nds), noLogTransform=noLogTransform)
              normResults[["CycLoess"]] <- performCyclicLoessNormalization(filterrawdata(nds), noLogTransform=noLogTransform)
              normResults[["RLR"]] <- performGlobalRLRNormalization(filterrawdata(nds), noLogTransform=noLogTransform)
              
              enoughDataForRT <- nrow(filterrawdata(nds)) >= rtWindowMinCount
              
              if (!enoughDataForRT) {
                  if (!quiet) {
                      warning("Number of features in data matrix is smaller than minimum normalization window (set by option 'rtWindowMinCount') ",
                              "for RT normalization - skipping RT normalizations.\n")
                  }
              }
              else if (rtNorm && rtColPresent) {

                  normResults[["RT-median"]] <- getSmoothedRTNormalizedMatrix(
                      rawMatrix=filterrawdata(nds), 
                      retentionTimes=retentionTimes(nds), 
                      normMethod=medianNormalization, 
                      stepSizeMinutes=rtStepSizeMinutes,
                      windowMinCount=rtWindowMinCount,
                      mergeMethod=rtWindowMergeMethod,
                      windowShifts=rtWindowShifts, 
                      noLogTransform=noLogTransform
                  )
                  
                  normResults[["RT-mean"]] <- getSmoothedRTNormalizedMatrix(
                      rawMatrix=filterrawdata(nds), 
                      retentionTimes=retentionTimes(nds), 
                      normMethod=meanNormalization, 
                      stepSizeMinutes=rtStepSizeMinutes,
                      windowMinCount=rtWindowMinCount,
                      mergeMethod=rtWindowMergeMethod,
                      windowShifts=rtWindowShifts,
                      noLogTransform=noLogTransform
                  )
                  
                  normResults[["RT-Loess"]] <- getSmoothedRTNormalizedMatrix(
                      rawMatrix=filterrawdata(nds), 
                      retentionTimes=retentionTimes(nds), 
                      normMethod=performCyclicLoessNormalization, 
                      stepSizeMinutes=rtStepSizeMinutes,
                      windowMinCount=rtWindowMinCount,
                      mergeMethod=rtWindowMergeMethod,
                      windowShifts=rtWindowShifts,
                      noLogTransform=noLogTransform
                  )
                  
                  if (!noLogTransform) {
                      normResults[["RT-VSN"]] <- getSmoothedRTNormalizedMatrix(
                          rawMatrix=filterrawdata(nds), 
                          retentionTimes=retentionTimes(nds), 
                          normMethod=performVSNNormalization, 
                          stepSizeMinutes=rtStepSizeMinutes,
                          windowMinCount=rtWindowMinCount,
                          mergeMethod=rtWindowMergeMethod,
                          windowShifts=rtWindowShifts,
                          noLogTransform=noLogTransform
                      )
                  }
                  else {
                      if (!quiet) {
                          message("Skipping RT-VSN, only available for non log-transformed data\n")
                      }
                  }
              }
              else {
                  if (!quiet) {
                      message("No RT column specified (column named 'RT') or option not specified",
                              " Skipping RT normalization.\n")
                  }
              }
              
              normalizations(nr) <- normResults
              
              nr
          }
)

---
title: "Evaluation and statistics of expression data using NormalyzerDE"
author: "Jakob Willforss, Aakash Chawade and Fredrik Levander"
date: "`r format(Sys.Date(), '%m/%d/%Y')`"
abstract: >
    Technical biases reduces the ability to see the desired biological changes when performing omics experiments. There are numerous normalization techniques available to counter the biases, but to find the optimal normalization is often a non-trivial task. Furthermore there are limited tools available to counter biases such as retention-time based biases caused by fluctuating electrospray intensities. NormalyzerDE helps this process by performing a wide range of normalization techniques including a general and openly available approach to countering retention-time based biases. More importantly it calculates and visualizes a number of performance measures to guide the users selection of normalization technique. 
    Furthermore, NormalyzerDE provides means to easily perform differential expression statistics using either the empirical Bayes Limma approach or ANOVA. Evaluation visualizations are available for both normalization performance measures and as P-value histograms for the subsequent differential expression analysis comparisons. 
    NormalyzerDE package version: `r packageVersion("NormalyzerDE")`
output: 
  rmarkdown::html_document:
    highlight: pygments
    toc: true
    fig_width: 5
vignette: >
  %\VignetteIndexEntry{Differential expression and countering technical biases using NormalyzerDE}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\usepackage[utf8]{inputenc}
---

```{r setup, echo=FALSE, results="hide"}
# knitr::opts_chunk$set(tidy=FALSE, cache=TRUE, dev="png", message=FALSE,
# error=FALSE, warning=TRUE)
```

# Installation

Installation is preferably performed using BiocManager (requires R version >= 3.5):

```
install.packages("BiocManager")
BiocManager::install("NormalyzerDE")
```

# Default use

## Citing

Willforss, J., Chawade, A., Levander, F. Submitted article.

## Input format

NormalyzerDE expects a raw data file. Columns can contain annotation information or sample data.
Each column should start with a header.

```
pepseq  s1      s2      s3      s4
ATAAGG  20.0    21.2    19.4    18.5
AWAG    23.3    24.1    23.5    17.3
ACATGM  22.1    22.3    22.5    23.2
```

This data should be provided with a design matrix where all data samples should be represented.
One column (default header "sample") should match the columns containing samples in the raw data.
Another column (default header "group") should contain condition levels which could be used for group-based
evaluations.

```
sample  group
s1      condA
s2      condA
s3      condB
s4      condB
```

Alternatively the data can be provided as an instance of a `SummarizedExperiment` S4 class.

## Running NormalyzerDE evaluation

The evaluation step can be performed with one command, `normalyzer`.
This command expects a path to the data file, a name for the run-job,
a path to a design matrix and finally a path to an output directory.

Alternatively the `designPath` and `dataPath` arguments can be replaced with the `experimentObj` argument
where the first assay should contain the data matrix of interest, the `colData` attribute the design matrix
and the `rowData` attribute the annotation columns.

```{r}
library(NormalyzerDE)
outDir <- tempdir()
designFp <- system.file(package="NormalyzerDE", "extdata", "tiny_design.tsv")
dataFp <- system.file(package="NormalyzerDE", "extdata", "tiny_data.tsv")
normalyzer(jobName="vignette_run", designPath=designFp, dataPath=dataFp, 
           outputDir=outDir)
```

## Running NormalyzerDE statistical comparisons

When you after performing the evaluation and having evaluated the report have decided
for which normalization approach seems to work best you can continue to the statistical step.

Here, expected parameters are the path to the target normalization matrix, the sample design matrix as
in the previous step, a job name, the path to an output directory and a list of the pairwise comparisons
for which you want to calculate contrasts. They are provided as a character vector with conditions to
compare separated by a dash ("-").

Similarly as for the normalization step the `designPath` and `dataPath` arguments can be replaced with
an instance of `SummarizedExperiment` sent to the `experimentObj` argument.

```{r}
normMatrixPath <- paste(outDir, "vignette_run/CycLoess-normalized.txt", sep="/")
normalyzerDE("vignette_run", 
             comparisons=c("4-5"),
             designPath=designFp, 
             dataPath=normMatrixPath,
             outputDir=outDir, 
             condCol="group")
```

## Running NormalyzerDE using a SummarizedExperiment object as input

A benefit of using a SummarizedExperiment object as input is that it allows executing NormalyzerDE using variables as input instead of loading from file.

The conversion of `designMatrix$sample` is required if using `read.table` as it otherwise is interpreted as a factor. For more intuitive behaviour you can use `read_tsv` from the `readr` package instead of `read.table`.

```{r}
dataMatrix <- read.table(dataFp, sep="\t", header = TRUE)
designMatrix <- read.table(designFp, sep="\t", header = TRUE)
designMatrix$sample <- as.character(designMatrix$sample)
dataOnly <- dataMatrix[, designMatrix$sample]
annotOnly <- dataMatrix[, !(colnames(dataMatrix) %in% designMatrix$sample)]

sumExpObj <- SummarizedExperiment::SummarizedExperiment(
    as.matrix(dataOnly),
    colData=designMatrix,
    rowData=annotOnly
)

normalyzer(jobName="sumExpRun", experimentObj = sumExpObj, outputDir=outDir)
```

# Retention time normalization

Retention time based normalization can be performed with an arbitrary normalization matrix.

## Basic usage

There are two points of access for the higher order normalization. Either by calling `getRTNormalizedMatrix` which applies the target normalization approach stepwise over the matrix based on retention times, or by calling `getSmoothedRTNormalizedMatrix` which generates multiple layered matrices and combines them. To use them you need your raw data loaded into a matrix, a list containing retention times and a normalization matrix able to take a raw matrix and return a normalized in similar format.

```{r}
fullDf <- read.csv(dataFp, sep="\t")
designDf <- read.csv(designFp, sep="\t")
head(fullDf, 1)
head(designDf, 1)
```

At this point we have loaded the full data into dataframes. Next, we use the sample names present in the design matrix to extract sample columns from the raw data. Be careful that the sample names is a character vector. If it is a factor it will extract wrong columns.

Make sure that sample names extracted from design matrix are in right format. We expect it to be in 'character' format.

```{r}
sampleNames <- as.character(designDf$sample)
typeof(sampleNames)
```

Now we are ready to extract the data matrix from the full matrix. We also need to get the retention time column from the full matrix.

```{r}
dataMat <- as.matrix(fullDf[, sampleNames])
retentionTimes <- fullDf$Average.RT

head(dataMat, 1)
```

If everything is fine the data matrix should be `double`, and have the same number of rows as the number of retention time values we have.

```{r}
typeof(dataMat)

print("Rows and columns of data")
dim(dataMat)

print("Number of retention times")
length(retentionTimes)
```

The normalization function is expected to take a raw intensity matrix and return log transformed values. We borrow the wrapper function for Loess normalization from NormalyzerDE. It can be replaced with any custom function as long as the wrapper has the same input/output format.

```{r}
performCyclicLoessNormalization <- function(rawMatrix) {
    log2Matrix <- log2(rawMatrix)
    normMatrix <- limma::normalizeCyclicLoess(log2Matrix, method="fast")
    colnames(normMatrix) <- colnames(rawMatrix)
    normMatrix
}
```

We are ready to perform the normalization.

```{r}
rtNormMat <- getRTNormalizedMatrix(dataMat, 
                                   retentionTimes, 
                                   performCyclicLoessNormalization, 
                                   stepSizeMinutes=1, 
                                   windowMinCount=100)
```

Let's double check the results. We expect a matrix in the same format and shape as before. Furthermore, we expect similar but not the exact same values as if we'd applied the normalization globally.

```{r}
globalNormMat <- performCyclicLoessNormalization(dataMat)
dim(rtNormMat)
dim(globalNormMat)
head(rtNormMat, 1)
head(globalNormMat, 1)
```

## Performing layered normalization

We have everything set up to perform the layered normalization. The result here is expected to be overall similar to the normal retention time approach.

```{r}
layeredRtNormMat <- getSmoothedRTNormalizedMatrix(
    dataMat, 
    retentionTimes, 
    performCyclicLoessNormalization, 
    stepSizeMinutes=1, 
    windowMinCount=100, 
    windowShifts=3, 
    mergeMethod="mean")

dim(layeredRtNormMat)
head(layeredRtNormMat, 1)
```

# Stepwise processing (normalization part)

NormalyzerDE consists of a set of steps. The workflow can be run as a whole, or step by step.

## Step 1: Loading data

This step performs input validation of the data, and generates an object of the class NormalyzerDataset.

```{r}
jobName <- "vignette_run"
experimentObj <- setupRawDataObject(dataFp, designFp, "default", TRUE, "sample", "group")
normObj <- getVerifiedNormalyzerObject(jobName, experimentObj)
```

The function `setupRawDataObject` returns a `SummarizedExperiment` object.
This object can be prepared directly and should in that case contain the raw
data as the default assay, the design matrix as `colData` and annotation rows
as `rowData`.

## Step 2: Generate normalizations

Here, normalizations are performed. This generates a NormalyzerResults object containing both a reference to its original dataset object, but also generated normalization matrices.

```{r}
normResults <- normMethods(normObj)
```

## Step 3: Generate performance measures

Performance measures are calculated for normalizations. These are stored in an object NormalizationEvaluationResults. A NormalyzerResults object similar to the one sent in is returned, but with this field added.

```{r}
normResultsWithEval <- analyzeNormalizations(normResults)
```

## Step 4: Output matrices to file

Generated normalization matrices are written to the provided folder.

```{r}
jobDir <- setupJobDir("vignette_run", tempdir())
writeNormalizedDatasets(normResultsWithEval, jobDir)
```

## Step 5: Generate evaluation plots

Performance measures are used to generate evaluation figures which is written in an evaluation report.

```{r}
generatePlots(normResultsWithEval, jobDir)
```

After this evaluation of normalizations and progression to statistics follows as described previously in this report.

# Stepwise processing (differential expression analysis part)

## Step 1: Setup folders and data matrices

For continued processing you select the matrix containing the normalized data
from the best performing normalization. The design matrix is the same as for
the normalization step.

```{r}
bestNormMatPath <- paste(jobDir, "RT-Loess-normalized.txt", sep="/")
experimentObj <- setupRawContrastObject(bestNormMatPath, designFp, "sample")
nst <- NormalyzerStatistics(experimentObj, logTrans=FALSE)
```

Similarly as to for the normalization evaluation step the `experimentObj`
above can be prepared directly as a `SummarizedExperiment` object.

## Step 2: Calculate statistics

Now we are ready to perform the contrasts. Contrasts are provided as a vector
in the format `c("condA-condB", "condB-condC")`, where `condX` is the group levels.

```{r}
comparisons <- c("4-5")
nst <- calculateContrasts(nst, comparisons, condCol="group", leastRepCount=2)
```

## Step 3: Generate final matrix and output

Finally we generate a table containing the statistics results for each feature
and write it to file together with an evaluation report containing P-value histograms
for each comparison.

```{r}
annotDf <- generateAnnotatedMatrix(nst)
utils::write.table(annotDf, file=paste(jobDir, "stat_table.tsv", sep="/"))
generateStatsReport(nst, "Vignette stats", jobDir)
```

# Code organization

NormalyzerDE consists of a number of scripts and classes. They are focused around
two separate workflows. One is for normalizing and evaluating the normalizations. The
second is for performing differential expression analysis. Classes are contained in scripts with the same name.

Code organization:

![NormalyzerDE schematics](180813_normalyzerde_schematics.png)

The standard workflow for the normalization is the following:

* The `normalyzer` function in the `NormalyzerDE.R` script is called, starting the process.
* If applicable (that is, input is in Proteois or MaxQuant format), the dataset is preprocessed into the standard format using code in `preparsers.R`.
* The input is verified to capture standard errors early on using code in `inputVerification.R`. This results in an instance of the `NormalyzerDataset` class.
* The data is normalized using several normalization methods present in `normMethods.R`. This yields an instance of `NormalyzerResults` which links to the original `NormalyzerDataset` instance and also contains all the resulting normalized datasets.
* If specified (and if a column with retention time values is present) retention-time segmented approaches are performed by applying normalizations from `normMethods.R` over retention time using functions present in `higherOrderNormMethods.R`.
* The results are analyzed using functions present in `analyzeResults.R`. This yields an instance of `NormalyzerEvaluationResults` containing the evaluation results. This instance is attached to the `NormalyzerResults` object.
* The final results are sent to `outputUtils.R` where the normalizations are written to an output directory, and to `generatePlots.R` which contains visualizations for the performance measures. It also uses code in `printMeta.R` and `printPlots.R` to output the results in a desired format.

When a normalized matrix is selected the analysis proceeds to the statistical analysis.

* The `normalyzerde` function in the `NormalyzerDE.R` script is called starting the differential expression analysis pipeline.
* An instance of `NormalyzerStatistics` is prepared containing the input data.
* Code in the `calculateStatistics.R` script is used to calculate the statistical contrasts. The results are attached to the `NormalyzerStatistics` object.
* The resulting statistics are used to generate a report and an annotated output matrix where key statistical measures are attached to the original matrix.

# Used packages

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{createDirectory}
\alias{createDirectory}
\title{Create directory, or return error if already present}
\usage{
createDirectory(targetPath)
}
\arguments{
\item{targetPath}{Path where to attempt to create directory}
}
\value{
None
}
\description{
Create directory, or return error if already present
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/printPlots.R
\name{printPlots}
\alias{printPlots}
\title{Generate PDF grid page filling it with provided list of plots}
\usage{
printPlots(plotlist, plotname, pageno, jobname, currentLayout)
}
\arguments{
\item{plotlist}{List of target plots to display.}

\item{plotname}{List of names corresponding to the provided plot list.}

\item{pageno}{Current page number.}

\item{jobname}{Name of ongoing job.}

\item{currentLayout}{Custom viewport layout.}
}
\value{
None
}
\description{
Generate PDF grid page filling it with provided list of plots
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generatePlots.R
\name{plotMDS}
\alias{plotMDS}
\title{MDS plots showing grouping of samples after normalizations}
\usage{
plotMDS(nr, currentLayout, pageno)
}
\arguments{
\item{nr}{Normalyzer results object.}

\item{currentLayout}{Layout used for document.}

\item{pageno}{Current page number.}
}
\value{
None
}
\description{
MDS plots showing grouping of samples after normalizations
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/higherOrderNormMethods.R
\name{getCombinedMatrix}
\alias{getCombinedMatrix}
\title{Merge multiple dataframes using provided function}
\usage{
getCombinedMatrix(mList, combFunc)
}
\arguments{
\item{mList}{List containing dataframes of same shape}

\item{combFunc}{Function performing elementwise merge of matrices}
}
\value{
combinedMatrix A single dataframe with combined data
}
\description{
Merge multiple dataframes using provided function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NormalyzerResults.R
\name{performNormalizations}
\alias{performNormalizations}
\alias{performNormalizations,NormalyzerResults-method}
\title{Main function for executing normalizations}
\usage{
performNormalizations(
  nr,
  forceAll = FALSE,
  rtNorm = FALSE,
  rtStepSizeMinutes = 1,
  rtWindowMinCount = 100,
  rtWindowShifts = 1,
  rtWindowMergeMethod = "median",
  noLogTransform = FALSE,
  quiet = FALSE
)

\S4method{performNormalizations}{NormalyzerResults}(
  nr,
  forceAll = FALSE,
  rtNorm = FALSE,
  rtStepSizeMinutes = 1,
  rtWindowMinCount = 100,
  rtWindowShifts = 1,
  rtWindowMergeMethod = "median",
  noLogTransform = FALSE,
  quiet = FALSE
)
}
\arguments{
\item{nr}{Normalyzer results object.}

\item{forceAll}{Ignore dataset size limits and run all normalizations
(only meant for testing purposes)}

\item{rtNorm}{Perform retention time based normalizations}

\item{rtStepSizeMinutes}{Retention time normalization window size.}

\item{rtWindowMinCount}{Minimum number of datapoints in each retention-time
segment.}

\item{rtWindowShifts}{Number of layered retention time normalized windows.}

\item{rtWindowMergeMethod}{Merge approach for layered retention time windows.}

\item{noLogTransform}{Prevent log-transforming input}

\item{quiet}{Don't show regular output messages}
}
\value{
nr NormalyzerDE results object
}
\description{
Main function for executing normalizations
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generatePlots.R
\name{plotSampleOutlierSummary}
\alias{plotSampleOutlierSummary}
\title{Write page containing sample summary of intensities, missing values and 
MDS plot to the viewport}
\usage{
plotSampleOutlierSummary(nr, currentLayout, pageno)
}
\arguments{
\item{nr}{Normalyzer results object.}

\item{currentLayout}{Layout used for document.}

\item{pageno}{Current page number.}
}
\value{
None
}
\description{
Write page containing sample summary of intensities, missing values and 
MDS plot to the viewport
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generatePlots.R
\name{plotMA}
\alias{plotMA}
\title{Produces a page containing expression vs. fold-change figures (MA plots)
The visualized fold is between the first sample in each group and the
average of the replicate to which that sample belongs}
\usage{
plotMA(nr, currentLayout, pageno)
}
\arguments{
\item{nr}{Normalyzer results object.}

\item{currentLayout}{Layout used for document.}

\item{pageno}{Current page number.}
}
\value{
None
}
\description{
Produces a page containing expression vs. fold-change figures (MA plots)
The visualized fold is between the first sample in each group and the
average of the replicate to which that sample belongs
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normMethods.R
\name{performSMADNormalization}
\alias{performSMADNormalization}
\title{Median absolute deviation normalization
Normalization subtracts the median and divides the data by the
median absolute deviation (MAD).}
\usage{
performSMADNormalization(rawMatrix, noLogTransform = FALSE)
}
\arguments{
\item{rawMatrix}{Target matrix to be normalized}

\item{noLogTransform}{Assumes no need for log transformation}
}
\value{
Normalized matrix
}
\description{
Median absolute deviation normalization
Normalization subtracts the median and divides the data by the
median absolute deviation (MAD).
}
\examples{
data(example_data_only_values_small)
normMatrix <- performSMADNormalization(example_data_only_values)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normMethods.R
\name{performGlobalRLRNormalization}
\alias{performGlobalRLRNormalization}
\title{Global linear regression normalization}
\usage{
performGlobalRLRNormalization(rawMatrix, noLogTransform = FALSE)
}
\arguments{
\item{rawMatrix}{Target matrix to be normalized}

\item{noLogTransform}{Assumes no need for log transformation}
}
\value{
Normalized matrix
}
\description{
Log2 transformed data is normalized by robust linear regression using
the function "rlm" from the MASS package.
}
\examples{
data(example_data_only_values_small)
normMatrix <- performGlobalRLRNormalization(example_data_only_values)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyzeResults.R
\name{calculateAvgMadMem}
\alias{calculateAvgMadMem}
\title{Calculate average MAD (Median Absolute Deviation) for each feature in
each condition and then calculates the average for each replicate group}
\usage{
calculateAvgMadMem(methodList, sampleReplicateGroups)
}
\arguments{
\item{methodList}{List containing normalized matrices.}

\item{sampleReplicateGroups}{Condition header.}
}
\value{
condAvgMadMat Matrix with average MAD for each biological condition.
}
\description{
Calculate average MAD (Median Absolute Deviation) for each feature in
each condition and then calculates the average for each replicate group
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyzeResults.R
\name{calculateAvgReplicateVariation}
\alias{calculateAvgReplicateVariation}
\title{Calculate average variance for each feature in each condition and then
calculate the average for each replicate group}
\usage{
calculateAvgReplicateVariation(methodList, sampleReplicateGroups)
}
\arguments{
\item{methodList}{List containing normalized matrices.}

\item{sampleReplicateGroups}{Condition header.}
}
\value{
avgVarianceMat Matrix with average variance for each biological
condition
}
\description{
Calculate average variance for each feature in each condition and then
calculate the average for each replicate group
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NormalyzerDataset.R
\name{detectSingletonSample}
\alias{detectSingletonSample}
\alias{detectSingletonSample,NormalyzerDataset-method}
\title{Detect single sample group}
\usage{
detectSingletonSample(nds, quiet)

\S4method{detectSingletonSample}{NormalyzerDataset}(nds, quiet = FALSE)
}
\arguments{
\item{nds}{Normalyzer dataset.}

\item{quiet}{Only print error messages}
}
\value{
None
}
\description{
Detect single sample group
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyzeResults.R
\name{calculateReplicateCV}
\alias{calculateReplicateCV}
\title{Calculate CV per replicate group and normalization technique}
\usage{
calculateReplicateCV(methodList, sampleReplicateGroups)
}
\arguments{
\item{methodList}{List containing normalized matrices.}

\item{sampleReplicateGroups}{Condition header.}
}
\value{
avgCVPerNormAndReplicates Matrix with group CVs as rows and
  normalization technique as columns
}
\description{
Iterates through each normalization method and calculate average CV values
per replicate group.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{getReplicateSortedData}
\alias{getReplicateSortedData}
\title{Get dataframe with raw data column sorted on replicates}
\usage{
getReplicateSortedData(rawDataOnly, groups)
}
\arguments{
\item{rawDataOnly}{Dataframe with unparsed input data matrix.}

\item{groups}{Vector containing condition levels.}
}
\value{
rawData sorted on replicate
}
\description{
Get dataframe with raw data column sorted on replicates
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputVerification.R
\name{loadRawDataFromFile}
\alias{loadRawDataFromFile}
\title{Try reading raw Normalyzer matrix from provided filepath}
\usage{
loadRawDataFromFile(inputPath)
}
\arguments{
\item{inputPath}{Path to Normalyzer data.}
}
\value{
Table containing raw data from input file.
}
\description{
Try reading raw Normalyzer matrix from provided filepath
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{example_data}
\alias{example_data}
\title{Small example dataset used to demonstrate code consistency in testing and
as dummy data in the vignette.}
\format{
A data frame containing annotation and expression data
}
\usage{
example_data
}
\description{
Small example dataset used to demonstrate code consistency in testing and
as dummy data in the vignette.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateStatistics.R
\name{plotContrastPHists}
\alias{plotContrastPHists}
\title{Takes an NormalyzerStatistics instance and generates and prints a p-value
histogram for each onto the viewport}
\usage{
plotContrastPHists(nst, jobName, currentLayout, pageno)
}
\arguments{
\item{nst}{NormalyzerDE statistics object.}

\item{jobName}{Name of processing run.}

\item{currentLayout}{Layout used for document.}

\item{pageno}{Current page number.}
}
\value{
None
}
\description{
Takes an NormalyzerStatistics instance and generates and prints a p-value
histogram for each onto the viewport
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyzeResults.R
\name{calculateANOVAPValues}
\alias{calculateANOVAPValues}
\title{Calculates ANOVA p-values comparing the different condition groups
returning a vector of resulting p-values with NA-values where too few
values were present in at least one of the groups.}
\usage{
calculateANOVAPValues(methodList, sampleReplicateGroups, categoricalANOVA)
}
\arguments{
\item{methodList}{List containing normalized matrices}

\item{sampleReplicateGroups}{Condition header}

\item{categoricalANOVA}{Whether the ANOVA should be calculated using ordered
or categorical groups}
}
\value{
avgVarianceMat Matrix with average variance for each biological
condition
}
\description{
Calculates ANOVA p-values comparing the different condition groups
returning a vector of resulting p-values with NA-values where too few
values were present in at least one of the groups.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generatePlots.R
\name{setupPlotting}
\alias{setupPlotting}
\title{Setup PDF report settings by initializing the color palette, format
for the PDF report and the graphical device}
\usage{
setupPlotting(currentJob, jobDir, suffix)
}
\arguments{
\item{currentJob}{Name of current run.}

\item{jobDir}{Path to output directory for run.}

\item{suffix}{Text to add to output filename.}
}
\value{
None
}
\description{
Setup PDF report settings by initializing the color palette, format
for the PDF report and the graphical device
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generatePlots.R
\name{plotCorrelation}
\alias{plotCorrelation}
\title{Visualize within-replicates correlations}
\usage{
plotCorrelation(nr, currentLayout, pageno)
}
\arguments{
\item{nr}{Normalyzer results object.}

\item{currentLayout}{Layout used for document.}

\item{pageno}{Current page number.}
}
\value{
None
}
\description{
Visualize within-replicates correlations
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateStatistics.R
\name{plotSigScatter}
\alias{plotSigScatter}
\title{Takes an NormalyzerStatistics instance and generates and prints a 
volcano plot}
\usage{
plotSigScatter(
  nst,
  jobName,
  currentLayout,
  pageno,
  type = "Volcano",
  sigThres = 0.1,
  sigThresType = "fdr",
  log2FoldThres = 0
)
}
\arguments{
\item{nst}{NormalyzerDE statistics object.}

\item{jobName}{Name of processing run.}

\item{currentLayout}{Layout used for document.}

\item{pageno}{Current page number.}

\item{type}{Specify whether to plot 'Volcano' or 'MA'.}

\item{sigThres}{FDR threshold for DE coloring.}
}
\value{
None
}
\description{
Takes an NormalyzerStatistics instance and generates and prints a 
volcano plot
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/higherOrderNormMethods.R
\name{getRTNormalizedMatrix}
\alias{getRTNormalizedMatrix}
\title{Perform RT-segmented normalization by performing the supplied normalization
over retention-time sliced data}
\usage{
getRTNormalizedMatrix(
  rawMatrix,
  retentionTimes,
  normMethod,
  stepSizeMinutes = 1,
  windowMinCount = 100,
  offset = 0,
  noLogTransform = FALSE
)
}
\arguments{
\item{rawMatrix}{Target matrix to be normalized}

\item{retentionTimes}{Vector of retention times corresponding to rawMatrix}

\item{normMethod}{The normalization method to apply to the time windows}

\item{stepSizeMinutes}{Size of windows to be normalized}

\item{windowMinCount}{Minimum number of values for window to not be expanded.}

\item{offset}{Whether time window should shifted half step size}

\item{noLogTransform}{Don't log-transform the data}
}
\value{
Normalized matrix
}
\description{
The function orders the retention times and steps through them using the
supplied step size (in minutes). If smaller than a fixed lower boundary 
the window is expanded to ensure a minimum amount of data in each 
normalization step. An offset can be specified which can be used to perform
multiple RT-segmentations with partial overlapping windows.
}
\examples{
data(example_data_small)
data(example_design_small)
data(example_data_only_values)
dataMat <- example_data_only_values
retentionTimes <- as.numeric(example_data[, "Average.RT"])
performCyclicLoessNormalization <- function(rawMatrix) {
    log2Matrix <- log2(rawMatrix)
    normMatrix <- limma::normalizeCyclicLoess(log2Matrix, method="fast")
    colnames(normMatrix) <- colnames(rawMatrix)
    normMatrix
}
rtNormMat <- getRTNormalizedMatrix(dataMat, retentionTimes, 
performCyclicLoessNormalization, stepSizeMinutes=1, windowMinCount=100)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generatePlots.R
\name{plotPHist}
\alias{plotPHist}
\title{Generate P-histograms for ANOVA calculated after each normalization}
\usage{
plotPHist(nr, currentLayout, pageno)
}
\arguments{
\item{nr}{Normalyzer results object.}

\item{currentLayout}{Layout used for document.}

\item{pageno}{Current page number.}
}
\value{
None
}
\description{
Generate P-histograms for ANOVA calculated after each normalization
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputVerification.R
\name{setupRawDataObject}
\alias{setupRawDataObject}
\title{Prepare SummarizedExperiment object for raw data to be normalized containing 
data, design and annotation information}
\usage{
setupRawDataObject(
  dataPath,
  designPath,
  inputFormat = "default",
  zeroToNA = FALSE,
  sampleColName = "sample",
  groupColName = "group"
)
}
\arguments{
\item{dataPath}{File path to data matrix.}

\item{designPath}{File path to design matrix.}

\item{inputFormat}{Type of matrix for data, can be either 'default',
'proteios', 'maxquantprot' or 'maxquantpep'}

\item{zeroToNA}{If TRUE zeroes in the data is automatically converted to
NA values}

\item{sampleColName}{Column name for column containing sample names}

\item{groupColName}{Column name for column containing condition levels}
}
\value{
experimentObj SummarizedExperiment object loaded with the data
}
\description{
Prepare SummarizedExperiment object for raw data to be normalized containing 
data, design and annotation information
}
\examples{
data_path <- system.file(package="NormalyzerDE", "extdata", "tiny_data.tsv")
design_path <- system.file(package="NormalyzerDE", "extdata", "tiny_design.tsv")
df <- setupRawDataObject(data_path, design_path)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{example_design}
\alias{example_design}
\title{Design matrix corresponding to the small example datasets.}
\format{
A design matrix corresponding to the dataset "example_data"
}
\usage{
example_design
}
\description{
Design matrix corresponding to the small example datasets.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NormalyzerDE.R
\name{normalyzerDE}
\alias{normalyzerDE}
\title{NormalyzerDE differential expression}
\usage{
normalyzerDE(
  jobName,
  comparisons,
  designPath = NULL,
  dataPath = NULL,
  experimentObj = NULL,
  outputDir = ".",
  logTrans = FALSE,
  type = "limma",
  sampleCol = "sample",
  condCol = "group",
  batchCol = NULL,
  techRepCol = NULL,
  leastRepCount = 1,
  quiet = FALSE,
  sigThres = 0.1,
  sigThresType = "fdr",
  log2FoldThres = 0,
  writeReportAsPngs = FALSE
)
}
\arguments{
\item{jobName}{Name of job}

\item{comparisons}{Character vector containing target contrasts. 
If comparing condA with condB, then the vector would be c("condA-condB")}

\item{designPath}{File path to design matrix}

\item{dataPath}{File path to normalized matrix}

\item{experimentObj}{SummarizedExperiment object, can be provided as input
as alternative to 'designPath' and 'dataPath'}

\item{outputDir}{Path to output directory}

\item{logTrans}{Log transform the input (needed if providing non-logged 
input)}

\item{type}{Type of statistical comparison, "limma", "limma_intensity" or 
"welch", where "limma_intensity" allows the prior to be fit according to
intensity rather than using a flat prior}

\item{sampleCol}{Design matrix column header for column containing sample IDs}

\item{condCol}{Design matrix column header for column containing sample 
conditions}

\item{batchCol}{Provide an optional column for inclusion of possible batch 
variance in the model}

\item{techRepCol}{Design matrix column header for column containing technical 
replicates}

\item{leastRepCount}{Minimum required replicate count}

\item{quiet}{Omit status messages printed during run}

\item{sigThres}{Significance threshold use for illustrating significant hits
in diagnostic plots}

\item{sigThresType}{Type of significance threshold, "fdr" or "p". "fdr" is
strongly recommended (Benjamini-Hochberg corrected p-values)}

\item{log2FoldThres}{Fold-size cutoff for being considered significant in
diagnostic plots}

\item{writeReportAsPngs}{Output report as separate PNG files instead of a
single PDF}
}
\value{
None
}
\description{
Performs differential expression analysis on a normalization matrix.
This command executes a pipeline processing the data and generates an
annotated normalization matrix and a report containing p-value histograms
for each of the performed comparisons.
}
\details{
When executed, it performs the following steps:

1: Read the data and the design matrices into dataframes.
2: Generate an instance of the NormalyzerStatistics class representing the
data and their statistical comparisons.
3: Optionally reduce technical replicates in both the data matrix and the 
design matrix
4: Calculate statistical contrats between supplied groups
5: Generate an annotated version of the original dataframe where columns
containing statistical key measures have been added
6: Write the table to file
7: Generate a PDF report displaying p-value histograms for each calculated
contrast
}
\examples{
data_path <- system.file(package="NormalyzerDE", "extdata", "tiny_data.tsv")
design_path <- system.file(package="NormalyzerDE", "extdata", "tiny_design.tsv")
out_dir <- tempdir()
normalyzerDE(
  jobName="my_jobname", 
  comparisons=c("4-5"), 
  designPath=design_path, 
  dataPath=data_path,
  outputDir=out_dir,
  condCol="group")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyzeResults.R
\name{findLowlyVariableFeaturesCVs}
\alias{findLowlyVariableFeaturesCVs}
\title{Uses a list of FDR-values to extract features with low variance in the
log2-transformed dataset. This is then used to calculate the average CV
for these 'lowly variable' features in each normalization approach}
\usage{
findLowlyVariableFeaturesCVs(referenceFDR, methodList)
}
\arguments{
\item{referenceFDR}{List of FDR values used as non-normalized reference}

\item{methodList}{List containing normalized matrices}
}
\value{
lowVarFeaturesAverageCVs Average CV values for lowly variable
features in each normalization approach
}
\description{
Uses a list of FDR-values to extract features with low variance in the
log2-transformed dataset. This is then used to calculate the average CV
for these 'lowly variable' features in each normalization approach
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NormalyzerResults.R
\name{NormalyzerResults}
\alias{NormalyzerResults}
\title{Representation of the results from performing normalization over a dataset}
\usage{
NormalyzerResults(nds)

NormalyzerResults(nds)
}
\arguments{
\item{nds}{NormalyzerDataset object}
}
\value{
nr Prepared NormalyzerResults object
}
\description{
It is linked to a NormalyzerDataset instance representing the raw data
which has been processed. After performing evaluation it also links to
an instance of NormalyzerEvaluationResults representing the results
from the evaluation.
}
\section{Slots}{

\describe{
\item{\code{normalizations}}{SummarizedExperiment object containing calculated
normalization results}

\item{\code{nds}}{Normalyzer dataset representing run data}

\item{\code{ner}}{Normalyzer evaluation results
for running extended normalizations}
}}

\examples{
data(example_summarized_experiment)
normObj <- getVerifiedNormalyzerObject("job_name", example_summarized_experiment)
emptyNormResults <- NormalyzerResults(normObj)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generatePlots.R
\name{plotCVvsIntensity}
\alias{plotCVvsIntensity}
\title{Plots page displaying coefficient of variance (CV) against raw intensity
for features across the performed normalizations}
\usage{
plotCVvsIntensity(nr, currentLayout, pageno)
}
\arguments{
\item{nr}{Normalyzer results object.}

\item{currentLayout}{Layout used for document.}

\item{pageno}{Current page number.}
}
\value{
None
}
\description{
Plots page displaying coefficient of variance (CV) against raw intensity
for features across the performed normalizations
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateStatistics.R
\name{reduceTechnicalReplicates}
\alias{reduceTechnicalReplicates}
\title{Remove technical replicates from data and design}
\usage{
reduceTechnicalReplicates(se, techRepColName, sampleColName)
}
\arguments{
\item{se}{Summarized experiment where the assay contains the data to be
reduced, and the colData the data frame}

\item{techRepColName}{Technical replicates column name in colData}

\item{sampleColName}{Sample names column name in colData}
}
\value{
reducedSe Summarized experiment with reduced data
}
\description{
Collapses sample values into their average. If only one value is present
due to NA-values in other technical replicates, then that value is used.
}
\details{
Takes a SummarizedExperiment where the data is present as the assay
and the colData contains the design conditions. In the design conditions
there should be one column with the technical replicate groups and one
column containing the sample names
}
\examples{
testData <- as.matrix(data.frame(
    c(1,1,1), 
    c(1,2,1), 
    c(7,7,7), 
    c(7,9,7)))
colnames(testData) <- c("a1", "a2", "b1", "b2")
designDf <- data.frame(
    sample=c("a1", "a2", "b1", "b2"), 
    techrep=c("a", "a", "b", "b"))
se <- SummarizedExperiment::SummarizedExperiment(
    assay=testData,
    colData=designDf
)
statObj <- reduceTechnicalReplicates(se, "techrep", "sample")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputVerification.R
\name{loadDesign}
\alias{loadDesign}
\title{Load raw design into dataframe}
\usage{
loadDesign(designPath, sampleCol = "sample", groupCol = "group")
}
\arguments{
\item{designPath}{File path to design matrix.}

\item{sampleCol}{Column name for column containing sample names.}

\item{groupCol}{Column name for column containing condition levels.}
}
\value{
designMatrix Design data loaded into data frame
}
\description{
Takes a design path, loads the matrix and ensures that the sample column
is in character format and that the group column is in factor format.
}
\examples{
\dontrun{
df <- loadDesign("design.tsv")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normMethods.R
\name{performNoNormalization}
\alias{performNoNormalization}
\title{Do no normalization (For debugging purposes)}
\usage{
performNoNormalization(rawMatrix)
}
\arguments{
\item{rawMatrix}{Target matrix to be normalized}
}
\value{
Normalized matrix
}
\description{
Do no normalization (For debugging purposes)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateStatistics.R
\name{plotComparisonVenns}
\alias{plotComparisonVenns}
\title{If multiple comparisons - Show overlap in Venn diagrams}
\usage{
plotComparisonVenns(
  nst,
  jobName,
  currentLayout,
  pageno,
  sigThres = 0.1,
  sigThresType = "fdr",
  log2FoldThres = 0,
  maxContrasts = 4
)
}
\arguments{
\item{nst}{NormalyzerDE statistics object.}

\item{jobName}{Name of processing run.}

\item{currentLayout}{Layout used for document.}

\item{pageno}{Current page number.}

\item{sigThres}{Cutoff value for significance theshold}

\item{sigThresType}{Type of significance cutoff}

\item{log2FoldThres}{Log2-fold based cutoff threshold}

\item{maxContrasts}{Maximum contrasts to show pairwise comparisons for}
}
\value{
None
}
\description{
If multiple comparisons - Show overlap in Venn diagrams
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{elapsedSecondsBetweenSystimes}
\alias{elapsedSecondsBetweenSystimes}
\title{Get number of seconds between two Sys.time() objects}
\usage{
elapsedSecondsBetweenSystimes(start, end)
}
\arguments{
\item{start}{Start-time object}

\item{end}{End-time object}
}
\value{
None
}
\description{
Get number of seconds between two Sys.time() objects
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normMethods.R
\name{globalIntensityNormalization}
\alias{globalIntensityNormalization}
\title{The normalization divides the intensity of each variable in a sample
with the sum of intensities of all variables in the sample and multiplies
with the median of sum of intensities of all variables in all samples.
The normalized data is then log2-transformed.}
\usage{
globalIntensityNormalization(rawMatrix, noLogTransform = FALSE)
}
\arguments{
\item{rawMatrix}{Target matrix to be normalized}

\item{noLogTransform}{Assumes no need for log transformation}
}
\value{
Normalized and log-transformed matrix
}
\description{
The normalization divides the intensity of each variable in a sample
with the sum of intensities of all variables in the sample and multiplies
with the median of sum of intensities of all variables in all samples.
The normalized data is then log2-transformed.
}
\examples{
data(example_data_only_values_small)
normMatrix <- globalIntensityNormalization(example_data_only_values)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normMethods.R
\name{normMethods}
\alias{normMethods}
\title{Perform normalizations on Normalyzer dataset}
\usage{
normMethods(
  nds,
  forceAll = FALSE,
  normalizeRetentionTime = TRUE,
  quiet = FALSE,
  rtStepSizeMinutes = 1,
  rtWindowMinCount = 100,
  rtWindowShifts = 1,
  rtWindowMergeMethod = "mean",
  noLogTransform = FALSE
)
}
\arguments{
\item{nds}{Normalyzer dataset object.}

\item{forceAll}{Force all methods to run despite not qualifying for thresholds.}

\item{normalizeRetentionTime}{Perform retention time based normalization methods.}

\item{quiet}{Prevent diagnostic output}

\item{rtStepSizeMinutes}{Retention time normalization window size.}

\item{rtWindowMinCount}{Minimum number of datapoints in each retention-time
segment.}

\item{rtWindowShifts}{Number of layered retention time normalized windows.}

\item{rtWindowMergeMethod}{Merge approach for layered retention time windows.}

\item{noLogTransform}{Per default NormalyzerDE performs a log-transformation
on the input data. If not needed, specify this option}
}
\value{
Returns Normalyzer results object with performed analyzes assigned
 as attributes
}
\description{
Perform normalizations on Normalyzer dataset
}
\examples{
data(example_summarized_experiment)
normObj <- getVerifiedNormalyzerObject("job_name", example_summarized_experiment)
normResults <- normMethods(normObj)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyzeResults.R
\name{calculateCorrSum}
\alias{calculateCorrSum}
\title{Calculates internal correlations for each condition having at least two
samples and returns a vector with correlation values corresponding to each
condition}
\usage{
calculateCorrSum(
  methodData,
  allReplicateGroups,
  sampleGroupsWithReplicates,
  corrType
)
}
\arguments{
\item{methodData}{Expression data matrix}

\item{allReplicateGroups}{Full condition header corresponding to data tables
columns}

\item{sampleGroupsWithReplicates}{Unique conditions where number of
replicates exceeds one}

\item{corrType}{Type of correlation (Pearson or Spearman)}
}
\value{
corSums
}
\description{
Calculates internal correlations for each condition having at least two
samples and returns a vector with correlation values corresponding to each
condition
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generatePlots.R
\name{plotSampleMappingPage}
\alias{plotSampleMappingPage}
\title{Write page with sample mapping}
\usage{
plotSampleMappingPage(nr, currentFont, currentLayout, currentjob, pageno)
}
\arguments{
\item{nr}{Normalyzer results object.}

\item{currentLayout}{Layout used for document.}

\item{pageno}{Current page number.}
}
\value{
None
}
\description{
Write page with sample mapping
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NormalyzerStatistics.R
\name{calculateContrasts}
\alias{calculateContrasts}
\alias{calculateContrasts,NormalyzerStatistics-method}
\title{Performs statistical comparisons between the supplied conditions.
It uses the design matrix and data matrix in the supplied 
NormalyzerStatistics object. A column is supplied specifying which of the
columns in the design matrix that is used for deciding the sample groups.
The comparisons vector specifies which pairwise comparisons between
condition levels that are to be calculated.}
\usage{
calculateContrasts(
  nst,
  comparisons,
  condCol,
  batchCol = NULL,
  splitter = "-",
  type = "limma",
  leastRepCount = 1
)

\S4method{calculateContrasts}{NormalyzerStatistics}(
  nst,
  comparisons,
  condCol,
  batchCol = NULL,
  splitter = "-",
  type = "limma",
  leastRepCount = 1
)
}
\arguments{
\item{nst}{Results evaluation object.}

\item{comparisons}{String with comparisons for contrasts.}

\item{condCol}{Column name in design matrix containing condition information.}

\item{batchCol}{Column name in design matrix containing batch information.}

\item{splitter}{Character dividing contrast conditions.}

\item{type}{Type of statistical test (Limma or welch).}

\item{leastRepCount}{Least replicates in each group to be retained for 
contrast calculations}
}
\value{
nst Statistics object with statistical measures calculated
}
\description{
Optionally, a batch column can be specified allowing compensation for
covariate variation in the statistical model. This is only compatible
with a Limma-based statistical analysis.
}
\examples{
data(example_stat_summarized_experiment)
nst <- NormalyzerStatistics(example_stat_summarized_experiment)
results <- calculateContrasts(nst, c("1-2", "2-3"), "group")
resultsBatch <- calculateContrasts(nst, c("1-2", "2-3"), "group", batchCol="batch")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NormalyzerDataset.R
\name{detectSingleReplicate}
\alias{detectSingleReplicate}
\alias{detectSingleReplicate,NormalyzerDataset-method}
\title{Detect single replicate, and assign related logical}
\usage{
detectSingleReplicate(nds, quiet)

\S4method{detectSingleReplicate}{NormalyzerDataset}(nds, quiet = FALSE)
}
\arguments{
\item{nds}{Normalyzer dataset}

\item{quiet}{Don't give non-error output}
}
\value{
bool on whether sample contains only one sample group
}
\description{
Detect single replicate, and assign related logical
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/outputUtils.R
\name{writeNormalizedDatasets}
\alias{writeNormalizedDatasets}
\title{Write normalization matrices to file}
\usage{
writeNormalizedDatasets(
  nr,
  jobdir,
  includePairwiseComparisons = FALSE,
  includeCvCol = FALSE,
  includeAnovaP = FALSE,
  normSuffix = "-normalized.txt",
  rawdataName = "submitted_rawdata.txt"
)
}
\arguments{
\item{nr}{Results object.}

\item{jobdir}{Path to output directory.}

\item{includePairwiseComparisons}{Include p-values for pairwise comparisons.}

\item{includeCvCol}{Include CV column in output.}

\item{includeAnovaP}{Include ANOVA p-value in output.}

\item{normSuffix}{String used to name output together with normalization names.}

\item{rawdataName}{Name of output raw data file.}
}
\value{
None
}
\description{
Outputs each of the normalized datasets to the specified directory.
}
\examples{
data(example_summarized_experiment)
normObj <- getVerifiedNormalyzerObject("job_name", example_summarized_experiment)
normResults <- normMethods(normObj)
normResultsWithEval <- analyzeNormalizations(normResults)
outputDir <- tempdir()
writeNormalizedDatasets(normResultsWithEval, outputDir)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyzeResults.R
\name{analyzeNormalizations}
\alias{analyzeNormalizations}
\title{Calculate measures for normalization results}
\usage{
analyzeNormalizations(nr, categoricalAnova = FALSE)
}
\arguments{
\item{nr}{Normalyzer results object with calculated results.}

\item{categoricalAnova}{Whether categorical or numerical (ordered) ANOVA
should be calculated.}
}
\value{
Normalyzer results with attached evaluation results object.
}
\description{
This function prepares an NormalyzerEvaluationResults object containing
the evaluation measures CV (coefficient of variance), MAD (median absolute
deviation), average variance, significance measures (ANOVA between
condition groups) and correlation between replicates.
}
\examples{
data(example_summarized_experiment)
normObj <- getVerifiedNormalyzerObject("job_name", example_summarized_experiment)
normResults <- normMethods(normObj)
normResultsWithEval <- analyzeNormalizations(normResults)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{example_wide_design}
\alias{example_wide_design}
\title{Design matrix belonging together with example_wide_data. Used for
internal testing.}
\format{
A design table ready for analysis in NormalyzerDE
}
\usage{
example_wide_design
}
\description{
Design matrix belonging together with example_wide_data. Used for
internal testing.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generatePlots.R
\name{generatePlots}
\alias{generatePlots}
\title{Generates a number of visualizations for the performance measures calculated
for the normalized matrices. These contain both general measures and
direct comparisons for different normalization approaches.}
\usage{
generatePlots(nr, jobdir, plotRows = 3, plotCols = 4, writeAsPngs = FALSE)
}
\arguments{
\item{nr}{Normalyzer results object.}

\item{jobdir}{Path to output directory for run.}

\item{plotRows}{Number of plot rows.}

\item{plotCols}{Number of plot columns.}

\item{writeAsPngs}{Output the report as PNG-plots instead of a single PDF}
}
\value{
None
}
\description{
They include:
}
\details{
"Total intensity" 
Barplot showing the summed intensity in each sample for thelog2-transformed 
data

"Total missing" 
Barplot showing the number of missing values found in each sample for the 
log2-tranformed data

Log2-MDS plot: MDS plot where data is reduced to two dimensions allowing 
inspection of the main global changes in the data

PCV - Intragroup: Mean of intragroup CV of all replicate groups

PMAD - Intragroup: Mean of intragroup median absolute deviation across 
replicate groups

PEV - Intragroup: Mean of intragroup pooled estimate of variance across the 
replicate groups

Relative PCV, PMAD and PEV compared to log2: The results from PCV, PMAD
and PEV from all normalized data compared to the log2 data

Stable variables plot:
5% of least differentially expressed variables are identified by ANOVA
analysis of log2 transformed data. Thereafter, global CV of these variables
is estimated from different normalized datasets. A plot of global CV of the
stable variables from all datsets on the y-axis and PCV-compared to log2 on
the x-axis is generated.

CV vs Raw Intensity plots:
For the first replicate group in each of the normalized dataset, a plot of
PCV of each variable compared to the average intensity of the variable in
the replicate group is plotted.

MA plots:
Plotted using the plotMA function of the limma package. The first sample in
each dataset is plotted against the average of the replicate group that
sample belong to.

Scatterplots:
The first two samples from each dataset are plotted.

Q-Q plots:
QQ-plots are plotted for the first sample in each normalized dataset.

Boxplots:
Boxplots for all samples are plotted and colored according to the replicate
grouping.

Relative Log Expression (RLE) plots:
Relative log expression value plots. Ratio between the expression of the
variable and the median expression of this variable across all samples.
The samples should be aligned around zero. Any deviation would indicate
discrepancies in the data.

Density plots:
Density distributions for each sample using the density function. Can 
capture outliers (if single densities lies far from the others) and see
if there is batch effects in the dataset (if for instance there is two
clear collections of lines in the data).

MDS plots
Multidimensional scaling plot using the cmdscale() function from the stats
package. Is often able to show whether replicates group together, and
whether there are any clear outliers in the data.

MeanSDplots
Displays the standard deviation values against values ordered according
to mean. If no dependency on mean is present (as is desired) a flat red
line is shown.

Pearson and Spearman correlation
Mean of intragroup Pearson and Spearman correlation values for each method.

Dendograms
Generated using the hclust function. Data is centered and scaled prior to
analysis. Coloring of replicates is done using as.phylo from the ape package.

P-value histograms
Histogram plots of p-values after calculating an ANOVA between different
condition groups. If no effect is present in the data a flat distribution
is expected. If an effect is present a flat distribution is still expected,
but with a sharp peak close to zero. If other effects are present it might
indicate that the data doesn't support the assumptions of ANOVA, for
instance if there are batch effects present in the data.
}
\examples{
data(example_summarized_experiment)
normObj <- getVerifiedNormalyzerObject("job_name", example_summarized_experiment)
normResults <- normMethods(normObj)
normResultsWithEval <- analyzeNormalizations(normResults)
outputDir <- tempdir()
generatePlots(normResultsWithEval, outputDir)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{example_wide_data}
\alias{example_wide_data}
\title{Full raw NormalyzerDE matrix used for internal testing}
\format{
A data table ready for analysis in NormalyzerDE
}
\usage{
example_wide_data
}
\description{
Full raw NormalyzerDE matrix used for internal testing
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/higherOrderNormMethods.R
\name{getWidenedRTRange}
\alias{getWidenedRTRange}
\title{Pick datapoints before and after window until a minimum number is reached
Expects the start and end retention times to match actual retention times
present in the data}
\usage{
getWidenedRTRange(
  rtStart,
  rtEnd,
  minimumDatapoints,
  retentionTimes,
  allowTooWideData = FALSE
)
}
\arguments{
\item{rtStart}{Original retention time start point}

\item{rtEnd}{Original retention time end point}

\item{minimumDatapoints}{Required number of datapoints to fulfill}

\item{retentionTimes}{Vector with all retention times}
}
\value{
Vector with start and end of new RT range
}
\description{
Pick datapoints before and after window until a minimum number is reached
Expects the start and end retention times to match actual retention times
present in the data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputVerification.R
\name{getVerifiedNormalyzerObject}
\alias{getVerifiedNormalyzerObject}
\title{Verify that input data is in correct format, and if so, return a generated
 NormalyzerDE data object from that input data}
\usage{
getVerifiedNormalyzerObject(
  jobName,
  summarizedExp,
  threshold = 15,
  omitSamples = FALSE,
  requireReplicates = TRUE,
  quiet = FALSE,
  noLogTransform = FALSE,
  tinyRunThres = 50
)
}
\arguments{
\item{jobName}{Name of ongoing run.}

\item{summarizedExp}{Summarized experiment input object}

\item{threshold}{Minimum number of features.}

\item{omitSamples}{Automatically omit invalid samples from analysis.}

\item{requireReplicates}{Require there to be at least to samples per
condition}

\item{quiet}{Don't print output messages during processing}

\item{noLogTransform}{Don't log-transform the provided data}

\item{tinyRunThres}{If less features in run, a limited run is performed}
}
\value{
Normalyzer data object representing verified input data.
}
\description{
This function performs a number of checks on the input data and provides
informative error messages if the data isn't fulfilling the required format.
Checks include verifying that the design matrix matches to the data matrix,
that the data matrix contains valid numbers and that samples have enough
values for analysis
}
\examples{
data(example_summarized_experiment)
normObj <- getVerifiedNormalyzerObject("job_name", example_summarized_experiment)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generatePlots.R
\name{plotMeanSD}
\alias{plotMeanSD}
\title{Visualize standard deviation over (expression?) for different values}
\usage{
plotMeanSD(nr, currentLayout, pageno)
}
\arguments{
\item{nr}{Normalyzer results object.}

\item{currentLayout}{Layout used for document.}

\item{pageno}{Current page number.}
}
\value{
None
}
\description{
Visualize standard deviation over (expression?) for different values
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normMethods.R
\name{performVSNNormalization}
\alias{performVSNNormalization}
\title{Log2 transformed data is normalized using the function "justvsn" from the
VSN package.}
\usage{
performVSNNormalization(rawMatrix)
}
\arguments{
\item{rawMatrix}{Target matrix to be normalized}
}
\value{
Normalized matrix
}
\description{
The VSN (Variance Stabilizing Normalization) attempts to transform the data 
in such a way that the variance remains nearly constant over the intensity 
spectrum
}
\examples{
data(example_data_only_values_small)
normMatrix <- performVSNNormalization(example_data_only_values)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputVerification.R
\name{validateSampleReplication}
\alias{validateSampleReplication}
\title{Check whether all samples have replicates}
\usage{
validateSampleReplication(
  dataMatrix,
  groups,
  requireReplicates = TRUE,
  quiet = FALSE
)
}
\arguments{
\item{dataMatrix}{Prepared matrix containing expression data.}

\item{groups}{Vector containing condition levels}

\item{requireReplicates}{By default stops processing if not all samples
have replicates}
}
\value{
None
}
\description{
Check whether all samples have replicates
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{example_data_only_values}
\alias{example_data_only_values}
\title{Same data as in "example_data", but omitting the annotation meaning that it
only contains the expression data.}
\format{
A data frame containing expression data
}
\usage{
example_data_only_values
}
\description{
Same data as in "example_data", but omitting the annotation meaning that it
only contains the expression data.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generatePlots.R
\name{plotRLE}
\alias{plotRLE}
\title{Boxplots showing relative log expression after normalizations}
\usage{
plotRLE(nr, currentLayout, pageno)
}
\arguments{
\item{nr}{Normalyzer results object.}

\item{currentLayout}{Layout used for document.}

\item{pageno}{Current page number.}
}
\value{
None
}
\description{
Boxplots showing relative log expression after normalizations
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{getRowNAFilterContrast}
\alias{getRowNAFilterContrast}
\title{Get contrast vector (TRUE/FALSE-values) indicating whether both at least
half values are present, and each sample has at least one non-NA value}
\usage{
getRowNAFilterContrast(dataMatrix, replicateHeader, minCount = 1)
}
\arguments{
\item{dataMatrix}{Matrix with expression values for entities in replicate 
samples.}

\item{replicateHeader}{Header showing how samples in matrix are replicated.}

\item{minCount}{Minimum number of required values present in samples.}
}
\value{
Contrast vector
}
\description{
Get contrast vector (TRUE/FALSE-values) indicating whether both at least
half values are present, and each sample has at least one non-NA value
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{getIndexList}
\alias{getIndexList}
\title{Return list containing vector positions of values in string}
\usage{
getIndexList(targetVector)
}
\arguments{
\item{targetVector}{}
}
\value{
indexList List where key is condition level and values are indices
  for the condition
}
\description{
Return list containing vector positions of values in string
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normMethods.R
\name{performQuantileNormalization}
\alias{performQuantileNormalization}
\title{Quantile normalization is performed by the function "normalize.quantiles"
from the package preprocessCore.}
\usage{
performQuantileNormalization(rawMatrix, noLogTransform = FALSE)
}
\arguments{
\item{rawMatrix}{Target matrix to be normalized}

\item{noLogTransform}{Assumes no need for log transformation}
}
\value{
Normalized matrix
}
\description{
It makes the assumption that the data in different samples should originate
from an identical distribution. It does this by generating a reference
distribution and then scaling the other samples accordingly.
}
\examples{
data(example_data_only_values_small)
normMatrix <- performQuantileNormalization(example_data_only_values)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NormalyzerDataset.R
\name{NormalyzerDataset}
\alias{NormalyzerDataset}
\title{Represents raw input data together with basic annotation information}
\usage{
NormalyzerDataset(
  jobName,
  designMatrix,
  rawData,
  annotationData,
  sampleNameCol,
  groupNameCol,
  tinyRunThres = 50,
  quiet = FALSE
)

NormalyzerDataset(
  jobName,
  designMatrix,
  rawData,
  annotationData,
  sampleNameCol,
  groupNameCol,
  tinyRunThres = 50,
  quiet = FALSE
)
}
\arguments{
\item{jobName}{Name of the NormalyzerDE processing run}

\item{designMatrix}{Matrix containing sample conditions}

\item{rawData}{Matrix containing raw input data}

\item{annotationData}{Matrix containing annotation information for each
input feature. Is expected to contain the same number of rows as the data
but can contain any number of features}

\item{sampleNameCol}{Name of column in design matrix containing sample
information}

\item{groupNameCol}{Name of column in design matrix containing condition
information}

\item{tinyRunThres}{If fewer features than this is present in the input
a limited run will be performed to avoid some steps requiring a more
extensive number of features.}

\item{quiet}{If set to TRUE no information messages will be printed}
}
\value{
nds Generated NormalyzerDataset instance
}
\description{
Takes a job name, a data matrix, a design matrix as well as specification
of the group and sample columns in the design matrix. Provides the
basic representation of a dataset in the NormalyzerDE normalization part.
}
\section{Slots}{

\describe{
\item{\code{jobName}}{Name of the job represented by the dataset.}

\item{\code{rawData}}{Matrix with raw values.}

\item{\code{sampleNameCol}}{Name column for sample.}

\item{\code{groupNameCol}}{Name column for groups.}

\item{\code{designMatrix}}{Data frame containing design.}

\item{\code{sampleNames}}{Vector containing sample names.}

\item{\code{filterrawdata}}{Reduced raw data matrix where low abundance rows are 
removed}

\item{\code{sampleReplicateGroups}}{Vector with sample replicate information}

\item{\code{samplesGroupsWithReplicates}}{Vector with replicated sample replicate information}

\item{\code{annotationValues}}{Annotation part of original dataframe.}

\item{\code{retentionTimes}}{Vector of retention time values.}

\item{\code{singleReplicateRun}}{Conditional whether run is single replicate.}
}}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{example_stat_summarized_experiment}
\alias{example_stat_summarized_experiment}
\title{SummarizedExperiment object prepared with design-matrix, data-matrix
and annotation columns for normalized data}
\format{
An instance of the class SummarizedExperiment with stats data
}
\usage{
example_stat_summarized_experiment
}
\description{
SummarizedExperiment object prepared with design-matrix, data-matrix
and annotation columns for normalized data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generatePlots.R
\name{plotFrontPage}
\alias{plotFrontPage}
\title{Generate first page in output report and write to viewport}
\usage{
plotFrontPage(currentjob, currentFont)
}
\arguments{
\item{currentjob}{Name of current run.}

\item{currentFont}{Font used for output document.}
}
\value{
None
}
\description{
Generate first page in output report and write to viewport
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NormalyzerEvaluationResults.R
\name{NormalyzerEvaluationResults}
\alias{NormalyzerEvaluationResults}
\title{Representation of evaluation results by calculating performance measures
for an an NormalyzerResults instance}
\usage{
NormalyzerEvaluationResults(nr, categoricalAnova = TRUE)

NormalyzerEvaluationResults(nr, categoricalAnova = TRUE)
}
\arguments{
\item{nr}{NormalyzerResults object}

\item{categoricalAnova}{Should ANOVA be categorical or not}
}
\value{
nds Generated NormalyzerEvaluationResults instance
}
\description{
Contains the resulting information from the processing which subsequently
can be used to generate the quality assessment report.
}
\section{Slots}{

\describe{
\item{\code{avgcvmem}}{Average coefficient of variance per method}

\item{\code{avgcvmempdiff}}{Percentage difference of mean coefficient of variance
compared to log2-transformed data}

\item{\code{featureCVPerMethod}}{CV calculated per feature and normalization method.}

\item{\code{avgmadmem}}{Average median absolute deviation}

\item{\code{avgmadmempdiff}}{Percentage difference of median absolute deviation
compared to log2-transformed data}

\item{\code{avgvarmem}}{Average variance per method}

\item{\code{avgvarmempdiff}}{Percentage difference of mean variance compared
to log2-transformed data}

\item{\code{lowVarFeaturesCVs}}{List of 5% least variable entries based on ANOVA
for log2-transformed data}

\item{\code{lowVarFeaturesCVsPercDiff}}{Coefficient of variance for least variable
entries}

\item{\code{anovaP}}{ANOVA calculated p-values}

\item{\code{repCorPear}}{Within group Pearson correlations}

\item{\code{repCorSpear}}{Within group Spearman correlations}
}}

\examples{
data(example_summarized_experiment)
normObj <- getVerifiedNormalyzerObject("job_name", example_summarized_experiment)
normResults <- normMethods(normObj)
normEval <- NormalyzerEvaluationResults(normResults)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateStatistics.R
\name{generateStatsReport}
\alias{generateStatsReport}
\title{Generate full output report plot document. Plots p-value histograms
for each contrast in the NormalyzerStatistics instance and writes these
to a PDF report.}
\usage{
generateStatsReport(
  nst,
  jobName,
  jobDir,
  sigThres = 0.1,
  sigThresType = "fdr",
  log2FoldThres = 0,
  plotRows = 3,
  plotCols = 4,
  writeAsPngs = FALSE
)
}
\arguments{
\item{nst}{NormalyzerDE statistics object.}

\item{jobName}{Name of processing run.}

\item{jobDir}{Path to output directory.}

\item{sigThres}{Significance threshold for indicating as significant}

\item{sigThresType}{Type of significance threshold (FDR or p)}

\item{log2FoldThres}{log2 fold-change required for being counted as significant}

\item{plotRows}{Number of plot rows.}

\item{plotCols}{Number of plot columns.}

\item{writeAsPngs}{Output the report as separate PNG files instead of a 
single PDF file}
}
\value{
None
}
\description{
Generate full output report plot document. Plots p-value histograms
for each contrast in the NormalyzerStatistics instance and writes these
to a PDF report.
}
\examples{
data(example_stat_summarized_experiment)
statObj <- NormalyzerStatistics(example_stat_summarized_experiment)
statObj <- calculateContrasts(statObj, comparisons=c("1-2", "2-3"), 
  condCol="group", type="limma")
outputDir <- tempdir()
generateStatsReport(statObj, "jobName", outputDir)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generatePlots.R
\name{plotBoxPlot}
\alias{plotBoxPlot}
\title{Boxplots showing distribution of values after different normalizations}
\usage{
plotBoxPlot(nr, currentLayout, pageno)
}
\arguments{
\item{nr}{Normalyzer results object.}

\item{currentLayout}{Layout used for document.}

\item{pageno}{Current page number.}
}
\value{
None
}
\description{
Boxplots showing distribution of values after different normalizations
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/higherOrderNormMethods.R
\name{getSmoothedRTNormalizedMatrix}
\alias{getSmoothedRTNormalizedMatrix}
\title{Generate multiple RT time-window normalized matrices where one is shifted.
Merge them using a specified method (mean or median) and return the result.}
\usage{
getSmoothedRTNormalizedMatrix(
  rawMatrix,
  retentionTimes,
  normMethod,
  stepSizeMinutes,
  windowShifts = 2,
  windowMinCount = 100,
  mergeMethod = "mean",
  noLogTransform = FALSE
)
}
\arguments{
\item{rawMatrix}{Target matrix to be normalized}

\item{retentionTimes}{Vector of retention times corresponding to rawMatrix}

\item{normMethod}{The normalization method to apply to the time windows}

\item{stepSizeMinutes}{Size of windows to be normalized}

\item{windowShifts}{Number of frame shifts.}

\item{windowMinCount}{Minimum number of features within window.}

\item{mergeMethod}{Layer merging approach. Mean or median.}

\item{noLogTransform}{Don't log transform the input}
}
\value{
Normalized matrix
}
\description{
Uses the function getRTNormalizedMatrix to generate multiple normalized
matrices which are shifted respective to each other and finally merged into
a single matrix. This could potentially reduce effect of fluctuations
within individual windows.
}
\examples{

data(example_data_small)
data(example_data_only_values)
data(example_design_small)
retentionTimes <- as.numeric(example_data[, "Average.RT"])
dataMat <- example_data_only_values
performCyclicLoessNormalization <- function(rawMatrix) {
    log2Matrix <- log2(rawMatrix)
    normMatrix <- limma::normalizeCyclicLoess(log2Matrix, method="fast")
    colnames(normMatrix) <- colnames(rawMatrix)
    normMatrix
}
rtNormMat <- getSmoothedRTNormalizedMatrix(dataMat, retentionTimes, 
    performCyclicLoessNormalization, stepSizeMinutes=1, windowMinCount=100, 
    windowShifts=2, mergeMethod="median")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyzeResults.R
\name{calculateFeatureCV}
\alias{calculateFeatureCV}
\title{Calculate CV values for each feature. Iterates through each normalization 
method and calculates a matrix of CV values where each column correspond to 
a method and each row corresponds to a feature.}
\usage{
calculateFeatureCV(methodList)
}
\arguments{
\item{methodList}{List containing normalized matrices.}
}
\value{
methodFeatureCVMatrix Matrix with feature as rows and normalization
  method as columns
}
\description{
Calculate CV values for each feature. Iterates through each normalization 
method and calculates a matrix of CV values where each column correspond to 
a method and each row corresponds to a feature.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputVerification.R
\name{setupRawContrastObject}
\alias{setupRawContrastObject}
\title{Prepare SummarizedExperiment object for statistics data}
\usage{
setupRawContrastObject(dataPath, designPath, sampleColName)
}
\arguments{
\item{dataPath}{Path to raw data matrix}

\item{designPath}{Path to design matrix}

\item{sampleColName}{Name for column in design matrix containing sample names}
}
\value{
experimentObj Prepared instance of SummarizedExperiment
}
\description{
Prepare SummarizedExperiment object for statistics data
}
\examples{
data_path <- system.file(package="NormalyzerDE", "extdata", "tiny_data.tsv")
design_path <- system.file(package="NormalyzerDE", "extdata", "tiny_design.tsv")
sumExpObj <- setupRawContrastObject(data_path, design_path, "sample")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputVerification.R
\name{getLowCountSampleFiltered}
\alias{getLowCountSampleFiltered}
\title{Verify that samples contain at least a lowest number of values}
\usage{
getLowCountSampleFiltered(
  dataMatrix,
  groups,
  threshold = 15,
  stopIfTooFew = TRUE
)
}
\arguments{
\item{dataMatrix}{Dataframe with processed input data.}

\item{groups}{Vector containing condition levels.}

\item{threshold}{Lowest number of allowed values in a column.}

\item{stopIfTooFew}{Abort run if lower than threshold number of values in
column}
}
\value{
None
}
\description{
Verify that samples contain at least a lowest number of values
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{calculatePercentageAvgDiffInMat}
\alias{calculatePercentageAvgDiffInMat}
\title{General function for calculating percentage difference of average column
means in matrix}
\usage{
calculatePercentageAvgDiffInMat(targetMat)
}
\arguments{
\item{targetMat}{Matrix for which column means should be compared}
}
\value{
percDiffVector Vector with percentage difference, where first element
  always will be 100
}
\description{
General function for calculating percentage difference of average column
means in matrix
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normMethods.R
\name{performCyclicLoessNormalization}
\alias{performCyclicLoessNormalization}
\title{Cyclic Loess normalization}
\usage{
performCyclicLoessNormalization(rawMatrix, noLogTransform = FALSE)
}
\arguments{
\item{rawMatrix}{Target matrix to be normalized}

\item{noLogTransform}{Assumes no need for log transformation}
}
\value{
Normalized matrix
}
\description{
Log2 transformed data is normalized by Loess method using the function
"normalizeCyclicLoess". Further information is available for the function
"normalizeCyclicLoess" in the Limma package.
}
\examples{
data(example_data_only_values_small)
normMatrix <- performCyclicLoessNormalization(example_data_only_values)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputVerification.R
\name{loadData}
\alias{loadData}
\title{Load raw data into dataframe}
\usage{
loadData(dataPath, inputFormat = "default")
}
\arguments{
\item{dataPath}{File path to design matrix.}

\item{inputFormat}{If input is given in standard NormalyzerDE format, 
Proteios format or in MaxQuant protein or peptide format}
}
\value{
rawData Raw data loaded into data frame
}
\description{
General function which allows specifying different types of input data
including "proteios", "maxquantpep" (peptide output from MaxQuant) and 
"maxquantprot" (protein output from MaxQuant) formats.
}
\examples{
\dontrun{
df <- loadData("data.tsv")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generatePlots.R
\name{plotReplicateVariance}
\alias{plotReplicateVariance}
\title{Generate normalization replicate variance summary by displaying
CV (coefficient of variance), MAD (mean of intragroup median absolute
deviation) and PEV (Pooled Estimate of Variance) as mean of intragroups}
\usage{
plotReplicateVariance(nr, currentLayout, pageno)
}
\arguments{
\item{nr}{Normalyzer results object.}

\item{currentLayout}{Layout used for document.}

\item{pageno}{Current page number.}
}
\value{
None
}
\description{
Generate normalization replicate variance summary by displaying
CV (coefficient of variance), MAD (mean of intragroup median absolute
deviation) and PEV (Pooled Estimate of Variance) as mean of intragroups
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputVerification.R
\name{verifyDesignMatrix}
\alias{verifyDesignMatrix}
\title{Verify that design matrix setup matches the data matrix}
\usage{
verifyDesignMatrix(fullMatrix, designMatrix, sampleCol)
}
\arguments{
\item{fullMatrix}{Dataframe with input data.}

\item{designMatrix}{Dataframe with design setup.}

\item{sampleCol}{Column in design matrix containing sample IDs.}
}
\value{
None
}
\description{
Verify that design matrix setup matches the data matrix
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/printMeta.R
\name{printMeta}
\alias{printMeta}
\title{Print meta information for Normalyzer plot page
! Needs refactoring to reduce redundancy in code
! Needs double check of functionality}
\usage{
printMeta(plotname, pageno, jobname, currentLayout)
}
\arguments{
\item{plotname}{Name of current plot.}

\item{pageno}{Current page number.}

\item{jobname}{Name of ongoing job.}

\item{currentLayout}{Custom viewport layout.}
}
\value{
None
}
\description{
Print meta information for Normalyzer plot page
! Needs refactoring to reduce redundancy in code
! Needs double check of functionality
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generatePlots.R
\name{plotQQ}
\alias{plotQQ}
\title{Produces page showing QQ-plots for the first sample for each normalization
method. This plot can be used to assess whether the data follows a normal
distribution.}
\usage{
plotQQ(nr, currentLayout, pageno)
}
\arguments{
\item{nr}{Normalyzer results object.}

\item{currentLayout}{Layout used for document.}

\item{pageno}{Current page number.}
}
\value{
None
}
\description{
Produces page showing QQ-plots for the first sample for each normalization
method. This plot can be used to assess whether the data follows a normal
distribution.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{setupJobDir}
\alias{setupJobDir}
\title{Create empty directory for run}
\usage{
setupJobDir(jobName, outputDir)
}
\arguments{
\item{jobName}{Name of the run.}

\item{outputDir}{Path to directory where to create the output directory.}
}
\value{
Path to newly created directory.
}
\description{
Creates a directory at provided path named to the jobname.
}
\examples{
setupJobDir("job_name", "path/to/outdir")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NormalyzerStatistics.R
\name{NormalyzerStatistics}
\alias{NormalyzerStatistics}
\title{Class representing a dataset for statistical processing in NormalyzerDE}
\usage{
NormalyzerStatistics(experimentObj, logTrans = FALSE)

NormalyzerStatistics(experimentObj, logTrans = FALSE)
}
\arguments{
\item{experimentObj}{Instance of SummarizedExperiment containing matrix
and design information as column data}

\item{logTrans}{Whether the input data should be log transformed}
}
\value{
nds Generated NormalyzerStatistics instance
}
\description{
Is initialized with an annotation matrix, a data matrix and a design
data frame. This object can subsequently be processed to generate statistical
values and in turn used to write a full matrix with additional statistical
information as well as a graphical report of the comparisons.
}
\section{Slots}{

\describe{
\item{\code{annotMat}}{Matrix containing annotation information}

\item{\code{dataMat}}{Matrix containing (normalized) expression data}

\item{\code{filteredDataMat}}{Filtered matrix with low-count rows removed}

\item{\code{designDf}}{Data frame containing design conditions}

\item{\code{filteringContrast}}{Vector showing which entries are filtered 
(due to low count)}

\item{\code{pairwiseCompsP}}{List with P-values for pairwise comparisons}

\item{\code{pairwiseCompsFdr}}{List with FDR-values for pairwise comparisons}

\item{\code{pairwiseCompsAve}}{List with average expression values}

\item{\code{pairwiseCompsFold}}{List with log2 fold-change values for pairwise 
comparisons}

\item{\code{contrasts}}{Spot for saving vector of last used contrasts}

\item{\code{condCol}}{Column containing last used conditions}

\item{\code{batchCol}}{Column containing last used batch conditions}
}}

\examples{
data(example_stat_summarized_experiment)
nst <- NormalyzerStatistics(example_stat_summarized_experiment)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyzeResults.R
\name{calculateSummarizedCorrelationVector}
\alias{calculateSummarizedCorrelationVector}
\title{Calculates correlation values between replicates for each condition matrix.
Finally returns a matrix containing the results for all dataset}
\usage{
calculateSummarizedCorrelationVector(
  methodlist,
  allReplicateGroups,
  sampleGroupsWithReplicates,
  corrType
)
}
\arguments{
\item{methodlist}{List containing normalized matrices for each normalization
method}

\item{allReplicateGroups}{Vector with condition groups matching the columns
found in the normalization methods}

\item{sampleGroupsWithReplicates}{Unique vector with condition groups
present in two or more samples}

\item{corrType}{Type of correlation (Pearson or Spearman)}
}
\value{
avgCorSum Matrix with column corresponding to normalization
approaches and rows corresponding to replicate group
}
\description{
Calculates correlation values between replicates for each condition matrix.
Finally returns a matrix containing the results for all dataset
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateStatistics.R
\name{plotContrastPCA}
\alias{plotContrastPCA}
\title{Show in a PCA plot what samples are compared in statistical contrast
This is useful to understand what conditions are compared and for checking
for outliers in the contrast}
\usage{
plotContrastPCA(nst, jobName, currentLayout, pageno, pcs = c(1, 2))
}
\arguments{
\item{nst}{NormalyzerDE statistics object.}

\item{jobName}{Name of processing run.}

\item{currentLayout}{Layout used for document.}

\item{pageno}{Current page number.}

\item{pcs}{Principal components to show.}
}
\value{
None
}
\description{
Show in a PCA plot what samples are compared in statistical contrast
This is useful to understand what conditions are compared and for checking
for outliers in the contrast
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generatePlots.R
\name{plotScatter}
\alias{plotScatter}
\title{Produces page containing scatter plot plotting the first two samples from 
each dataset against each other for each normalization method}
\usage{
plotScatter(nr, currentLayout, pageno)
}
\arguments{
\item{nr}{Normalyzer results object.}

\item{currentLayout}{Layout used for document.}

\item{pageno}{Current page number.}
}
\value{
None
}
\description{
Produces page containing scatter plot plotting the first two samples from 
each dataset against each other for each normalization method
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{example_stat_data}
\alias{example_stat_data}
\title{Same data as in "example_data", but normalized and ready for statistical
processing.}
\format{
A normalized data frame ready for statistical processing
}
\usage{
example_stat_data
}
\description{
Same data as in "example_data", but normalized and ready for statistical
processing.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputVerification.R
\name{preprocessData}
\alias{preprocessData}
\title{Replace empty values (0 or empty field) with NA in input data}
\usage{
preprocessData(dataMatrix, quiet = FALSE)
}
\arguments{
\item{dataMatrix}{Matrix with raw data.}

\item{quiet}{Don't show diagnostic messages}
}
\value{
Parsed rawdata where 0 values are replaced with NA
}
\description{
Replace empty values (0 or empty field) with NA in input data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputVerification.R
\name{verifyMultipleSamplesPresent}
\alias{verifyMultipleSamplesPresent}
\title{Check whether more than one sample is present}
\usage{
verifyMultipleSamplesPresent(
  dataMatrix,
  groups,
  requireReplicates = TRUE,
  quiet = FALSE
)
}
\arguments{
\item{dataMatrix}{Prepared dataframe.}

\item{groups}{Vector containing condition levels}

\item{requireReplicates}{By default stops processing if not all samples
have replicates}
}
\value{
None
}
\description{
Check whether more than one sample is present
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{filterLowRep}
\alias{filterLowRep}
\title{Filter rows with lower than given number of replicates for any condition}
\usage{
filterLowRep(df, groups, leastRep = 2)
}
\arguments{
\item{df}{Dataframe with expression data to filter}

\item{groups}{Condition groups header}

\item{leastRep}{Minimum number of replicates in each group
to retain}
}
\value{
collDesignDf Reduced design matrix
}
\description{
Filter rows with lower than given number of replicates for any condition
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{setupTestData}
\alias{setupTestData}
\title{Generate a random test dataset with features, sample values and retention times}
\usage{
setupTestData(nSamples, nFeatures, rtMin = 40, rtMax = 80, mean = 20, sd = 4)
}
\arguments{
\item{nSamples}{Number of samples}

\item{nFeatures}{Number of features}

\item{rtMin}{Minimum retention time}

\item{rtMax}{Maximum retention time}

\item{mean}{Mean value for sample intensities}

\item{sd}{Standard deviation for sample intensities}
}
\value{
Test dataset
}
\description{
Generate a random test dataset with features, sample values and retention times
}
\examples{
df <- setupTestData(6, 20)
df <- setupTestData(6, 20, mean=15, sd=1)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateStatistics.R
\name{generateAnnotatedMatrix}
\alias{generateAnnotatedMatrix}
\title{Generate an annotated data frame from statistics object}
\usage{
generateAnnotatedMatrix(nst, prefixSep = "_", compLabels = NULL)
}
\arguments{
\item{nst}{NormalyzerDE statistics object.}

\item{prefixSep}{Character string for separating the prefix names from the statistics suffix}

\item{compLabels}{Vector containing strings to use as prefix for statistical comparisons}
}
\value{
outDf Annotated statistics matrix
}
\description{
Extracts key values (p-value, adjusted p-value, log2-fold change and
average expression values) from an NormalyzerStatistics instance
and appends these to the annotation- and data-matrices
}
\examples{
data(example_stat_summarized_experiment)
statObj <- NormalyzerStatistics(example_stat_summarized_experiment)
statObj <- calculateContrasts(statObj, comparisons=c("1-2", "2-3"), condCol="group", type="limma")
annotDf <- generateAnnotatedMatrix(statObj)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{example_summarized_experiment}
\alias{example_summarized_experiment}
\title{SummarizedExperiment object prepared with design-matrix, data-matrix
and annotation columns loaded for raw data}
\format{
An instance of the class SummarizedExperiment
}
\usage{
example_summarized_experiment
}
\description{
SummarizedExperiment object prepared with design-matrix, data-matrix
and annotation columns loaded for raw data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NormalyzerStatistics.R
\name{verifyContrasts}
\alias{verifyContrasts}
\title{Check that a given contrast string is valid given a particular design
matrix. Each level tested for in the contrast should be present in the
condition column for the design matrix.}
\usage{
verifyContrasts(designLevels, contrasts)
}
\arguments{
\item{designLevels}{Vector containing condition levels present in design}

\item{contrasts}{A string containing one or several (comma delimited)
strings for which contrasts should be performed}
}
\value{
None
}
\description{
Mainly meant to verify strings received during server usage.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NormalyzerDE.R
\name{normalyzer}
\alias{normalyzer}
\title{NormalyzerDE pipeline entry point}
\usage{
normalyzer(
  jobName,
  designPath = NULL,
  dataPath = NULL,
  experimentObj = NULL,
  outputDir = ".",
  forceAllMethods = FALSE,
  omitLowAbundSamples = FALSE,
  sampleAbundThres = 5,
  tinyRunThres = 50,
  requireReplicates = TRUE,
  normalizeRetentionTime = TRUE,
  plotRows = 3,
  plotCols = 4,
  zeroToNA = FALSE,
  sampleColName = "sample",
  groupColName = "group",
  inputFormat = "default",
  skipAnalysis = FALSE,
  quiet = FALSE,
  noLogTransform = FALSE,
  writeReportAsPngs = FALSE,
  rtStepSizeMinutes = 1,
  rtWindowMinCount = 100,
  rtWindowShifts = 1,
  rtWindowMergeMethod = "mean"
)
}
\arguments{
\item{jobName}{Give the current run a name.}

\item{designPath}{Path to file containing design matrix.}

\item{dataPath}{Specify an output directory for generated files.
Defaults to current working directory.}

\item{experimentObj}{SummarizedExperiment object, can be provided as input
as alternative to 'designPath' and 'dataPath'}

\item{outputDir}{Directory where results folder is created.}

\item{forceAllMethods}{Debugging function. Run all normalizations even if
they aren't in the recommended range of number of values}

\item{omitLowAbundSamples}{Automatically remove samples with fewer non-NA
values compared to threshold given by sampleAbundThres. 
Will otherwise stop with error message if such sample is encountered.}

\item{sampleAbundThres}{Threshold for omitting low-abundant
samples. Is by default set to 15.}

\item{tinyRunThres}{If total number of features is less than this, a limited
run is performed.}

\item{requireReplicates}{Require multiple samples per condition to pass input 
validation.}

\item{normalizeRetentionTime}{Perform normalizations over retention time.}

\item{plotRows}{Number of plot-rows in output documentation.}

\item{plotCols}{Number of plot-columns in output documentation.}

\item{zeroToNA}{Convert zero values to NA.}

\item{sampleColName}{Column name in design matrix containing sample IDs.}

\item{groupColName}{Column name in design matrix containing condition IDs.}

\item{inputFormat}{Type of input format.}

\item{skipAnalysis}{Only perform normalization steps.}

\item{quiet}{Omit status messages printed during run.}

\item{noLogTransform}{Don't log-transform the input.}

\item{writeReportAsPngs}{Output the evaluation report as PNG files instead of
a single PDF}

\item{rtStepSizeMinutes}{Retention time normalization window size.}

\item{rtWindowMinCount}{Minimum number of datapoints in each retention-time
segment.}

\item{rtWindowShifts}{Number of layered retention time normalized windows.}

\item{rtWindowMergeMethod}{Merge approach for layered retention time windows.}
}
\value{
None
}
\description{
This function is the main execution point for the normalization part of
the NormalyzerDE analysis pipeline. When executed it performs the following
steps:
}
\details{
1: Loads the data matrix containing expression values and optional
annotations, as well as the design matrix containing the experimental setup
2: Performs input data verification to validate that the data is in correct
format. This step captures many common formatting errors. It returns an
instance of the NormalyzerDataset class representing the unprocessed data.
3: Calculate a range of normalizations for the dataset. The result is
provided as a NormalyzerResults object containing the resulting data matrices
from each normalization.
4: Analyze the normalizations and generate performance measures for each
of the normalized datasets. This result is provided as a 
NormalyzerEvaluationResults object.
5: Output the matrices containing the normalized datasets to files.
6: Generate visualizations overviewing the performance measures and
write them to a PDF report.
}
\examples{
\dontrun{
data_path <- system.file(package="NormalyzerDE", "extdata", "tiny_data.tsv")
design_path <- system.file(package="NormalyzerDE", "extdata", "tiny_design.tsv")
out_dir <- tempdir()
normalyzer(
    jobName="my_jobname", 
    designPath=design_path, 
    dataPath=data_path, 
    outputDir=out_dir)
normalyzer(
    "my_jobname", 
    designMatrix="design.tsv", 
    "data.tsv", 
    outputDir="path/to/output", 
    normalizeRetentionTime=TRUE, 
    retentionTimeWindow=2)
normalyzer(
    "my_jobname", 
    designMatrix="design.tsv", 
    "data.tsv", 
    outputDir="path/to/output", 
    inputFormat="maxquantprot")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generatePlots.R
\name{plotReplicateVarAndStableVariables}
\alias{plotReplicateVarAndStableVariables}
\title{Write figures displaying pooled coefficient of variance, median absolute 
deviation and pooled estimate of variance percentage compared to 
log2-transformed and stable variables plot displaying CV of stable variables
against pooled CV measure. The stable variables are calculated by an 
ANOVA comparison across sample conditions and selecting features with the 
least clear difference.}
\usage{
plotReplicateVarAndStableVariables(nr, currentLayout, pageno)
}
\arguments{
\item{nr}{Normalyzer results object.}

\item{currentLayout}{Layout used for document.}

\item{pageno}{Current page number.}
}
\value{
None
}
\description{
Write figures displaying pooled coefficient of variance, median absolute 
deviation and pooled estimate of variance percentage compared to 
log2-transformed and stable variables plot displaying CV of stable variables
against pooled CV measure. The stable variables are calculated by an 
ANOVA comparison across sample conditions and selecting features with the 
least clear difference.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generatePlots.R
\name{plotDensity}
\alias{plotDensity}
\title{Density plots showing value distributions after normalizations}
\usage{
plotDensity(nr, currentLayout, pageno)
}
\arguments{
\item{nr}{Normalyzer results object.}

\item{currentLayout}{Layout used for document.}

\item{pageno}{Current page number.}
}
\value{
None
}
\description{
Density plots showing value distributions after normalizations
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generatePlots.R
\name{plotDendrograms}
\alias{plotDendrograms}
\title{Visualize dendrogram grouping of samples}
\usage{
plotDendrograms(nr, currentLayout, pageno)
}
\arguments{
\item{nr}{Normalyzer results object.}

\item{currentLayout}{Layout used for document.}

\item{pageno}{Current page number.}
}
\value{
None
}
\description{
Visualize dendrogram grouping of samples
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normMethods.R
\name{meanNormalization}
\alias{meanNormalization}
\title{Intensity of each variable in a given sample is divided by the mean of sum of
intensities of all variables in the sample and then multiplied by the mean
of sum of intensities of all variables in all samples. The normalized data
is then transformed to log2.}
\usage{
meanNormalization(rawMatrix, noLogTransform = FALSE)
}
\arguments{
\item{rawMatrix}{Target matrix to be normalized}

\item{noLogTransform}{Assumes no need for log transformation}
}
\value{
Normalized and log-transformed matrix
}
\description{
Intensity of each variable in a given sample is divided by the mean of sum of
intensities of all variables in the sample and then multiplied by the mean
of sum of intensities of all variables in all samples. The normalized data
is then transformed to log2.
}
\examples{
data(example_data_only_values_small)
normMatrix <- meanNormalization(example_data_only_values)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputVerification.R
\name{verifyValidNumbers}
\alias{verifyValidNumbers}
\title{Verify that input fields conform to the expected formats}
\usage{
verifyValidNumbers(rawDataOnly, groups, noLogTransform = FALSE, quiet = FALSE)
}
\arguments{
\item{rawDataOnly}{Dataframe with input data.}

\item{groups}{Condition levels for comparisons.}
}
\value{
None
}
\description{
Verify that input fields conform to the expected formats
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputVerification.R
\name{verifySummarizedExperiment}
\alias{verifySummarizedExperiment}
\title{Verify that SummarizedExperiment setup matches the data matrix}
\usage{
verifySummarizedExperiment(summarizedExp, sampleCol)
}
\arguments{
\item{summarizedExp}{SummarizedExperiment object.}

\item{sampleCol}{Column in the SummarizedExperiment colData containing sample IDs.}
}
\value{
None
}
\description{
Verify that SummarizedExperiment setup matches the data matrix
}
\keyword{internal}
